;@***h* MLT_V4_RSS/append_data_to_noaa_nav_files.pro
; NAME
;	append_data_to_noaa_nav_files.pro
;
; PURPOSE
;	Combines navigation files together for MSU satellites.
;
; DESCRIPTION
;	Combines navigation files together for MSU satellites.  Here, the crossing time data from RSS files, which are good for the first part of the mission
;     of various satellites are combined with navigation data deduced directly from the MSU L1B files.
;
; INPUTS
;	sat_num should be set in first line executed to the desired satellite.
;
; OUTPUTS
;	writes and MSU "extended" navigation file.  These are the files used for subsequent processing.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	.r append_data_to_noaa_nav_files;
;@*****


sat_num = 4
MSU_ORBIT_TIME = 0.070799d0

; read in the background crossing time file.  This contains the data from the noaa eqx files, as well as the 
; old RSS inventory files.  RSS orbit numbers are adjusted to match NOAA orbit numbers when possible.

read_MSU_nav_file,sat_num,bkgrnd_eqx_time,bkgrnd_eqx_lon,bkgrnd_a,bkgrnd_orbit_valid,error,type = 'noaa_fixed'

if sat_num eq 0 then bkgrnd_a[6150L:99999L] = !values.f_nan  ; remove bad values toward the end of TIROS-N
ok = where(finite(bkgrnd_a) eq 1,ct)
max_bkgrnd_orbit = max(ok)
if sat_num eq 6 then bkgrnd_a[35000L:45394L] = !values.f_nan  ; remove bad values in gap in NOAA-11

; read in the deduced crossing time data obtained from L1B files from NOAA
read_msu_raw_crossing_file,sat_num,L1B_orbit_num,_L1B_orbit_real,L1B_eqx_time,L1B_a,L1B_eqx_lon

if sat_num eq 3 then begin
   L1B_a[4216:4397] = L1B_a[4216:4397] - MSU_ORBIT_TIME
endif

if sat_num eq 0 then begin ; remove a few bad values for TIROS-N
  L1B_a[1123] = !values.f_nan
  L1B_a[1136] = !values.f_nan
endif

if sat_num eq 6 then begin
  j = findgen(120000L)
  ok = where((j  gt 45000L) and (L1B_a lt -4113.8),ct)
  if ct gt 0 then begin
     L1B_a[ok] = L1B_a[ok] + MSU_ORBIT_TIME
  endif
  ok = where((j  gt 45000L) and (L1B_a lt -4113.8),ct)
  if ct gt 0 then begin
    L1B_a[ok] = L1B_a[ok] + MSU_ORBIT_TIME
  endif
  
  bkgrnd_a[33765L:35100L] = bkgrnd_a[33765L:35100L] + bkgrnd_a[33764L] - bkgrnd_a[33765L]
endif

L1B_ok = where(finite(L1B_a) eq 1,ct)
max_L1B_orbit = max(L1B_ok)

max_orbit = max([max_bkgrnd_orbit,max_L1B_orbit])

print,max_bkgrnd_orbit,max_L1B_orbit,max_orbit

; combine the two datasets together

combined_a = replicate(!values.d_nan,max_orbit+1)
orbit_num = lindgen(max_orbit+1)
combined_a[L1B_ok] = L1B_a[L1B_ok]
bkgrnd_useful = where((finite(bkgrnd_a) eq 1) and (finite(combined_a) eq 0),ct) 
if ct gt 0 then combined_a[bkgrnd_useful] = bkgrnd_a[bkgrnd_useful]

if sat_num eq 7 then begin
  combined_a[21440L:n_elements(combined_a)-1] = combined_a[21440L:n_elements(combined_a)-1] - MSU_ORBIT_TIME 
  combined_a[39450L:n_elements(combined_a)-1] = combined_a[39450l:n_elements(combined_a)-1] + MSU_ORBIT_TIME 
  combined_a[50921L:n_elements(combined_a)-1] = combined_a[50921L:n_elements(combined_a)-1] + 0.043
endif

;if sat_num eq 6 then begin ;NOAA-11
;  combined_a[45635L:n_elements(combined_a)-1] = combined_a[45635L:n_elements(combined_a)-1] + MSU_ORBIT_TIME
;  combined_a[50179L:n_elements(combined_a)-1] = combined_a[50179L:n_elements(combined_a)-1] + MSU_ORBIT_TIME
 ; combined_a[53423L:n_elements(combined_a)-1] = combined_a[53423L:n_elements(combined_a)-1] - MSU_ORBIT_TIME
;endif
; this routine replaces values for a by a local linear fit.  linear fit parameters are deduced from the median
; of a number of smaller fits.

a_reconstructed = reconstruct_ts_using_local_fits(combined_a,fit_length_in=900,fit_function = 'median_linfit')

if sat_num eq 6 then begin
  x = findgen(1000)
  fit = linfit(x[600:799],a_reconstructed[600:799])
  a_reconstructed[0:599] = fit[0] + fit[1]*x[0:599]
endif

eqx_time_reconstructed = a_reconstructed+orbit_num*MSU_ORBIT_TIME
ok = where(finite(eqx_time_reconstructed) eq 1,ct)
orbit_valid_reconstructed = intarr(n_elements(eqx_time_reconstructed))
if ct gt 0 then orbit_valid_reconstructed[ok] = 1

; compute median longitude change per orbit

med_lon_diff = median((720 + L1B_eqx_lon[1:max_L1B_orbit-1] - L1B_eqx_lon[0:max_L1B_orbit-2]) mod 360.0)-360.0
print,'Median Longitude Difference = ',med_lon_diff
x = orbit_num*(med_lon_diff)

lon_offset_L1B = -180 + (6*360.0 + 180.0+L1B_eqx_lon-(360.0D0 + (x mod 360D0))) mod 360.0D0
lon_offset_bkgrnd = -180 + (6*360.0 +180 + bkgrnd_eqx_lon-(360.0D0 + (x mod 360D0))) mod 360.0D0

combined_lon_offset = replicate(!values.d_nan,max_orbit+1)
L1B_ok = where(finite(lon_offset_L1B) eq 1,ct)
combined_lon_offset[L1B_ok] = lon_offset_L1B[L1B_ok]
bkgrnd_useful = where((finite(lon_offset_bkgrnd) eq 1) and (finite(combined_lon_offset) eq 0),ct) 
if ct gt 0 then combined_lon_offset[bkgrnd_useful] = lon_offset_bkgrnd[bkgrnd_useful]


lon_offset_reconstructed = reconstruct_ts_using_local_fits(combined_lon_offset,fit_length_in=900,fit_function = 'median_linfit')
eqx_lon_reconstructed    = 360.0D0 + ((lon_offset_reconstructed + orbit_num*med_lon_diff) mod 360.0D0)

window,1
plot,bkgrnd_a
oplot,l1b_a,color=40
oplot,a_reconstructed,color=200
oplot,l1b_a,color=40
oplot,bkgrnd_a

window,2
plot,bkgrnd_a,xrange=[40000L,60000L]
oplot,l1b_a,color=40
oplot,a_reconstructed,color=200
oplot,l1b_a,color=40
oplot,bkgrnd_a

window,3
plot,bkgrnd_a,xrange=[30000L,40000L]
oplot,l1b_a,color=40
oplot,a_reconstructed,color=200
oplot,l1b_a,color=40
oplot,bkgrnd_a

write_MSU_nav_file,sat_num,eqx_time_reconstructed,eqx_lon_reconstructed,a_reconstructed,orbit_valid_reconstructed,error,type = 'extended'

end