;@***h* MLT_V4_RSS/read_MSU_nav_file.pro
; NAME
;	read_MSU_nav_file.pro
;
; PURPOSE
;	reads in an MSU naviagation file. 
;
; DESCRIPTION
;	Reads in an MSU naviagation file for a given MSU satellite and file type.  The navigation file
;     contains the equator crossing time (eqx time) and the equaltor crossing longitude (eqx_lon).  There are 
;     three "types" of these files that are used in various stages of removing errors in the satellite navigation,
;	rss, noaa_raw, and noaa_fixed.
;
; INPUTS
;	sat_num	integer		satellite number (1..9)
;	type		string		file "type", ('rss','noaa_raw','noaa_fixed')
;
; OUTPUTS
;	eqx_time	double,dimension(number of orbits)		equator crossing time, ascending
;	eqx_lon	double,dimension(number of orbits)		equator crossing longitude, ascending
;	a		double,dimension(number of orbits)		difference between eqx_time and orbit_number*orbit period.
;	orbit_valid integer,dimension(number of orbits)	valid robit flag
;	error 	integer						set to 0 for valid read
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	read_MSU_nav_file,sat_num,eqx_time,eqx_lon,a,orbit_valid,error,type = type
;
;@*****

pro read_MSU_nav_file,sat_num,eqx_time,eqx_lon,a,orbit_valid,error,type = type

xtime_path = 'M:\MSU_AMSU_Data\MSU_crossing_times\'
sat_names = ['TIROS-N','NOAA-6','NOAA-7','NOAA-8','NOAA-9','NOAA-10','NOAA-11','NOAA-12','NOAA-14']
sat_name_short = ['tn','n6','n7','n8','n9','n10','n11','n12','n14']


if keyword_set(type) eq 0 then type = 'rss'

case type of 
  'rss': xtime_file = xtime_path+sat_names[sat_num]+'\'+sat_name_short[sat_num]+'ephem.rss.txt
  'noaa_raw' : xtime_file = xtime_path+sat_names[sat_num]+'\'+sat_name_short[sat_num]+'ephem.txt
  'noaa_fixed': xtime_file = xtime_path+sat_names[sat_num]+'\'+sat_name_short[sat_num]+'ephem.noaa.txt
else: begin
  print,'Invalid type ',type
  error = -1
  return
  end
endcase

error = 0
    
    
mx_orbits =  100000L

eqx_time    = replicate(!values.d_nan,mx_orbits)
eqx_lon     = replicate(!values.d_nan,mx_orbits)
a           = replicate(!values.d_nan,mx_orbits)
orbit_valid = intarr(mx_orbits)

close,3
openr,3,xtime_file
    
on_ioerror,file_end

for i = 0,MX_ORBITS-1 do begin
  satx = '  '
  orbitx = 0L
  yearx  = 0
  dayx = 0
  hourx = 0
  minutex = 0
  secondx = 0.0
  xlon_inx = 0L
  
  readf,3,satx,orbitx,yearx,monthx,dayx,hourx,minutex,secondx,xlon_inx, $
  format = '(a2,2x,i5,11x,i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f6.3,1x,i6)'
  
  if ((orbitx ge 0) and (orbitx le MX_ORBITS-1)) then begin
      jday =  julday(monthx,dayx,yearx,hourx,minutex,secondx)
      ok = convert_date(date_in = jday,from ='julday',to='rd_2000',date_out = rd_2000)
      eqx_time[orbitx] = rd_2000
      eqx_lon[orbitx] = (720.0D0 + xlon_inx/100.0D0) mod 360.0D0
      if xlon_inx eq -99999 then eqx_lon[orbitx]=!values.d_nan
      orbit_valid[orbitx] = 1
   endif
endfor


file_end:

MSU_ORBIT_TIME = 0.070799d0

for i = 0,MX_ORBITS-1 do begin
  if orbit_valid[i] gt 0 then begin
       A[i] =  eqx_time[i] - i*MSU_ORBIT_TIME
  endif
endfor
print,'Read ',i,' Lines from NOAA NAvigation File '+xtime_file

end 
   