;@***h* MLT_V4_RSS/reconstruct_ts_using_local_fits.pro
; NAME
;	reconstruct_ts_using_local_fits.pro
;
; PURPOSE
;	Fixes errors in MSU navigation files.
;
; DESCRIPTION
;	Fixes errors in MSU navigation files.  Reconstructs a time series so that outliers are replaces by values from median values from
; 	local linear fits.  
;
; INPUTS
;	ts			real,dimension(num)		input time series with bad data
;	fit_length_in	integer				length of time series to be evaluated
;	fit_function	string				name of function used to do local fits.
;	show_plot		integer				set to non-zero to plot analysis
;	  
;
; OUTPUTS
;	return value: real,dimension(num)		reconstructed time series	
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	ts_reconstructed = reconstruct_ts_using_local_fits(ts,fit_length_in=200,fit_function = 'median_linfit',show_plot=1)
;
;@*****

function reconstruct_ts_using_local_fits,ts,fit_length_in=fit_length_in,fit_function = fit_function,show_plot=show_plot

if keyword_set(fit_function) eq 0 then fit_function = 'median_linfit'
if keyword_set(fit_length_in) eq 0 then fit_length_in = 300

step_length = floor(fit_length_in/3.0)
fit_length = step_length*3

num = n_elements(ts)

num_sets = floor(num/step_length)
num_arr  = dindgen(num+1)

ts_reconstruct_tot = dblarr(num)
ts_weight = dblarr(num)

; first, do the first set, and reconstruct from zero to fit_length

num_subset = num_arr[0:(2*step_length)-1]
ts_subset = ts[0:(2*step_length)-1]
ok = where(finite(ts_subset) eq 1,ct)

if ct gt 10 then begin  ; this 10 is kind of arbitrary
  fit = call_function(fit_function,num_subset,ts_subset)
  fitted_ts = fit[0]+fit[1]*num_subset
  
  local_wt = dblarr(2*step_length)
  local_wt[0:step_length-1] = 1.0D0
  local_wt[step_length:2*step_length-1] = ((step_length-1) - dindgen(step_length))/step_length
  
  ts_reconstruct_tot[0:(2*step_length)-1] = ts_reconstruct_tot[0:(2*step_length)-1] + fitted_ts*local_wt
  ts_weight[0:(2*step_length)-1] = ts_weight[0:(2*step_length)-1] + local_wt
endif

for set = 0,num_sets-3 do begin
  
  set_start = set*step_length
  set_end = set*step_length + fit_length-1
  
  
  
  num_subset = num_arr[set_start:set_end]
  ts_subset =  ts[set_start:set_end]
  ok = where(finite(ts_subset) eq 1,ct)
  if ct gt 10 then begin
    fit = median_linfit(num_subset,ts_subset)
    if (finite(fit[0]) eq 1 and finite(fit[1] eq 1))then begin
      fitted_ts = fit[0] + fit[1]*num_subset
      
      local_wt = dblarr(fit_length)
      local_wt[0:step_length-1] = dindgen(step_length)/step_length
      local_wt[step_length:2*step_length-1] = 1.0
      local_wt[step_length*2:3*step_length-1] = ((step_length-1) - dindgen(step_length))/step_length
  
      ts_reconstruct_tot[set_start:set_end] = ts_reconstruct_tot[set_start:set_end] + fitted_ts*local_wt
      ts_weight[set_start:set_end] = ts_weight[set_start:set_end] + local_wt
    
      if (keyword_set(show_plot) eq 1) then begin
        window,0
        plot,num_subset,ts_subset
        oplot,num_subset,fitted_ts
        window,1
        plot,num_subset,ts_subset-fitted_ts
      endif
    endif
  endif
endfor

print,set_start,set_end,num

set_end = num-1
set_start = num-300

print,set_start,set_end,num

num_subset = num_arr[set_start:set_end]
ts_subset =  ts[set_start:set_end]
ok = where(finite(ts_subset) eq 1,ct)
if ct gt 10 then begin
  fit = median_linfit(num_subset,ts_subset)
  fitted_ts = fit[0] + fit[1]*num_subset
  
  local_wt = dblarr(fit_length)
  local_wt[0:step_length-1] = dindgen(step_length)/step_length
  local_wt[step_length:2*step_length-1] = 1.0
  local_wt[step_length*2:3*step_length-1] = 1.0

  ts_reconstruct_tot[set_start:set_end] = ts_reconstruct_tot[set_start:set_end] + fitted_ts*local_wt
  ts_weight[set_start:set_end] = ts_weight[set_start:set_end] + local_wt
  
  if keyword_set(show_plot) eq 1 then begin
    window,2
    plot,ts_weight[num-300:num-1],xrange=[0,299],xstyle=1
  endif
endif

return,ts_reconstruct_tot/ts_weight
  
end