;@***h* MLT_V4_RSS/write_MSU_nav_file.pro
; NAME
;	write_MSU_nav_file.pro
;
; PURPOSE
;	writes an MSU navigation file.
;
; DESCRIPTION
;	writes an MSU navigation file.  satellite number, file "type" are specified in the call.
;
; INPUTS
;	sat_num	integer		satellite number (1..9)
;	eqx_time	double,dimension(number of orbits)		equator crossing time, ascending
;	eqx_lon	double,dimension(number of orbits)		equator crossing longitude, ascending
;	a		double,dimension(number of orbits)		difference between eqx_time and orbit_number*orbit period.
;	orbit_valid integer,dimension(number of orbits)	valid robit flag
;	type		string		file "type", ('rss','noaa_raw','noaa_fixed')
;
; OUTPUTS
;	error		integer		Set to 0 if no error occurs, set to -1 if error occurs.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	write_MSU_nav_file,sat_num,eqx_time,eqx_lon,a,orbit_valid,error,type = type
;
;@*****

pro write_MSU_nav_file,sat_num,eqx_time,eqx_lon,a,orbit_valid,error,type = type


error = 0
    
xtime_path = 'M:\MSU_AMSU_Data\MSU_crossing_times\'
sat_names = ['TIROS-N','NOAA-6','NOAA-7','NOAA-8','NOAA-9','NOAA-10','NOAA-11','NOAA-12','NOAA-14']
sat_name_short = ['tn','n6','n7','n8','n9','n10','n11','n12','n14']
sat_name_short2 = ['TN ','N06','N07','N08','N09','N10','N11','N12','N14']

if keyword_set(type) eq 0 then type = 'rss'
case type of 
  'rss': xtime_file = xtime_path+sat_names[sat_num]+'\'+sat_name_short[sat_num]+'ephem.rss.txt
  'noaa_raw' : xtime_file = xtime_path+sat_names[sat_num]+'\'+sat_name_short[sat_num]+'ephem.txt
  'noaa_fixed': xtime_file = xtime_path+sat_names[sat_num]+'\'+sat_name_short[sat_num]+'ephem.noaa.txt
  'extended': xtime_file = xtime_path+sat_names[sat_num]+'\'+sat_name_short[sat_num]+'ephem.extended.txt
else: begin
  print,'Invalid type ',type
  error = -1
  return
  end
endcase

close,3
openw,3,xtime_file
    
min_orbit = min(where(finite(eqx_time) eq 1))
max_orbit = max(where(finite(eqx_time) eq 1))

for i = min_orbit,max_orbit do begin
  if finite(eqx_time[i]) eq 1 then begin
    ok = convert_date(date_in = eqx_time[i],from ='rd_2000',to='julday',date_out = jday)
    caldat,jday,Month,Day,Year,Hour,Minute,Second
  endif else begin
    year = 9999
    month = 99
    day = 99
    hour = 99
    minute = 99
    second = 99.9
  endelse
  if (finite(eqx_lon[i]) eq 1) then begin
    lon_out = ((round(eqx_lon[i]*100) + 18000L) mod 36000L)-18000L
  endif else begin
    lon_out = -99999L
  endelse
  
  if (finite(a[i]) eq 1) then begin
    a_out = a[i]
  endif else begin
    a_out = -99999.99
  endelse
    
  printf,3,sat_name_short2[sat_num],i,year,month,day,hour,minute,second,lon_out,a_out,orbit_valid[i], $
  format = '(a3,"|",i5.5,"|ephemeris|",i4.4,"-",i2.2,"-",i2.2,1x,i2.2,":",i2.2,":",f6.3,"|",i6,"|",f18.11,"|",i1)'
  
endfor

close,3


end 
   