$freeform
	! reads incoming MSU L1B files and cuts them up into orbits



	module MSU_L1B_Ingest

		use msu_level_1a
		use MSU_Level_1B
		use nan_support
		use date_and_time
		use msu_types
		use msu_constants
		use msu_date
		use interpolate
		use get_file_list
		use numerical_recipes_sorting
		use MSU_L1B_Quality
		use MSU_navigation

		implicit none

		integer(4),parameter  :: MAX_FILES = 100000
		
	contains

;@***h* MLT_V4_RSS/ingest_MSU_l1b
; NAME
;	ingest_MSU_l1b
;
; PURPOSE
;	ingests MSU L1B data from CLASS
;
; DESCRIPTION
;	ingests MSU L1B data from CLASS.  Data is quality controlled and checked for duplicate scans.  All data is written to a
; 	monthly L1B file
;
; INPUTS
;	msu_incoming_path				character(len = FILE_LEN)		path containing CLASS L1B files		
;	sat_num					integer(4)					satellite number (1 = TIROS-N, 2 - NOAA-06, etc)
;	year						integer(4)					year to do
;	month						integer(4)					month to do
;	wrt						logical					flag, set true to write output files
;	eval_crossing_only			logical					flag, set true just write out all crossing times

;
; OUTPUTS
;	none passed back from subroutine
;	subroutine writes monthly L1B file.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call ingest_MSU_l1b(msu_incoming_path,sat_num,year,month,wrt,eval_crossing_only)

;
;@*****
	subroutine ingest_MSU_l1b(msu_incoming_path,sat_num,year,month,wrt,eval_crossing_only)

        	USE IFPORT

        	integer(4),dimension(12)                            ::  statarray
        	integer(4)                                          ::  istat
        	integer(4)                                          ::  size_of_L1B_file
	  	character(len = FILE_LEN)				    ::  msu_incoming_path	 ! incoming path name
	  	integer(4)                                          ::  year, month
	  	logical							    ::  wrt                  ! set true to write out data
	  	logical							    ::  eval_crossing_only   ! set true just write out all crossing times
	  	character(len = FILE_LEN),dimension(MAX_FILES)      ::  raw_l1b_file         ! files that are in this month
	  	character(len = FILE_LEN),dimension(2000)           ::  l1b_files_to_process ! files that are in this month
	  	character(len = FILE_LEN)				    ::  file		     ! incoming file name

		type(msu_raw)						    :: msu_l1b_scan	         ! structure containing the scan data
		type(msu_raw)                                       :: test_scan
		type(msu_raw),dimension(MAX_SCANS_PER_MONTH)        :: msu_l1b_scan_array
		integer(4),dimension(MAX_SCANS_PER_MONTH)           :: msu_l1b_scan_valid
		
		! nvigation
		
		real(8),dimension(MX_ORBITS)        :: nav_eqx_time
		real(8),dimension(MX_ORBITS)        :: nav_eqx_longitude
		integer(4),dimension(MX_ORBITS)     :: nav_orbit_valid
	    	real(8),dimension(MX_ORBITS)        :: nav_a              ! orbit time offset (eqx_time = a + orbit_num*orbit_time)
		integer(4)                          :: nav_max_orbit
		integer(4)                          :: nav_min_orbit
		integer(4)                          :: nav_satnum_loaded = 0 

		! errors

		integer(4)					:: read_Error
		integer(4)				      :: err

		integer(4)					:: sat_num
		integer(4)					:: orbit_index		
		integer(4)					:: orbit_index2
		integer(4)                      	:: scan_jump
									 
		integer(4)					:: num_scan
		integer(4)                      	:: scan_index
		integer(4)					:: num_scan_this_file

		type(date_str)				:: start_date
		type(date_str)				:: end_date
		type(date_str)	            	:: start_date_ideal
		integer(4)                      	:: date_error


        	logical                          	::  scan_index_updated 
	    	integer(4)					::  scan_guess
		real(4)                         	::  scan_delta
		real(4)                         	::  scan_delta_abs
		real(4)                          	::  tot_scan_delta_err
		integer(4)                       	::  num_scan_guess_out_of_range


		integer(4)					:: file_num
		integer(4)					:: num_files
		integer(4)					:: num_files_to_process 

        	real(8)                             :: scan_time_last
        	real(8)                             :: scan_time
        	real(8)                             :: scan_time_diff
        
        	integer(4)                          :: start_orbit_index
        	integer(4)                          :: start_scan_index
        
        	real(8)                             :: eqx_time_guess
        	real(8)                             :: next_eqx
       	real(8)                             :: eqx_long_guess
        	real(8)                             :: lon1,lon2
        	real(8)                             :: start_orbit_time,end_orbit_time
        

        	logical                             :: using_noaa_to_start

        
        	real(8)                             :: orbit_guess
        	integer(4)                          :: orbit_guess_int
        	integer(4)                          :: delta_orbit
        	logical                             :: orbit_found
        
        	real(8),save                        :: A_init = 0.0D0
        	real(8)                             :: last_valid_a
        	integer(4)                          :: last_valid_a_loc
        
        	real(8)                             :: last_eqx_time
        	real(8)                             :: first_eqx_time

       

		integer(4),parameter			:: ILU = 20

		integer(4),parameter			:: log_unit = 24
		integer(4),parameter			:: eqx_unit = 23

		character(len = FILE_LEN)		::	log_file	
		character(len = FILE_LEN)		::	eqx_file	
		character(len = FILE_LEN)     	:: output_file
		
		type(date_str)	  			:: cur_date
		character(len=20) 			:: cur_date_string
		
		real(8)         :: nav_min_time
		
		integer(4)      :: num_duplicate_scans
		integer(4)      :: num_backward_steps  
		integer(4)      :: num_non_integer_steps 
		integer(4)      :: num_duplicate_scan_problems
		integer(4)      :: num_good_fovs_new
		integer(4)      :: num_good_fovs_exist
		
		logical         :: cal_fovs_ok_new
		logical         :: cal_fovs_ok_exist
		integer(4)      :: fov
		
		
		
		integer(4)      :: num_scan_located_this_file
		integer(4)      :: num_scan_lost_this_file
				
		character(len = 2)   :: rev_string = 'R1'
		character(len = 528) :: scan_char_long
		
		logical              :: verbose = .false.

		call init_NAN
		tot_scan_delta_err = 0.0
    		
    		! zero all the scans
    		
		do scan_index = 1,MAX_SCANS_PER_MONTH
		        call zero_msu_l1b_orbit_file_scan(msu_l1b_scan_array(scan_index))
		        msu_l1b_scan_valid(scan_index) = 0
		enddo
		    
		if (eval_crossing_only) then
		        call read_MSU_NOAA_nav_file(sat_num,read_error)
		else
		        call read_MSU_extended_nav_file(sat_num,read_error)
		endif		
		        
		! open the log files

		cur_date		= current_date(err)
		cur_date_string = date_string_for_file(cur_date)
            write(log_file,557)trim(MSU_L1B_PATH),sat_names_2(sat_num),sat_names_2(sat_num),year,month
      557   format(a,a,'\logs\',a,'_ingest_',i4.4,'_',i2.2,'.txt')
		open(unit=log_unit,file = log_file)
		write(log_unit,123)sat_names_2(sat_num),month,year
		write(6,123)sat_names_2(sat_num),month,year
	123   format('Processing ',a,2x,i2.2,'/',i4)
		    
		    
		if (eval_crossing_only) then
		        write(eqx_file,559)trim(MSU_L1B_PATH),sat_names_2(sat_num),sat_names_2(sat_num)
      559       	  format(a,a,'\logs\',a,'_raw_crossings.v2.txt')  ! special version to fix NOAA-09
                    close(eqx_unit)
		        open(unit=eqx_unit,file = eqx_file,access = 'APPEND')
		 endif

		 ! choose the proper navigation data
		    
		 if (eval_crossing_only) then
		        nav_eqx_time            = noaa_nav_eqx_time
		        nav_eqx_longitude       = noaa_nav_eqx_longitude
		        nav_orbit_valid         = noaa_nav_orbit_valid
	              nav_a                   = noaa_nav_a              ! orbit time offset (eqx_time = a + orbit_num*orbit_time)
		        nav_max_orbit           = noaa_nav_max_orbit
		        nav_min_orbit           = noaa_nav_min_orbit
		        nav_satnum_loaded       = noaa_nav_satnum_loaded 
		 else
		        nav_eqx_time            = extended_nav_eqx_time
		        nav_eqx_longitude       = extended_nav_eqx_longitude
		        nav_orbit_valid         = extended_nav_orbit_valid
	              nav_a                   = extended_nav_a              ! orbit time offset (eqx_time = a + orbit_num*orbit_time)
		        nav_max_orbit           = extended_nav_max_orbit
		        nav_min_orbit           = extended_nav_min_orbit
		        nav_satnum_loaded       = extended_nav_satnum_loaded 
		  endif

		  ! read the directory in the incoming file list
			
		   call get_files_in_path(MSU_INCOMING_PATH,MAX_FILES,num_files,raw_l1b_file,err)
			
		   write(log_unit,101)num_files,trim(MSU_INCOMING_PATH)
	101	   format(' Found ',i6,' files in ',a)

		   ! sort the files according to date using the appropriate part of the file name

		   call sort_msu_l1b_file_list(num_files,raw_l1b_file)		
				
      !---------------------------------------------------------------------------
	! read through all the file names, and store the ones with the right year and month
	!---------------------------------------------------------------------------

		    num_files_to_process = 0
		    do file_num = 1,num_files
                call decode_times_from_L1B_file(raw_l1b_file(file_num),start_date,end_date,date_error)

                if (((start_date%year .eq. year) .and. (start_date%month .eq. month)) .or.   &
                    ((end_date%year .eq. year) .and. (end_date%month .eq. month))) then
                    num_files_to_process = num_files_to_process + 1
                    l1b_files_to_process(num_files_to_process) = raw_l1b_file(file_num)
                endif
            enddo
            
    !---------------------------------------------------------------------------
	! read through all the files and try and remove duplicate scans and backwards steps
	!---------------------------------------------------------------------------
            write(log_unit,102)num_files_to_process
            write(6,102)num_files_to_process
	102	format(' Processing ',i6,' files that have correct date')
	      if (num_files_to_process .le. 0) then 
	            write(6,*)'No Files'
	            return
	      endif
	
		num_Scan  = 0
		num_scan_this_file  = 0
		num_duplicate_scans = 0
		num_backward_steps  = 0
		num_non_integer_steps = 0
		num_duplicate_scan_problems = 0
		scan_index = 10
			
		scan_time_last = -9.999e30
		nav_min_time = nav_eqx_time(nav_min_orbit)
		
		do file_num = 1,num_files_to_process
			file  = trim(MSU_INCOMING_PATH) // l1b_files_to_process(file_num)
			call decode_times_from_L1B_file(l1b_files_to_process(file_num),start_date,end_date,date_error)
			open(unit = ILU,file = file,status= 'OLD',form = 'binary')
	            istat = FSTAT (ILU,statarray)
                	if (istat .eq. 0) then
                  	size_of_L1B_file = statarray(8)
                	else
                  	size_of_L1B_file = -1
                	endif

	
			read_error = 0
			num_Scan_this_file = 0
			num_scan_located_this_file = 0
			num_scan_lost_this_file = 0

			call check_record_length(ILU,read_error)

			do while (read_Error .le. 0)
				call read_L1A_msu_scan_saa(ILU,msu_l1b_scan,read_error)
				if (read_error .eq. 0) then
					if (msu_l1b_scan%date%month .ne. month) cycle
					if (msu_l1b_scan%date%rd_2000 .lt. nav_min_time) cycle         ! before start of noaa crossing time data
					if (iand(msu_l1b_scan%scan_quality,2) .gt. 0) cycle            ! no location data 
					if (msu_l1b_scan%date%rd_2000 .lt. start_date%rd_2000) cycle   ! date outside range that is supposed to be in this file
					if (msu_l1b_scan%date%rd_2000 .gt. end_date%rd_2000) cycle     ! date outside range that is supposed to be in this file

					if (num_scan .eq. 0) then 
						start_date_ideal = set_date(month,1,year,date_error)
						scan_index = 5 + nint((msu_l1b_scan%date%rd_2000 - start_date_ideal%rd_2000)/MSU_SCAN_TIME)
					endif
					        
					msu_l1b_scan%sat_num = sat_num
					num_scan = num_Scan + 1
					num_scan_this_file = num_Scan_this_file + 1
						
					if (num_scan .eq. 1) then
						scan_index_updated = .true.
					else
						! check for backward time step
    						scan_index_updated = .false.
    						scan_time_last = msu_l1b_scan_array(scan_index)%date%rd_2000
    						scan_delta = (msu_l1b_scan%date%rd_2000 - scan_time_last)/MSU_SCAN_TIME
    						scan_delta_abs = abs(scan_delta -1.0)
    						if (scan_delta_abs .lt. 0.002) then
                                		! most probably case -- stepped forward one scan
                                		scan_index = scan_index + 1
                                		scan_index_updated = .true.
                                		scan_jump = 1
                            		else
    						    if (verbose)print *,'scan_delta = ',scan_delta
						    if (scan_delta .le. -0.1) then
						    		if (verbose) print *,'Backwards Time Step ',scan_delta,nint(scan_delta)
						      	num_backward_steps = num_backward_steps+1
						            
						     		if (abs(scan_delta - nint(scan_delta)) .lt. 0.05) then
						      		scan_jump = nint(scan_delta)
						            	scan_index = scan_index + scan_jump 
						            	scan_index_updated = .true.
						      	else 
						            	scan_jump = nint(scan_delta)
						            	scan_index = scan_index + scan_jump 
						            	scan_index_updated = .true.
						            	num_non_integer_steps = num_non_integer_steps + 1
                                    		endif  ! if (abs(scan_delta - nint(scan_delta)) .lt. 0.05) then   
							endif ! if (scan_delta .le. -0.1) 
						      if (abs(scan_delta) .lt. 0.1) then  ! about 2.96e-5
                                    		! more or less at same time -- dupicate candidate
                                    		if (verbose) print *,'Duplicate Time',scan_delta
                                    		scan_index = scan_index 
						            scan_index_updated = .true.
                                		endif !if (abs(scan_delta) .lt. 0.1) then 
                        
                                		if (scan_delta .ge. 0.1) then
                                    		!scan_delta = (msu_l1b_scan%date%rd_2000 - scan_time_last)/MSU_SCAN_TIME
                                    		if (verbose) print *,'Anomalous Forward Time Step ',scan_delta,nint(scan_delta)
                                    		if (abs(scan_delta - nint(scan_delta)) .lt. 0.05) then
                                        			! looks good for an integer number of scans missing
                                        			scan_jump = nint(scan_delta)
                                        			scan_index = scan_index + scan_jump
                                        			scan_index_updated = .true.
                                    		else  ! problem, need to deal
                                        			tot_scan_Delta_err = tot_scan_Delta_err+(scan_delta - nint(scan_delta))
                                        			print *,'Non Int Scan',scan_delta,tot_scan_Delta_err
                                        			scan_jump = nint(scan_delta)
                                        
                                   
                                        			scan_index = scan_index + scan_jump
                                        			num_non_integer_steps = num_non_integer_steps + 1
                                        			scan_index_updated = .true.
                                    		endif
                                		endif ! if (scan_delta .ge. 0.1) then
                             		endif  ! if (scan_delta_abs .lt. 0.002) then
                          	endif     ! if (num_scan .eq. 1) then 
                          
                          	
                        
                          	if ((scan_index_updated) .and. (scan_index .ge. 1)) then
                            		! test to see if there is a scan already there
                            		test_scan = msu_l1b_scan_array(scan_index)
                            		if (test_scan%chksum .eq. 0) then   ! check sum = zero means spot is blank
                                		msu_l1b_scan_array(scan_index) = msu_l1b_scan
                                		msu_l1b_scan_valid(scan_index) = 1
                                		scan_time_last = msu_l1b_scan%date%rd_2000
                            		else
                                		if (test_scan%chksum .eq. msu_l1b_scan%chksum) then
                                    		! duplicate scan - do nothing
                                    		num_duplicate_scans = num_duplicate_scans + 1
                                    		scan_time_last = msu_l1b_scan%date%rd_2000
                                		else  ! problem, need to deal
                                     		num_duplicate_scan_problems = num_duplicate_scan_problems + 1
                                    		cal_fovs_ok_new = .true.
                                    		cal_fovs_ok_exist = .true.
                                    
                                    		num_good_fovs_new = 0
                                    		num_good_fovs_exist = 0
                                    
                                    		do fov = 12,13
                                        			if (test_scan%fov_quality(fov) .gt. 0) cal_fovs_ok_exist = .false.
                                        			if (msu_l1b_scan%fov_quality(fov) .gt. 0) cal_fovs_ok_new = .false.
                                    		enddo
                                    
                                    		do fov = 1,11
                                        			if (test_scan%fov_quality(fov) .eq. 0) num_good_fovs_exist = num_good_fovs_exist+1
                                        			if (msu_l1b_scan%fov_quality(fov) .eq. 0) num_good_fovs_new = num_good_fovs_new+1
                                    		enddo
                                    
                                    		if (cal_fovs_ok_new) then
                                        			if (.not.( cal_fovs_ok_exist)) then  ! new scan is better
                                            			msu_l1b_scan_array(scan_index) = msu_l1b_scan
                                            			msu_l1b_scan_valid(scan_index) = 1
                                            			scan_time_last = msu_l1b_scan%date%rd_2000
                                        			else
                                            			! cal exists for both scans -- use number of data FOVS to decide
                                            			if (num_good_fovs_new .gt. num_good_fovs_exist) then 
                                                			msu_l1b_scan_array(scan_index) = msu_l1b_scan
                                                			msu_l1b_scan_valid(scan_index) = 1
                                                			scan_time_last = msu_l1b_scan%date%rd_2000
                                            			else
                                                			scan_time_last = test_scan%date%rd_2000
                                            			endif  
                                        			endif
                                    		else
                                        			! new scan is missing calibration data.
                                        			scan_time_last = msu_l1b_scan%date%rd_2000
                                    		endif ! if (cal_fovs_ok_new)
                                 		endif !if (test_scan%chksum .eq. msu_l1b_scan%chksum)
                           		endif !if (test_scan%chksum .eq. 0)
                         	endif  ! ((scan_index_updated) .and. (scan_index .ge. 1))
                     endif  ! read error eq 0
                 enddo !(read_Error .le. 0)  loop in one file
                 write(6,110)trim(file(20:)),num_scan_this_file,size_of_L1B_file
				 write(log_unit,110)trim(file(20:)),num_scan_this_file,size_of_L1B_file
	110			 format('Scanned',1x,a,' Num_Scans:',i7,'Size of File:',i8)
            enddo      ! file_num = 1,num_files_to_process

            write(log_unit,354)
            print *,'Num_Scans = ',num_Scan
            print *,'num_duplicate_scans = ',num_duplicate_scans
            print *,'num_backward_steps = ',num_backward_steps
            print *,'num_non_integer_steps = ',num_non_integer_steps
            print *,'num_duplicate_scan_problems = ',num_duplicate_scan_problems 
            write(log_unit,1111)'Num_Scans = ',num_Scan
            write(log_unit,1111)'num_duplicate_scans = ',num_duplicate_scans
            write(log_unit,1111)'num_backward_steps = ',num_backward_steps
            write(log_unit,1111)'num_non_integer_steps = ',num_non_integer_steps
            write(log_unit,1111)'num_duplicate_scan_problems = ',num_duplicate_scan_problems 
    1111    format(a,1x,i8)
            continue
            write(log_unit,354)
            
            ! ok now all read in 
            ! check once again for duplicate scans and backward steps
            ! 
            orbit_index = 100
            do scan_index = 1,MAX_SCANS_PER_MONTH
                if (msu_l1b_scan_valid(scan_index) .gt. 0) then
                    if (msu_l1b_scan_valid(scan_index+1) .gt. 0) then
                        if (abs(msu_l1b_scan_array(scan_index+1)%date%rd_2000 - msu_l1b_scan_array(scan_index)%date%rd_2000) .lt. 0.00001) then
                            if (msu_l1b_scan_array(scan_index+1)%chksum .eq. msu_l1b_scan_array(scan_index)%chksum) then
                                print *,'duplicate scan',msu_l1b_scan_array(scan_index)%date%rd_2000,msu_l1b_scan_array(scan_index+1)%date%rd_2000
                                msu_l1b_scan_valid(scan_index) = 0
                            else
                                print *,'same time, different chksum',msu_l1b_scan_array(scan_index)%chksum,msu_l1b_scan_array(scan_index+1)%chksum
                                msu_l1b_scan_valid(scan_index) = 0
                                msu_l1b_scan_valid(scan_index+1) = 0
                            endif     
                        endif
                        if (abs(msu_l1b_scan_array(scan_index+1)%date%rd_2000 - msu_l1b_scan_array(scan_index)%date%rd_2000) .lt. -0.00001) then
                            print *,'backward time step',msu_l1b_scan_array(scan_index)%date%rd_2000,msu_l1b_scan_array(scan_index+1)%date%rd_2000
                            print *,'fatal error'
                            stop
                        endif
                    endif
                endif
            enddo
            
		!-------------------------------------------------------------------------------------------------------------------------
            ! this next section assigns orbit numbers and scan numbers in the orbit to each scan, based on the loaded navigation data.
		!-------------------------------------------------------------------------------------------------------------------------

            start_orbit_index = nav_min_orbit+5
            
            ! first, go through RSS navigation data to get first orbit, A values
            
            start_scan_index = 0
            
            first_eqx_time = nav_eqx_time(nav_min_orbit)
            last_eqx_time  = nav_eqx_time(nav_max_orbit)-MSU_ORBIT_TIME
            
            
            using_noaa_to_start = .false.
            do scan_index = 1,MAX_SCANS_PER_MONTH
                if (msu_l1b_scan_valid(scan_index) .gt. 0) then
                    scan_time = MSU_L1B_scan_array(scan_index)%date%rd_2000
                    
                    if (.not.(eval_crossing_only)) then 
                        if (scan_time .gt. last_eqx_time) then
                          print *,'Scan Time after first last time'
                          cycle
                        endif
                    endif
                    
                    if (scan_time .lt. first_eqx_time) then
                        print *,'Scan Time before first crossing time'
                        cycle
                    endif
                    
                    orbit_found = .false.
                    
                    do while (.not.(orbit_found))
                    
                        print *,start_orbit_index,scan_time,nav_eqx_time(start_orbit_index),nav_eqx_time(start_orbit_index+1)
                        next_eqx = nav_eqx_time(start_orbit_index+1)
                        if (abs((nav_eqx_time(start_orbit_index+1)-nav_eqx_time(start_orbit_index)) - MSU_ORBIT_TIME) .gt. 0.001) then 
                            next_eqx = nav_eqx_time(start_orbit_index) + MSU_ORBIT_TIME
                        endif
                        if (nav_orbit_valid(start_orbit_index) .gt. 0) then
                            if (.not. finite(next_eqx)) then
                                next_eqx = nav_eqx_time(start_orbit_index) + MSU_ORBIT_TIME
                            endif
                            if ((scan_time .ge. nav_eqx_time(start_orbit_index)).and. (scan_time .lt. next_eqx)) then
                                orbit_found = .true. 
                            else
                                if (scan_time .lt. nav_eqx_time(start_orbit_index)) then
                                    start_orbit_index = start_orbit_index - 1
                                endif 
                                if (scan_time .ge. next_eqx) then
                                    start_orbit_index = start_orbit_index + 1
                                endif
                            endif
                        
                           
                            if ((start_orbit_index .gt. nav_max_orbit) .or. (start_orbit_index .lt. nav_min_orbit)) then
                                print *,'Problem finding first orbit'
                                if (eval_crossing_only) then
                                    
                                    ! guess the orbit if time is beyond last noaa orbit
                                    if (scan_time .gt. last_eqx_time) then
                                        print *,'Extraploating past last orbit'
                                        start_orbit_index = nav_max_orbit + nint((scan_time - nav_eqx_time(noaa_nav_max_orbit))/MSU_ORBIT_TIME)
                                        orbit_found = .true.
                                        exit
                                    endif
                                endif
                                continue
                            endif
                        else  ! we get here if we are not at an orbit number with a known crossing time.
                              ! rescan entire set of orbits to see if there is a match anywhere (this would take too long to do everytime)
                            do orbit_index2 = 1,MX_ORBITS
                                if (nav_orbit_valid(orbit_index2) .gt. 0) then
                                    start_orbit_time = nav_eqx_time(orbit_index2)
                                    if (nav_orbit_valid(orbit_index2+1) .gt. 0) then
                                        end_orbit_time = nav_eqx_time(orbit_index2+1)
                                    else
                                        end_orbit_time = start_orbit_time + MSU_ORBIT_TIME
                                    endif
                                    
                                    if ((scan_time .ge. start_orbit_time) .and. (scan_time .le. end_orbit_time)) then
                                        orbit_found = .true.
                                        start_orbit_index = orbit_index2
                                        exit
                                    endif
                                endif
                            enddo
                            if (.not.(orbit_found)) exit
                        endif
                    enddo
  
                    start_scan_index = scan_index
                    exit
                endif
            enddo
            
            if (start_scan_index .eq. 0) then
                PRINT *,'No Valid Scans'
                return
            endif
            
            if (start_orbit_index .gt. nav_max_orbit) then
                if (abs(a_init) .lt. 0.00001) then
                    A_init = nav_a(nav_max_orbit)
                endif
            else
                A_init = nav_a(start_orbit_index)
            endif
            
            write(log_unit,354)
            354 format('---------------------------------------------------')   
            write(log_unit,*)'Crossing Times Detected'
            write(log_unit,*)'Orbit Guess    A      Eqx_Time'
            
            do scan_index = start_scan_index+1,MAX_SCANS_PER_MONTH
                if ((msu_l1b_scan_valid(scan_index) .gt. 0) .and. (msu_l1b_scan_valid(scan_index-1) .gt. 0)) then
                    scan_time_diff = MSU_L1B_scan_array(scan_index)%date%rd_2000 - MSU_L1B_scan_array(scan_index-1)%date%rd_2000
                    
                    if (abs(scan_time_diff - MSU_SCAN_TIME) .lt. (MSU_SCAN_TIME/20)) then
                        if ((MSU_L1B_scan_array(scan_index-1)%fov_latitude(6) .lt. 0.0) .and. &
                            (MSU_L1B_scan_array(scan_index)%fov_latitude(6)   .gt. 0.0) .and. &
                            ((MSU_L1B_scan_array(scan_index)%fov_latitude(6)-MSU_L1B_scan_array(scan_index-1)%fov_latitude(6)) .lt. 2.0)) then
                                ! ascending crossing detected:
                                
                                ! estimate crossing time
                                
                                eqx_time_guess = MSU_L1B_scan_array(scan_index-1)%date%rd_2000 + &
                                    MSU_SCAN_TIME*(1.0-  &
                                    MSU_L1B_scan_array(scan_index)%fov_latitude(6)/  &
                                   (MSU_L1B_scan_array(scan_index)%fov_latitude(6)-MSU_L1B_scan_array(scan_index-1)%fov_latitude(6)))
                                   
                                lon1 = MSU_L1B_scan_array(scan_index-1)%fov_longitude(6)
                                lon2 = MSU_L1B_scan_array(scan_index)%fov_longitude(6)
                                print *,lon1,lon2
                                if ((lon1 .lt. 10) .and. (lon2 .gt. 350)) then
                                    lon2 = lon2-360.0
                                endif
                                   
                                eqx_long_guess = lon1 + &
                                        (lon2-lon1)*(1.0-  &
                                         MSU_L1B_scan_array(scan_index)%fov_latitude(6)/  &
                                        (MSU_L1B_scan_array(scan_index)%fov_latitude(6)-MSU_L1B_scan_array(scan_index-1)%fov_latitude(6)))
                                        
                                if (eqx_long_guess .lt. 0.0) then
                                    eqx_long_guess = eqx_long_guess + 360.0
                                endif

                                
                                ! figure out which orbit it is
                                
                                orbit_guess = (eqx_time_guess - A_init)/MSU_ORBIT_TIME
                                orbit_guess_int = nint(orbit_guess)
                                if (orbit_guess_int .ge. 21446)then
                                    print *,orbit_guess,orbit_guess_int,abs(orbit_guess - orbit_guess_int)
                                endif
                                if (eval_crossing_only) write(eqx_unit,149),orbit_guess_int,orbit_guess,eqx_time_guess,eqx_time_guess - orbit_guess_int*MSU_ORBIT_TIME, eqx_long_guess
                                write(6,149),orbit_guess_int,orbit_guess,eqx_time_guess,eqx_time_guess - orbit_guess_int*MSU_ORBIT_TIME, eqx_long_guess

                                if (abs(orbit_guess - orbit_guess_int) .lt. 0.2) then 
                                    
                                    rss_nav_a_from_data(orbit_guess_int) = eqx_time_guess - orbit_guess_int*MSU_ORBIT_TIME
                                    rss_nav_orbit_valid_from_data(orbit_guess_int) = 1
                                    rss_nav_eqx_time_from_data(orbit_guess_int) = eqx_time_guess
                                    a_init = rss_nav_a_from_data(orbit_guess_int)
                                    
                                    !print 149,         orbit_guess_int,rss_nav_eqx_time_from_data(orbit_guess_int),nav_eqx_time(orbit_guess_int),rss_nav_eqx_time_from_data(orbit_guess_int)-nav_eqx_time(orbit_guess_int)
                                    !write(log_unit,149)orbit_guess_int,rss_nav_eqx_time_from_data(orbit_guess_int),nav_eqx_time(orbit_guess_int),rss_nav_eqx_time_from_data(orbit_guess_int)-nav_eqx_time(orbit_guess_int)
                                    
                                    if (abs(rss_nav_eqx_time_from_data(orbit_guess_int)-nav_eqx_time(orbit_guess_int)) .gt. 0.001) then
                                        rss_nav_orbit_valid_from_data(orbit_guess_int) = 4
                                        if (abs(rss_nav_eqx_time_from_data(orbit_guess_int)-nav_eqx_time(orbit_guess_int)) .gt. 0.03) then
                                            print *,'crossing time difference really big'
                                            write(log_unit,*)'crossing time difference really big'

                                            rss_nav_orbit_valid_from_data(orbit_guess_int) = 8
                                        endif
                                    endif
      
                                    eqx_long_guess = MSU_L1B_scan_array(scan_index-1)%fov_longitude(6) + &
                                        (MSU_L1B_scan_array(scan_index)%fov_longitude(6)-MSU_L1B_scan_array(scan_index-1)%fov_longitude(6))*(1.0-  &
                                         MSU_L1B_scan_array(scan_index)%fov_latitude(6)/  &
                                        (MSU_L1B_scan_array(scan_index)%fov_latitude(6)-MSU_L1B_scan_array(scan_index-1)%fov_latitude(6)))
                             
                                    rss_nav_eqx_longitude_from_data(orbit_guess_int) = eqx_long_guess
                                
                                    !write(eqx_unit,149),orbit_guess_int,orbit_guess,rss_nav_eqx_time_from_data(orbit_guess_int),rss_nav_eqx_longitude_from_data(orbit_guess_int),rss_nav_a_from_data(orbit_guess_int)
                                    write(log_unit,149),orbit_guess_int,orbit_guess,rss_nav_eqx_time_from_data(orbit_guess_int),rss_nav_eqx_longitude_from_data(orbit_guess_int),rss_nav_a_from_data(orbit_guess_int)
                                    !write(6,149),orbit_guess_int,orbit_guess,rss_nav_eqx_time_from_data(orbit_guess_int),rss_nav_eqx_longitude_from_data(orbit_guess_int),rss_nav_a_from_data(orbit_guess_int)
                             149    format(i7,2x,f14.6,2x,f14.6,2x,f14.6,2x,f14.6)
                                else
                                    print *,'Orbit crossing time difference large'
                                endif
                         endif
                     endif
                 endif
             enddo
             
             
            if (eval_crossing_only) return 
             
            write(log_unit,354)
            
            ! compare values of rss_nav_a_from_data with values in nav_1
            
            do orbit_index = 1,MX_ORBITS
                if ((finite(nav_a(orbit_index))) .and. (abs(nav_a(orbit_index) .gt. 0.001))) then
                    if (finite(nav_a(orbit_index))) then
                        if (abs(nav_a(orbit_index) - nav_a(orbit_index)) .gt. 5*MSU_SCAN_TIME) then
                            print *,'Large time error, leaving a unchanged'
                        else
                            nav_a(orbit_index) = rss_nav_a_from_data(orbit_index)
                            nav_eqx_time = (orbit_index*MSU_ORBIT_TIME) + nav_a(orbit_index)
                            nav_orbit_valid(orbit_index) = 4
                        endif
                    endif
                 endif
            enddo

            ! now, need to fill in the values for A from the L1B data.  Data is also extrapolated into large gaps
                     
            nav_max_orbit = 0
            nav_min_orbit  = MX_ORBITS
             
            do orbit_index = 1,MX_ORBITS
                if (abs(nav_a(orbit_index)) .gt. 0.0001) then
                    if (orbit_index .lt. nav_min_orbit) nav_min_orbit = orbit_index
                    if (orbit_index .gt. nav_max_orbit) nav_max_orbit = orbit_index
                endif
            enddo
            
            last_valid_a = nav_a(nav_max_orbit)
            first_eqx_time = nav_eqx_time(nav_min_orbit)
            last_eqx_time = nav_eqx_time(nav_max_orbit)
             
            
            orbit_index = nav_min_orbit+100
            delta_orbit = -1
            num_scan_guess_out_of_range = 0
             
            do scan_index = 1,MAX_SCANS_PER_MONTH
                if (msu_l1b_scan_valid(scan_index) .gt. 0) then
                    scan_time = msu_l1b_scan_array(scan_index)%date%rd_2000
                    if ((scan_time .ge. first_eqx_time) .and. (scan_time .le. last_eqx_time)) then 
                        orbit_found = .false.
                       
                        do while (.not.(orbit_found))
                            if ((orbit_index .lt. 1) .or. (orbit_index .gt. MX_ORBITS)) then  ! we have run off the end of possible orbits without finding match
                                exit
                            endif
                            if (nav_orbit_valid(orbit_index) .gt. 0) then
                            
                                ! figure out start and end times for this orbit.
                                ! prefer that the end orbit time is from nav data, but if not available, use start time + orbit time.
                                
                                start_orbit_time = nav_eqx_time(orbit_index)
                                
                                if (nav_orbit_valid(orbit_index+1) .gt. 0) then
                                    end_orbit_time = nav_eqx_time(orbit_index+1)
                                else
                                    end_orbit_time = start_orbit_time + MSU_ORBIT_TIME
                                endif
                        
                                ! check to see if scan is in this orbit
                                ! previous eqx_time is the last valid eqx time seen -- used to decide direction when eqx time for the current orbit
                                ! is not found.
                                
                                if ((scan_time .ge. start_orbit_time) .and. (scan_time .le. end_orbit_time)) then
                                    orbit_found = .true.
                                else ! this is not the right orbit -- increment or decrement the orbit as appropriate
                                    if (scan_time .lt. start_orbit_time) then
                                        orbit_index = orbit_index - 1
                                        delta_orbit = -1.0D0
                                    endif
                                    if (scan_time .gt. end_orbit_time) then
                                        orbit_index = orbit_index + 1     
                                        delta_orbit = -1.0D0 
                                    endif
                                endif
                            else  ! we get here if we are not at an orbit number with a known crossing time.
                                  ! rescan entire set of orbits to see if there is a match anywhere (this would take too long to do everytime)
                                do orbit_index2 = 1,MX_ORBITS
                                    if (rss_nav_orbit_valid_from_data(orbit_index2) .gt. 0) then
                                        start_orbit_time = rss_nav_eqx_time_from_data(orbit_index2)
                                        if (rss_nav_orbit_valid_from_data(orbit_index2+1) .gt. 0) then
                                            end_orbit_time = rss_nav_eqx_time_from_data(orbit_index2+1)
                                        else
                                            end_orbit_time = start_orbit_time + MSU_ORBIT_TIME
                                        endif
                                        
                                        if ((scan_time .ge. start_orbit_time) .and. (scan_time .le. end_orbit_time)) then
                                            orbit_found = .true.
                                            orbit_index = orbit_index2
                                            exit
                                        endif
                                    endif
                                enddo
                                if (.not.(orbit_found)) exit
                            endif
                        enddo
                        
                        if (orbit_found) then
                            scan_guess = 1 + floor((scan_time - nav_eqx_time(orbit_index))/MSU_SCAN_TIME)
                            msu_l1b_scan_array(scan_index)%orbit_num = orbit_index
                            msu_l1b_scan_array(scan_index)%scan_number = scan_guess
                        else
                            print *,'Scan Time out of range with known orbit numbers',scan_guess
                            num_scan_guess_out_of_range = num_scan_guess_out_of_range + 1
                            msu_l1b_scan_array(scan_index)%scan_number = -99
                            msu_l1b_scan_array(scan_index)%orbit_num = -99
                        endif
                    else
                        ! scan time out of range
                        print *,'Scan Time out of range with known orbit numbers',scan_guess
                        num_scan_guess_out_of_range = num_scan_guess_out_of_range + 1
                        msu_l1b_scan_array(scan_index)%scan_number = -99
                        msu_l1b_scan_array(scan_index)%orbit_num = -99
                    endif
                    if ((scan_guess .lt. 0) .or. (scan_guess .gt. 500)) then
                        print *,'Scan guess too large/small',scan_guess
                        num_scan_guess_out_of_range = num_scan_guess_out_of_range + 1
                        msu_l1b_scan_array(scan_index)%scan_number = -99
                        msu_l1b_scan_array(scan_index)%orbit_num = -99
                    endif
                endif
            enddo
            
            print *,num_scan_guess_out_of_range,' scan indices were out of range'
            write(log_unit,189)num_scan_guess_out_of_range
            189 format(i9,' scan indices were out of range')
            
             
            ! now write out the data to a monthly L1b file.
            
            if (wrt) then
                num_scan = 0
                write(output_file,601)trim(MSU_L1B_PATH),sat_names_2(sat_num),sat_names_2(sat_num),year,month,rev_string
          601   format(a,a,'\monthly\',a,'_monthly_L1B_',i4.4,'_',i2.2,'_',a,'.dat')
                close(3)
                open(unit = 3,file = output_file,form='binary')
                do scan_index = 1,MAX_SCANS_PER_MONTH
                    if (msu_l1b_scan_valid(scan_index) .gt. 0) then
                        call encode_msu_l1b_orbit_file_scan_long(	msu_l1b_scan_array(scan_index),		&
											                        scan_char_long)
			            write(3)scan_char_long
			            num_scan = num_Scan + 1
                    endif
                enddo
                write(log_unit,354)
                close(3)
                write(log_unit,1112)'file_written: ',output_file
                write(6,1112)'file_written: ',output_file
        1112    format(a,a)
                write(log_unit,1111)'num_scans_written = ',num_scan
                write(6,1111)'num_scans_written = ',num_scan
                write(log_unit,354)
                close(log_unit)
                if (eval_crossing_only) close(eqx_unit)
             else
                write(log_unit,1111)'No Scans Written Because wrt = .false.'
                write(6,1111)'No Scans Written Because wrt = .false.'
                write(log_unit,354)
                close(log_unit) 
                if (eval_crossing_only) close(eqx_unit)
             endif
             if (eval_crossing_only) close(eqx_unit)
         end subroutine ingest_MSU_l1b 
       
;@***h* MLT_V4_RSS/sort_msu_l1b_file_list
; NAME
;	sort_msu_l1b_file_list
;
; PURPOSE
;	sorts a list of MSU L1B CLASS files into date order
;
; DESCRIPTION
;	sorts a list of MSU L1B CLASS files into date order using the file name
;
; INPUTS
;	num_files		integer(4)								number of files in list
;	raw_l1b_file	character(len = FILE_LEN),dimension(MAX_FILES)		list of file names, unsorted
;
; OUTPUTS
;	raw_l1b_file	character(len = FILE_LEN),dimension(MAX_FILES)		list of file names, sorted into date order

;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call sort_msu_l1b_file_list(num_files,raw_l1b_file)
;
;@*****
         subroutine sort_msu_l1b_file_list(num_files,raw_l1b_file)

			integer(4)							::  num_files
			character(len = FILE_LEN),dimension(MAX_FILES)	::	raw_l1b_file
			character(len = FILE_LEN),dimension(MAX_FILES)	::	temp_file_list

			character(len = 5)				:: temp1
			character(len = 3)				:: temp2
			character(len = 9)				:: temp3

			real(4),dimension(num_files) :: file_index
			integer(4),dimension(num_files) :: indx
			integer(4)						:: i
			
			
			! first extract integer related to date and time

			do i = 1,num_files
			    if (raw_l1b_file(i)(1:3) .eq. 'NSS') then
				    temp1 = raw_l1b_file(i)(14:18)
				    temp2 = raw_l1b_File(i)(21:23)
				    temp3 = temp1 // temp2
				    read(temp3,111) file_index(i)
	111			    format(f9.0)
	            else
	                file_index(i) = -9999.99
	            endif
			enddo

			! sort this integer array

			call indexx(num_files,file_index,indx)

			! put sorted file names in temp variable

			do i = 1, num_files
				temp_file_list(i) = raw_l1b_file(indx(i))
			enddo

			! put back into original array

			raw_l1b_file = temp_file_list

			return

		end subroutine sort_msu_l1b_file_list   
		
;@***h* MLT_V4_RSS/fill_in_gaps
; NAME
;	fill_in_gaps
;
; PURPOSE
;	fills in gaps in the A time series
;
; DESCRIPTION
;	fills in short gaps in the A time series using linear interpolation
;
; INPUTS
;	A				real(8),dimension(MAX_ORBITS)		time series for gap filling
;	orbit_valid			integer(4),dimension(MAX_ORBITS)	valid flag.  0 means not valid, 1 means valid, 2 means valid, but already obtained from interpolation
;
; OUTPUTS
;	A				real(8),dimension(MAX_ORBITS)		time series for gap filling
;	orbit_valid			integer(4),dimension(MAX_ORBITS)	valid flag.  0 means not valid, 1 means valid, 2 means valid, but already obtained from interpolation
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call fill_in_gaps(A,orbit_valid)
;
;@*****
		subroutine fill_in_gaps(A,orbit_valid)

		    real(8),dimension(MX_ORBITS)            :: A
		    integer(4),dimension(MX_ORBITS)         :: orbit_valid
		    
		    integer(4)      :: first_A
		    integer(4)      :: last_A
		    
		    integer(4)      :: i,k
		    
		    integer(4),dimension(1000)      :: gap_start
		    integer(4),dimension(1000)      :: gap_end
		    integer(4),dimension(1000)      :: gap_length
		    integer(4)                      :: gap_num,num_gaps
		    integer(4)                      :: last_good
		    
		    logical(4)                      :: in_gap
		    
		    real(8)                         :: delta_a
		    
		    first_a = MX_orbits
		    last_a = 1
		    
		    do i = 1,MX_ORBITS
		        if (abs(a(i)) .gt. 1.0) then
		            if (i .lt. first_a) first_a = i
		            if (i .gt. last_a) last_a = i
		        endif
		    enddo
		    
		    ! now go through and look for gaps
		    gap_num = 0 
		    in_gap = .false.
		    last_good = 0
		    do i = 1,mx_orbits
		        if (abs(a(i)) .lt. 1.0) then 
		            in_gap = .true.
		        else
		            if (in_gap) then
		                gap_num = gap_num + 1
		                gap_start(gap_num) = last_good+1
		                gap_end(gap_num) = i - 1
		                gap_length(gap_num) = 1 + gap_end(gap_num) - gap_start(gap_num)
		                in_gap = .false.
		            endif
		            last_good = i
		        endif
		     enddo
		     num_gaps = gap_num
		     do gap_num = 1,num_gaps
		        if (gap_length(gap_num) .le. 0) cycle
		        if (gap_length(gap_num) .lt. 300) then
		            if (gap_start(gap_num) .gt. 1) then
		                if ((orbit_valid(gap_start(gap_num)-1) .ge. 1) .and. (orbit_valid(gap_end(gap_num)+1) .ge. 1)) then
		                    do i = 1,gap_length(gap_num)
		                        if ((gap_start(gap_num)-1+i) .gt. MX_orbits) cycle
		                        A(gap_start(gap_num)-1+i) = A(gap_start(gap_num)-1) + (float(i)/(gap_length(gap_num) + 1.0))*(A(gap_end(gap_num)+1) -A(gap_start(gap_num)-1))
		                        orbit_valid(gap_start(gap_num)-1+i) = 2
		                    enddo
		                else
		                    print *,'Gap bordered by already interpolated data',gap_start(gap_num),gap_end(gap_num)
		                endif
		            else
		                if (abs(a(gap_end(gap_num) + 2)) .gt. 1.0) then
		                    ! linear interpolate a 20 orbits backwards from end of gap
		                    delta_a = a(gap_end(gap_num) +2) - a(gap_end(gap_num) +1)
		                else
		                    delta_a = 0.0
		                endif
		                if (orbit_valid(gap_end(gap_num)+1) .ne. 1) then 
		                    do k = 0,19
		                        if (gap_end(gap_num)-k .gt. 0) then
		                            if ((gap_end(gap_num)-k) .lt. 1) cycle
		                            a(gap_end(gap_num)-k) = a(gap_end(gap_num)+1)-(k+1)*delta_a
		                            orbit_valid(gap_end(gap_num)-k) = 2
		                        endif
		                    enddo 
		                else
			                print *,'Gap ended by already interpolated data',gap_start(gap_num),gap_end(gap_num)
		                endif
		            endif
		        else
		            print *,'gap_to_big to span',gap_length(gap_num)
		            if (gap_start(gap_num) .gt. 1) then
                        if ((orbit_valid(gap_start(gap_num) -1) .ge. 1) .and. (orbit_valid(gap_start(gap_num) -2) .ge. 1)) then 
		                    if (abs(a(gap_start(gap_num) -2)) .gt. 1.0) then
		                        ! linear interpolate a 20 orbits forward from start
		                        delta_a = a(gap_start(gap_num) -1) - a(gap_start(gap_num) -2)
		                    else
		                        delta_a = 0.0
		                    endif
		                    do k = 0,19 
		                       if ((gap_start(gap_num)+k) .gt. MX_orbits) cycle
		                       a(gap_start(gap_num)+k) = a(gap_start(gap_num)-1)+(k+1)*delta_a
		                       orbit_valid(gap_start(gap_num)+k) = 2
		                    enddo 
		                else
		                    print *,'Gap bordered by already interpolated data',gap_start(gap_num),gap_end(gap_num)
		                endif   
		            endif
		            if (gap_end(gap_num) .lt. MX_orbits) then
		                if ((orbit_valid(gap_end(gap_num) + 2) .ge. 1) .and. (orbit_valid(gap_end(gap_num) + 2) .ge. 1)) then
		                    if (abs(a(gap_end(gap_num) + 2)) .gt. 1.0) then
		                        ! linear interpolate a 20 orbits forward from start
		                        delta_a = a(gap_end(gap_num) +2) - a(gap_end(gap_num) +1)
		                    else
		                        delta_a = 0.0
		                    endif
		                    do k = 0,19
		                       if ((gap_end(gap_num)-k) .lt. 1) cycle
		                       a(gap_end(gap_num)-k) = a(gap_end(gap_num)-1)+(k+1)*delta_a
		                       orbit_valid(gap_end(gap_num)-k) = 2
		                    enddo
		                 else
		                    print *,'Gap bordered by already interpolated data',gap_start(gap_num),gap_end(gap_num)
		                endif
		            endif     
		        endif
		     enddo
		     
		     ! now extrapolate the data 1 orbit forward
		     
		     if (abs(a(last_a-1)) .gt. 0) then
		        delta_a = a(last_a) - a(last_a-1)
		        a(last_a +1) = a(last_a) + delta_a
		        orbit_valid(last_a + 1) = 2
		     endif
		     
		     
		     
		end subroutine fill_in_gaps                 							 
	end module msu_l1b_ingest