;@***h* MLT_V4_RSS/MSU_Navigation
; NAME
;	MSU_Navigation
;
; PURPOSE
;	module containing MSU navigation data structures and routines
;
; DESCRIPTION
;	module containing MSU navigation data structures and routines
;	
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	use MSU_Navigation
;
;@*****

$freeform
module MSU_Navigation

		use msu_types 
		use msu_constants
		use NAN_support
		use Date_and_Time
		use type_conversion
		implicit none
		
		integer(4),parameter                :: MX_ORBITS = 100000
		
		real(8),dimension(MX_ORBITS)        :: noaa_nav_eqx_time
		real(8),dimension(MX_ORBITS)        :: noaa_nav_eqx_longitude
		integer(4),dimension(MX_ORBITS)     :: noaa_nav_orbit_valid
	      real(8),dimension(MX_ORBITS)        :: noaa_nav_a              ! orbit time offset (eqx_time = a + orbit_num*orbit_time)
		integer(4)                          :: noaa_nav_max_orbit
		integer(4)                          :: noaa_nav_min_orbit
		integer(4)                          :: noaa_nav_satnum_loaded = 0  

		real(8),dimension(MX_ORBITS)        :: extended_nav_eqx_time
		real(8),dimension(MX_ORBITS)        :: extended_nav_eqx_longitude
		integer(4),dimension(MX_ORBITS)     :: extended_nav_orbit_valid
	    	real(8),dimension(MX_ORBITS)        :: extended_nav_a              ! orbit time offset (eqx_time = a + orbit_num*orbit_time)
		integer(4)                          :: extended_nav_max_orbit
		integer(4)                          :: extended_nav_min_orbit
		integer(4)                          :: extended_nav_satnum_loaded = 0  
		
	    	real(8),dimension(MX_ORBITS)        :: rss_nav_eqx_time
	    	real(8),dimension(MX_ORBITS)        :: rss_nav_eqx_time_from_data
		real(8),dimension(MX_ORBITS)        :: rss_nav_eqx_longitude
		real(8),dimension(MX_ORBITS)        :: rss_nav_eqx_longitude_from_data
		integer(4),dimension(MX_ORBITS)     :: rss_nav_orbit_valid
		integer(4),dimension(MX_ORBITS)     :: rss_nav_orbit_valid_from_data	

		real(8),dimension(MX_ORBITS)        :: rss_nav_a_from_data
		real(8),dimension(MX_ORBITS)        :: rss_nav_a

		integer(4)                          :: rss_nav_max_orbit
		integer(4)                          :: rss_nav_min_orbit
		integer(4)                          :: rss_nav_satnum_loaded = 0  

		
contains


;@***h* MLT_V4_RSS/read_MSU_NOAA_nav_file
; NAME
;	read_MSU_NOAA_nav_file
;
; PURPOSE
;	reads NOAA MSU navigation file
;
; DESCRIPTION
;	reads NOAA MSU navigation file
;;
; INPUTS
;	sat_num	integer(4)		satellite number
;
; OUTPUTS
;	error		integer(4)		error flag.  0 means OK, -1 means error
;	data from file is loaded into module data structures.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call read_MSU_NOAA_nav_file(sat_num,error)
;
;@*****

	subroutine read_MSU_NOAA_nav_file(sat_num,error)

		
		integer(4)						:: sat_num
		integer(4)						:: error

		integer(4)						:: j
		integer(4)						:: date_err
		integer(4)						:: io_error

		character(len=100)				:: xtime_file
		
		character(len = 50)             :: xtime_path = 'M:\MSU_AMSU_Data\MSU_crossing_times\'
		character(len = 3),dimension(NUM_SATS) :: sat_name_short = (/'tn ','n6 ','n7 ','n8 ','n9 ','n10','n11','n12','n14'/)
		
		integer(4)                      :: sat
		integer(4)                      :: orbit
		integer(4)                      :: year
		integer(4)                      :: month
		integer(4)                      :: day
		integer(4)                      :: hour
		integer(4)                      :: minute
		real(8)                         :: second
		integer(4)                      :: xlon_in
		real(8)                         :: a_in
		integer(4)                      :: valid_in
		
		type(date_str)                  :: crossing_date
		
		noaa_nav_eqx_time      = NAN
		noaa_nav_eqx_longitude = NAN
		noaa_nav_orbit_valid   = 0
		
		noaa_nav_min_orbit = 1000000
		
		! make the file name

		write(xtime_file,111) trim(xtime_path),trim(sat_names(sat_num)),trim(sat_name_short(sat_num))
111		format(a,a,'\',a,'ephem.noaa.txt')

		! open the file

		open(unit = 3, file = xtime_file,iostat = io_error,status = 'old')
		
		error = 0
		if(io_error .ne. 0) then
			print *,'Fatal Error -- could not open NOAA crossing time file'
			print *,xtime_file
			error = -1
			return
		endif
		
	    do j = 1,MX_ORBITS
		  read(3,888,end=999) sat,orbit,year,month,day,hour,minute,second,xlon_in,a_in,valid_in
	888			format(a2,2x,i5,11x,i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f6.3,1x,i6,1x,f18.11,1x,i1)
	
          
		  if ((orbit .gt. 0) .and. (orbit .le. MX_ORBITS)) then
            crossing_date = set_date(month,day,year,hour,minute,second,date_err)
            if (date_err .eq. 0) then
                noaa_nav_eqx_time(orbit) = crossing_date%rd_2000
                if (xlon_in .eq. -99999) then
                    noaa_nav_eqx_longitude(orbit) = NAN
                else
                    noaa_nav_eqx_longitude(orbit) = modulo((720.0D0 + xlon_in/100.0),360.0D0)
                endif
                noaa_nav_orbit_valid(orbit) = valid_in
                noaa_nav_a(orbit) = a_in
                if (orbit .lt. noaa_nav_min_orbit) noaa_nav_min_orbit = orbit
			endif
		  endif
		enddo
		
   999  write(6,222) j,trim(xtime_file)
  222   format('Read ',i6,' Lines from NOAA NAvigation File ',a)
        close(3)
        
        noaa_nav_max_orbit = orbit
        noaa_nav_satnum_loaded = sat_num
   end subroutine read_MSU_NOAA_nav_file
   
;@***h* MLT_V4_RSS/read_MSU_extended_nav_file
; NAME
;	read_MSU_extended_nav_file
;
; PURPOSE
;	reads "extended" MSU navigation file
;
; DESCRIPTION
;	reads "extended" MSU navigation file
;;
; INPUTS
;	sat_num	integer(4)		satellite number
;
; OUTPUTS
;	error		integer(4)		error flag.  0 means OK, -1 means error
;	data from file is loaded into module data structures.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call read_MSU_extended_nav_file(sat_num,error)
;
;@*****
   subroutine read_MSU_extended_nav_file(sat_num,error)
		
		integer(4)						:: sat_num
		integer(4)						:: max_orbit
		integer(4)						:: error

		integer(4)						:: j
		integer(4)						:: date_err
		integer(4)						:: io_error

		character(len=100)				:: xtime_file
	
		character(len = 50)             :: xtime_path = 'M:\MSU_AMSU_Data\MSU_crossing_times\'
		character(len = 3),dimension(NUM_SATS) :: sat_name_short = (/'tn ','n6 ','n7 ','n8 ','n9 ','n10','n11','n12','n14'/)
		
		integer(4)                      :: sat
		integer(4)                      :: orbit
		integer(4)                      :: year
		integer(4)                      :: month
		integer(4)                      :: day
		integer(4)                      :: hour
		integer(4)                      :: minute
		real(8)                         :: second
		integer(4)                      :: xlon_in
		real(8)                         :: a_in
		integer(4)                      :: valid_in
		
		type(date_str)                  :: crossing_date
		
		extended_nav_eqx_time      = NAN
		extended_nav_eqx_longitude = NAN
		extended_nav_orbit_valid   = 0
		
		extended_nav_min_orbit = 1000000
		
		! make the file name

		write(xtime_file,111) trim(xtime_path),trim(sat_names(sat_num)),trim(sat_name_short(sat_num))
111		format(a,a,'\',a,'ephem.extended.txt')

		! open the file

		open(unit = 3, file = xtime_file,iostat = io_error,status = 'old')
			
		if(io_error .ne. 0) then
			print *,'Error -- could not open extended crossing time file'
			print *,xtime_file
			error = -1
			return
		endif
		
	    do j = 1,MX_ORBITS
		  read(3,888,end=999) sat,orbit,year,month,day,hour,minute,second,xlon_in,a_in,valid_in
	888			format(a2,2x,i5,11x,i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f6.3,1x,i6,1x,f18.11,1x,i1)
	
          
		  if ((orbit .gt. 0) .and. (orbit .le. MX_ORBITS)) then
		    if (orbit .eq. 12497) then
		        print *,' '
		    endif
		        
            crossing_date = set_date(month,day,year,hour,minute,second,date_err)
            if (date_err .eq. 0) then
                extended_nav_eqx_time(orbit) = crossing_date%rd_2000
                if (xlon_in .eq. -99999) then
                    extended_nav_eqx_longitude(orbit) = NAN
                else
                    extended_nav_eqx_longitude(orbit) = modulo((720.0D0 + xlon_in/100.0),360.0D0)
                endif
                extended_nav_orbit_valid(orbit) = valid_in
                extended_nav_a(orbit) = a_in
                if (orbit .lt. extended_nav_min_orbit) extended_nav_min_orbit = orbit
			endif
		  endif
		enddo
   999  write(6,222) j,trim(xtime_file)
  222   format('Read ',i6,' Lines from extended navigation File ',a)
        max_orbit = orbit
        extended_nav_max_orbit = orbit
        extended_nav_satnum_loaded = sat_num
        close(3)
   end subroutine read_MSU_extended_nav_file
   
;@***h* MLT_V4_RSS/read_MSU_RSS_nav_file
; NAME
;	read_MSU_RSS_nav_file
;
; PURPOSE
;	reads RSS MSU navigation file
;
; DESCRIPTION
;	reads RSS MSU navigation file
;
; INPUTS
;	sat_num	integer(4)		satellite number
;
; OUTPUTS
;	error		integer(4)		error flag.  0 means OK, -1 means error
;	data from file is loaded into module data structures.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call read_MSU_RSS_nav_file(sat_num,error)
;
;@*****
   subroutine read_MSU_RSS_nav_file(sat_num,error)
		
		integer(4)						:: sat_num
		integer(4)						:: max_orbit
		integer(4)						:: error

		integer(4)						:: j
		integer(4)						:: date_err
		integer(4)						:: io_error

		character(len=100)				:: xtime_file
		
		character(len = 50)             :: xtime_path = 'M:\MSU_AMSU_Data\MSU_crossing_times\'
		character(len = 3),dimension(NUM_SATS) :: sat_name_short = (/'tn ','n6 ','n7 ','n8 ','n9 ','n10','n11','n12','n14'/)
		
		integer(4)                      :: sat
		integer(4)                      :: orbit
		integer(4)                      :: year
		integer(4)                      :: month
		integer(4)                      :: day
		integer(4)                      :: hour
		integer(4)                      :: minute
		real(8)                         :: second
		integer(4)                      :: xlon_in
		real(8)                         :: a_in
		integer(4)                      :: valid_in
		
		type(date_str)                  :: crossing_date
		
		rss_nav_eqx_time      = NAN
		rss_nav_eqx_longitude = NAN
		rss_nav_orbit_valid   = 0
		
		rss_nav_min_orbit = 1000000
		
		! make the file name

		write(xtime_file,111) trim(xtime_path),trim(sat_names(sat_num)),trim(sat_name_short(sat_num))
111		format(a,a,'\',a,'ephem.rss.txt')

		! open the file

		open(unit = 3, file = xtime_file,iostat = io_error,status = 'old')
			
		if(io_error .ne. 0) then
			print *,'Error -- could not open RSS crossing time file'
			print *,xtime_file
			error = -1
			return
		endif
		
	    do j = 1,MX_ORBITS
		  read(3,888,end=999) sat,orbit,year,month,day,hour,minute,second,xlon_in,a_in,valid_in
	888			format(a2,2x,i5,11x,i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f6.3,1x,i6,1x,f18.11,1x,i1)
	
          
		  if ((orbit .gt. 0) .and. (orbit .le. MX_ORBITS)) then
            crossing_date = set_date(month,day,year,hour,minute,second,date_err)
            if (date_err .eq. 0) then
                rss_nav_eqx_time(orbit) = crossing_date%rd_2000
                if (xlon_in .eq. -99999) then
                    rss_nav_eqx_longitude(orbit) = NAN
                else
                    rss_nav_eqx_longitude(orbit) = modulo((720.0D0 + xlon_in/100.0),360.0D0)
                endif
                rss_nav_orbit_valid(orbit) = valid_in
                rss_nav_a(orbit) = a_in
                if (orbit .lt. rss_nav_min_orbit) rss_nav_min_orbit = orbit
			endif
		  endif
		enddo
   999  write(6,222) j,trim(xtime_file)
  222   format('Read ',i6,' Lines from rss NAvigation File ',a)
        max_orbit = orbit
        rss_nav_max_orbit = orbit
        rss_nav_satnum_loaded = sat_num
   end subroutine read_MSU_rss_nav_file
   
;@***h* MLT_V4_RSS/write_MSU_RSS_nav_file
; NAME
;	write_MSU_RSS_nav_file
;
; PURPOSE
;	writes navigation data to RSS navigation file
;
; DESCRIPTION
;	writes navigation data to RSS navigation file
;
; INPUTS
;	sat_num	integer(4)		satellite number
;
; OUTPUTS
;	error		integer(4)		error flag.  0 means OK, -1 means error
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call write_MSU_RSS_nav_file(sat_num,error)
;
;@*****
   subroutine write_MSU_RSS_nav_file(sat_num,error)
		
		integer(4)						:: sat_num
		integer(4)						:: error

		integer(4)						:: j
		integer(4)						:: io_error

		character(len=100)				:: xtime_file
	
		character(len = 50)             :: xtime_path = 'M:\MSU_AMSU_Data\MSU_crossing_times\'
		character(len = 3),dimension(NUM_SATS) :: sat_name_short = (/'tn ','n6 ','n7 ','n8 ','n9 ','n10','n11','n12','n14'/)	
		integer(4)                      :: xlon_out
		
		type(date_str)                  :: crossing_date
		
		integer(4)                      :: num_written
				
		! make the file name

		write(xtime_file,111) trim(xtime_path),trim(sat_names(sat_num)),trim(sat_name_short(sat_num))
111		format(a,a,'\',a,'ephem.rss.txt')

		! open the file

		open(unit = 3, file = xtime_file,iostat = io_error)
			
		if(io_error .ne. 0) then
			print *,'Fatal Error -- could not open rss crossing time file'
			print *,xtime_file
			error = -1
			stop
		endif
		num_written = 0
	    do j = 1,MX_ORBITS
	      if (rss_nav_orbit_valid_from_data(j) .gt. 0) then
	        crossing_date = set_date_2000(rss_nav_eqx_time_from_data(j),error)
	        if (finite(rss_nav_eqx_longitude_from_data(j))) then
	            if (abs(rss_nav_eqx_longitude_from_data(j)) .gt. 0.000001) then
	                xlon_out = nint(rss_nav_eqx_longitude_from_data(j)*100.0)
	                if (xlon_out .gt. 18000) xlon_out = xlon_out - 36000
	            else
	                xlon_out = -99999
	            endif
	        else
	            xlon_out = -99999
	        endif
	        
		    write(3,888) sat_name_short(sat_num),j,crossing_date%year,crossing_date%month,crossing_date%day,crossing_date%time%hour,crossing_date%time%minute,crossing_date%time%sec,xlon_out,rss_nav_a_from_data(j),rss_nav_orbit_valid_from_data(j)
	888			format(a3,'|',i5,'|ephemeris|',i4.4,'-',i2.2,'-',i2.2,1x,i2.2,':',i2.2,':',f6.3,'|',i6,'|',f18.11,'|',i1)
            num_written = num_written + 1
		  endif
		enddo
   999  write(6,222) num_written,trim(xtime_file)
  222   format('Wrote ',i6,' Lines to RSS Navigation File ',a)
        close(6)
   end subroutine write_MSU_rss_nav_file

;@***h* MLT_V4_RSS/previous_valid_orbit
; NAME
;	previous_valid_orbit
;
; PURPOSE
;	finds the previous valid orbit before the orbit passed
;
; DESCRIPTION
;	finds the previous valid orbit before the orbit passed
;
; INPUTS
;	orbit_index		integer(4)		orbit number passed to routine
;
; OUTPUTS
;	error			integer(4)		error flag, 0 means OK, -1 means error occured
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	previous_orbit_num = previous_valid_orbit(orbit_index,error)
;
;@*****
   
   integer(4) function previous_valid_orbit(orbit_index,error)
   
        integer(4)  :: orbit_index
        integer(4)  :: error
        
        logical     :: found
        integer(4)  :: orbit_index_test
        
        error = 0
        found = .false.
        
        orbit_index_test = orbit_index
        
        do while (.not. found)
            orbit_index_test = orbit_index_test - 1
            if (orbit_index_test .le. 0) then
                previous_valid_orbit = -1
                error = -1
                return
            endif
            if (noaa_nav_orbit_valid(orbit_index_test) .eq. 1) then
                found = .true.
            endif
        enddo
        
        previous_valid_orbit = orbit_index_test
        return
    end function previous_valid_orbit


;@***h* MLT_V4_RSS/next_valid_orbit
; NAME
;	next_valid_orbit
;
; PURPOSE
;	finds the next valid orbit following the orbit passed
;
; DESCRIPTION
;	finds the next valid orbit following the orbit passed
;
; INPUTS
;	orbit_index		integer(4)		orbit number passed to routine
;
; OUTPUTS
;	error			integer(4)		error flag, 0 means OK, -1 means error occured
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	next_orbit_num = next_valid_orbit(orbit_index,error)
;
;@*****
  
    integer(4) function next_valid_orbit(orbit_index,error)
   
        integer(4)  :: orbit_index
        integer(4)  :: error
        
        logical     :: found
        integer(4)  :: orbit_index_test
        
        error = 0
        found = .false.
        
        orbit_index_test = orbit_index
        
        do while (.not. found)
            orbit_index_test = orbit_index_test + 1
            if (orbit_index_test .gt. noaa_nav_max_orbit) then
                next_valid_orbit = noaa_nav_max_orbit+1
                error = -1
                return
            endif
            if (noaa_nav_orbit_valid(orbit_index_test) .ge. 1) then
                found = .true.
            endif
        enddo
        
        next_valid_orbit = orbit_index_test
        return
    end function next_valid_orbit
    
;@***h* MLT_V4_RSS/predict_next_crossing_time
; NAME
;	predict_next_crossing_time
;
; PURPOSE
;	predicts next crossing time and longitude based on the previous 100 orbits
;
; DESCRIPTION
;	predicts next (after the last orbit with current data) crossing time and longitude based on the previous 100 orbits
;	the predicted value is then appended to the navigation data
;
; INPUTS
;	none passed
;
; OUTPUTS
;	none passed
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call predict_next_crossing_time
;
;@*****
    subroutine predict_next_crossing_time
        
        real(8),dimension(100)  :: eqx_time_differences
        real(8),dimension(100)  :: eqx_long_differences
        real(8)                 :: eqx_time_diff
        real(8)                 :: tot_time_diff
        real(8)                 :: tot_long_diff
        integer(4)              :: orbit_index,start_orbit,end_orbit,num_good_differences
        
        real(8)                 :: predicted_time_diff
        real(8)                 :: predicted_long_diff
        
        start_orbit = noaa_nav_max_orbit-100
        if (start_orbit .lt. noaa_nav_min_orbit) start_orbit = noaa_nav_min_orbit
        end_orbit = noaa_nav_max_orbit -1
        num_good_differences = 0
        do orbit_index = start_orbit,end_orbit
            eqx_time_diff = noaa_nav_eqx_time(orbit_index+1) - noaa_nav_eqx_time(orbit_index)
            if (abs(eqx_time_diff - MSU_ORBIT_TIME) .lt. (MSU_ORBIT_TIME/20.0)) then
                num_good_differences = num_good_differences+1
                eqx_time_differences(num_good_differences) = eqx_time_diff
                eqx_long_differences(num_good_differences) = noaa_nav_eqx_longitude(orbit_index+1) - noaa_nav_eqx_longitude(orbit_index)
                if (eqx_long_differences(num_good_differences) .gt. 180.0) eqx_long_differences(num_good_differences) = eqx_long_differences(num_good_differences)- 360.0D0
            endif
        enddo
        tot_time_diff = 0.0D0
        tot_long_diff = 0.0D0
        do orbit_index = 1,num_good_differences
            tot_time_diff = tot_time_diff + eqx_time_differences(orbit_index)
            tot_long_diff = tot_long_diff + eqx_long_differences(orbit_index)
        enddo
        
        predicted_time_diff = tot_time_diff/num_good_differences
        predicted_long_diff = tot_long_diff/num_good_differences
        
        ! add the predicted orbit info:
        
        noaa_nav_max_orbit = noaa_nav_max_orbit + 1
        noaa_nav_eqx_time(noaa_nav_max_orbit)      = noaa_nav_eqx_time(noaa_nav_max_orbit-1) + predicted_time_diff
        noaa_nav_eqx_longitude(noaa_nav_max_orbit) = noaa_nav_eqx_longitude(noaa_nav_max_orbit-1) + predicted_long_diff
        noaa_nav_orbit_valid(noaa_nav_max_orbit)   = 4
        
        continue
        
    end subroutine predict_next_crossing_time
        
;@***h* MLT_V4_RSS/find_msu_orbit
; NAME
;	find_msu_orbit;
; PURPOSE
;	find MSU orbit number given a value for rd_2000 
;
; DESCRIPTION
;	find MSU orbit number given a value for rd_2000.  It is assumed that navigation data is already loaded 
;  	into noaa_nav_eqx_time and noaa_nav_eqx_time
;
; INPUTS
;	rd2000_to_test		real(8)		time for which to determine orbit number
;
; OUTPUTS
;	error				integer(4)		error flag.  0 means OK, -1 means error
;     return valeu		integer(4)		MSU orbit number corresponding to rd2000_to_test
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	orbit_number = function find_msu_orbit(rd2000_to_test,error)
;
;@*****

   integer(4) function find_msu_orbit(rd2000_to_test,error)
   
        real(8)    :: rd2000_to_test
        integer(4) :: error
        
        integer(4),save  :: i
        integer(4)       :: j,k
        
        find_msu_orbit = -999
        error = -1
        
        if (i .lt. 1) i = 1
        if (i .gt. noaa_nav_max_orbit-1) i = noaa_nav_max_orbit-1
        
        ! first check current orbit

        if (finite(noaa_nav_eqx_time(i))) then
            if ((rd2000_to_test .ge. noaa_nav_eqx_time(i)) .and. (rd2000_to_test .lt. noaa_nav_eqx_time(i+1))) then
                find_msu_orbit = i
                error = 0
                return
            endif
        endif
        
        ! check 20 orbits on each side
        
        j = i-20
        if (j .lt. 1) j = 1
        k = i+20
        if (k .gt. noaa_nav_max_orbit-1) k = noaa_nav_max_orbit-1
        do i = j,k
          if (finite(noaa_nav_eqx_time(i))) then
            if ((rd2000_to_test .ge. noaa_nav_eqx_time(i)) .and. (rd2000_to_test .lt. noaa_nav_eqx_time(i+1))) then
                find_msu_orbit = i
                error = 0
                return
            endif
          endif
        enddo
        
        ! ok, check entire list (this should not happen very often)
        
        do i = 1,noaa_nav_max_orbit-1
          if (finite(noaa_nav_eqx_time(i))) then
            if ((rd2000_to_test .ge. noaa_nav_eqx_time(i)) .and. (rd2000_to_test .lt. noaa_nav_eqx_time(i+1))) then
                find_msu_orbit = i
                error = 0
                return
            endif
          endif
        enddo
        
        return   ! if we get here, we did not find a match, and we return -999 and error = -1
	end function find_msu_orbit
end module MSU_Navigation

