;@***h* MLT_V4_RSS/assign_corrected_msu_height.pro
; NAME
; assign_corrected_msu_height
;
; PURPOSE
; assigns satellite height to scans based on data from a table
;
; DESCRIPTION
; Routine to fix errors in satellite height for a given satellite name, year, and month.  
; Heights are assigned to scans based on data from a table of satellite heights vs scan position and date.
;
; INPUTS
; satellite_name       string containing satellite name
; year_to_process      integer containing the year to process
; month_to_process     integer containing the month to process
; verbose              flag - set to a non-zero value for more reporting
;
; OUTPUTS
; None passed back
; The program modifies the MSU L1B file so that it contains the corrected height in its navigation data
;
; AUTHOR
; Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
; THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
; AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
; AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
; SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
; 5/10/2018 Initial Version prepared for NCEI
;
; USAGE
; assign_corrected_msu_height('NOAA-06',1981,1,verbose=1)
;
;@*****

pro assign_corrected_msu_height,satellite_name,year_to_process,month_to_process,verbose=verbose
    MSU_SCAN_TIME = 2.962963d-4
    if keyword_Set(verbose) eq 0 then verbose = 0
    log_unit = 4
    close,log_unit
    log_file = 'C:\Users\Mears\IDL_MSU_V2\MSU_v2_height_assign\height_assign_logs\' + $
                string(satellite_name,year_to_process,month_to_process,format = '(a,"_hgt_assign_log_",i4.4,"_",i2.2,".txt")')
    openw,log_unit,log_file
    
    savfile = 'C:\Users\mears\IDL_MSU_V2\MSU_v2_height_tables\height_tables\'+satellite_name+'\sat_height_table_'+satellite_name+'.sav'
    restore,savfile
    sz_h_table = size(h_table,/dimensions)
    for i = 237,sz_h_table[1]-1 do begin
      bad = where(finite(h_table[*,i]) eq 0,ct)
      if ct gt 0 then begin
        h_table[bad,i] = h_table[bad,i-1]
      endif
    endfor
    
            
    read_msu_L1Bv2_monthly_file,satellite_name,year_to_process,month_to_process,msu_L1bv2,level_string = 'R1',error
    if error ne 0 then begin
      print,'Number of RSSv2 scans:  0'
      printf,log_unit,'Number of RSSv2 scans:  0'
      return
    endif
    num_rssv2_points = n_elements(msu_L1bv2)
    print,'Number of RSSv2 scans:  ',num_rssv2_points
    printf,log_unit,'Number of RSSv2 scans:  ',num_rssv2_points

    h = fltarr(num_rssv2_points,2)
    for i = 0,num_rssv2_points-1 do begin
    ;for i = 0,9999 do begin

      rd2000 = msu_L1bv2[i].rd2000
      scan_num = msu_L1bv2[i].scan_number
      if scan_num lt sz_h_table[1]+1 then begin
        if scan_num ge sz_h_table[1] then scan_num = scan_num - 1
        index_lower = floor(rd2000) + 8000
        index_upper = index_lower + 1
        wt_upper = rd2000+8000 - index_lower
        height_from_table = (1.0 - wt_upper)*h_table[index_lower,scan_num] + wt_upper*h_table[index_upper,scan_num]
        ;print, msu_L1bv2[i].height,height_from_table
        h[i,0] = msu_L1bv2[i].height
        h[i,1] = height_from_table
        msu_L1bv2[i].height_corrected = height_from_table
        if finite(height_from_table) eq 0 then begin
           printf,log_unit,'Bad table height',index_lower,index_upper,scan_num
        endif
      endif else begin
        h[i,0] = msu_L1bv2[i].height
        h[i,1] = !values.f_nan
        printf,log_unit,'Scan Number out of range ',rd2000,scan_num
      endelse
    endfor
    
    ; write out file as level = "R3" file.
    write_msu_L1Bv2_monthly_file,satellite_name,year_to_process,month_to_process,msu_L1bv2,'R3',error
    
    bad_height = where(finite(h[*,0]) eq 0,num_bad_original)
    bad_corr_height = where(finite(h[*,1]) eq 0,num_bad_corrected)
    print,'There were '+string(num_bad_original,format='(i6)')+' bad heights in R1 file'
    printf,log_unit,'There were '+string(num_bad_original,format='(i6)')+' bad heights in R1 file'
    print,'There were '+string(num_bad_corrected,format='(i6)')+' heights that could not be corrected'
    printf,log_unit,'There were '+string(num_bad_corrected,format='(i6)')+' heights that could not be corrected'
    close,log_unit  
end