
;@***h* MLT_V4_RSS/read_msu_1lbv4_monthly_file.pro
; This file contains msu_l1bv4 read code 
; There are 3 routines in this file
; 
;   init_msu_monthly_L1Bv4_data
;   decode_msu_l1bv4_orbit_file_scan
;   read_msu_L1Bv4_monthly_file
;   
; NAME
; init_msu_monthly_L1Bv2_data
;
; PURPOSE
; creates and initializes msul1b data structures with default values.
;
; DESCRIPTION
; creates and initializes msul1b data structures with default values.
;
; INPUTS
; number_of_swaths     integer containing number of scans to initialize
; y
;
; OUTPUTS
; msu_L1b        array of structures containing msu L1B data, initialized to default values.
;
; AUTHOR
; Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
; THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
; AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
; AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
; SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
; 5/10/2018 Initial Version prepared for NCEI
;
; USAGE
; init_msu_monthly_L1Bv2_data,900,msu_L1b
;
;@*****

pro init_msu_monthly_L1Bv2_data,number_of_swaths,msu_L1b

       msu_calib_coeffs = {msu_cal,            $                       
                        A0:replicate(!values.f_nan,4),   $
                        A1:replicate(!values.f_nan,4),   $
                        B0:replicate(!values.d_nan,4),   $
                        B1:replicate(!values.d_nan,4),   $
                        B2:replicate(!values.d_nan,4),   $
                        B3:replicate(!values.d_nan,4)}
        scan = {msu_L1B_scan,         $
            scan_number:-99L,             $  
            lyear:-99,           $
            jday:-99,            $    
            secdy:-99.0,          $         ; # of seconds since midnight for observation
            height:-99.0,         $         ; height, in kilometers wrt geoid
            height_corrected:-99.0, $       ; corrected for anavigation errors
            loc_zen_angle:0.0,    $         ; local zenith angle
            FOV_latitude:replicate(!values.f_nan,11),      $  ; longitude of each measurement
            FOV_longitude:replicate(!values.f_nan,11),     $  ; latitude of each measurement
            MSU_counts:replicate(-99,7,14),      $  ; Raw MSU counts, and other data
            set_coeff:-99,                $  ;
            earth_location_delta:-99.0,   $  ; Some sort of Earth location error (could be clock error)
            scan_quality:-99,             $  ;
            FOV_quality:replicate(-99,14),$  ;
            chksum:-9999L,                $  ;
            sat_num:-99L,                 $
            orbit_num:-99L,               $ 
            scan_number2:-99L,            $ 
            rd2000:-999999.0D0,           $
            cal:msu_calib_coeffs}
            
       msu_L1b = replicate(scan,number_of_swaths)
end
            
; NAME
; decode_msu_l1bv2_orbit_file_scan
;
; PURPOSE
; decodes and unpacks msu L1B files in an IDL structure
;
; DESCRIPTION
; decodes and unpacks msu L1B files in an IDL structure
;
; INPUTS
; scan_byte_arr     array of bytes containing msu L1B data in binary form 
;
; OUTPUTS
; scan             IDL structures containing msu L1B data from the binary scan
;
; AUTHOR
; Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
; THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
; AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
; AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
; SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
; 5/10/2018 Initial Version prepared for NCEI
;
; USAGE
; decode_msu_l1bv2_orbit_file_scan,scan_byte_arr,scan
;
;@*****         

pro decode_msu_l1bv2_orbit_file_scan,scan_byte_arr,    $
                                     scan

    ;character(len = 340)    :: scan_char
    ;type(msu_raw)           :: msu_orbit_scan

    ;integer(4)          :: fov
    ;integer(4)          :: offset
    ;integer(4)          :: chan
    ;integer(4)          :: year
    ;integer(4)          :: day
    ;integer(4)          :: error
    ;real(8)           :: msecs
    
    msu_calib_coeffs = {msu_cal,            $
                        A0:replicate(!values.f_nan,4),   $
                        A1:replicate(!values.f_nan,4),   $
                        B0:replicate(!values.d_nan,4),   $
                        B1:replicate(!values.d_nan,4),   $
                        B2:replicate(!values.d_nan,4),   $
                        B3:replicate(!values.d_nan,4)}
  
    scan = {msu_L1B_scan,         $
            scan_number:-99L,             $  
            lyear:-99L,           $
            jday:-99L,            $    
            secdy:-99.0,          $         ; # of seconds since midnight for observation
            height:-99.0,         $         ; height, in kilometers wrt geoid
            height_corrected:-99.0, $
            loc_zen_angle:0.0,    $         ; local zenith angle
            FOV_latitude:replicate(!values.f_nan,11),      $  ; longitude of each measurement
            FOV_longitude:replicate(!values.f_nan,11),     $  ; latitude of each measurement
            MSU_counts:replicate(-99,7,14),      $  ; Raw MSU counts, and other data
            set_coeff:-99,                $  ;
            earth_location_delta:-99.0,   $  ; Some sort of Earth location error (could be clock error)
            scan_quality:-99,             $  ;
            FOV_quality:replicate(-99,14),$  ;
            chksum:-9999L,                $  ;
            sat_num:-99L,                 $
            orbit_num:-99L,               $
            scan_number2:-99L,            $
            rd2000:-999999.0D0,           $
            cal:msu_calib_coeffs}


    scan.scan_number    = long(scan_byte_arr,0)
    scan.lyear      = fix(scan_byte_arr,4)
    scan.jday       = fix(scan_byte_arr,6)
    scan.secdy      = float(scan_byte_arr,8)
    scan.height     = float(scan_byte_arr,12)
    scan.height_corrected     = float(scan_byte_arr,16)
    scan.loc_zen_angle  = float(scan_byte_arr,20)

    
    day = scan.jday
    year = scan.lyear
    msecs = scan.secdy
    jday = julday(1,day,year,0,0,msecs)
    ok = convert_date(date_in = jday,from ='julday',to='rd_2000',date_out = rd2000)
    if ok eq 0 then scan.rd2000 = rd2000
    
    scan.fov_latitude   = float(scan_byte_arr,24,11)
    scan.fov_longitude  = (float(scan_byte_arr,68,11)+720.0) mod 360.0

    scan.MSU_counts = fix(scan_byte_arr,112,7,14)
    
    scan.set_coeff              = fix(scan_byte_arr,308)
    scan.earth_location_delta   = float(scan_byte_arr,310)
    scan.scan_quality           = fix(scan_byte_arr,314)

    scan.fov_quality = fix(scan_byte_arr,312,14)
    
    scan.chksum      = long(scan_byte_arr,344)
    scan.sat_num     = long(scan_byte_arr,348)
    scan.orbit_num   = long(scan_byte_arr,352)
    scan.scan_number2 = long(scan_byte_arr,356)
    rd2000_check     = double(scan_byte_arr,360)
    
    scan.cal.A0 = float(scan_byte_arr,368,4)
    scan.cal.A1 = float(scan_byte_arr,384,4)
    scan.cal.B0 = double(scan_byte_arr,400,4)
    scan.cal.B1 = double(scan_byte_arr,432,4)
    scan.cal.B2 = double(scan_byte_arr,464,4)
    scan.cal.B3 = double(scan_byte_arr,496,4)
   
end

; NAME
; read_msu_L1Bv2_monthly_file
;
; PURPOSE
; reads and decodes monthly msu L1B files into an array of IDL structures
;
; DESCRIPTION
; reads and decodes monthly msu L1B files into an array of IDL structures
;
; INPUTS
; satellite_name       string containing satellite name
; year_to_read         integer containing the year to process
; month_to_read        integer containing the month to process
; level_string         string containing "level" of the L1B data.  
;
; OUTPUTS
; msu_L1b             array of IDL structures containing msu L1B scans
;
; AUTHOR
; Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
; THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
; THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
; AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
; AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
; SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
; THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
; 5/10/2018 Initial Version prepared for NCEI
;
; USAGE
; read_msu_L1Bv2_monthly_file,scan_byte_arr,scan
;
;@*****

pro read_msu_L1Bv2_monthly_file,satellite_name,year_to_read,month_to_read,msu_L1b,level_string = level_string,error

    if keyword_Set(level_string) eq 0 then level_string = 'R1'
    MAX_SCANS_PER_Month=120000L

    ; find satellite_number

    for sat_num = 1,9 do begin
      if (sat_name(sat_num,/fill) eq satellite_name) then break
    endfor

    if (sat_num gt 9) then begin
      error = -1
      return
    endif

    error = 0

    MSU_L1B_PATH = 'M:\MSU_AMSU_Data\MSU_L1Bv2\'
    filename = string(strtrim(MSU_L1B_PATH),satellite_name,satellite_name,year_to_read,month_to_read,level_string, $
    format = '(a,a,"\monthly\",a,"_monthly_L1B_",i4.4,"_",i2.2,"_",a,".dat")')

  ; Determine if file exists

    info = file_info(filename)
    if info.exists then begin
      print,filename
      close,3
      openr,3,filename
      buffer = bytarr(528,MAX_SCANS_PER_MONTH)
      buffer_line = bytarr(528)
      on_ioerror,file_end
      j = 0L
      while 1 do begin
        readu,3,buffer_line
        buffer[*,j] = buffer_line
        j = j+1L
      endwhile
      file_end:
      on_ioerror,null
      num_scans = j-1
      ;init_msu_orbit_data,num_scans,msu_l1b
      init_msu_monthly_L1Bv2_data,num_scans,msu_L1b
      for j = 0,num_scans-1 do begin
        decode_msu_l1bv2_orbit_file_scan,buffer[*,j],scan
        msu_l1b[j] = scan
      endfor  
      error = 0   
    endif else begin
      error = -1
    endelse
  end 