
$freeform
;@***h* MLT_V4_RSS/incidence_angle
; NAME
;	incidence_angle
;
; PURPOSE
;	module containing incidence angle routines.
;
; DESCRIPTION
;	module containing incidence angle routines.
;	contains:
;		calc_EIA

; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	use incidence_angle
;
;@*****

module incidence_angle

	use NAN_support
	use msu_constants

	real(4),parameter,dimension(6)	::  NOM_EIA = (/0.0,10.71,21.51,32.51,43.91,56.19/)



contains

;@***h* MLT_V4_RSS/calc_EIA
; NAME
;	calc_EIA
;
; PURPOSE
;	calculates Earth incidence angle given FOV index, height and latitude.
;
; DESCRIPTION
;	calculates Earth incidence angle given FOV index, height and latitude.
;
; INPUTS
;	FOV_index		integer(4)	field of view index (1..11)
;	height		real(4)	satellite height
;	latitude		real(4)	latitude of satellite nadir point
;	roll_error		real(4)	additional roll error
;
; OUTPUTS
;	error			integer(4)	error flag: 0 means OK, -1 means error
;	return value	real(4)	Earth incidence angle
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	eia = calc_EIA(FOV_index,height,latitude,roll_error,error)
;
;@*****

	real(4) function calc_EIA(FOV_index,height,latitude,roll_error,error)

		!   calculates EIA tht for given nadir looking angle tht_nadir 
		!   at geodesic latitude xlat and height z above ground
		! 
		!   approximate the surface of the earth by a sphere with
		!   radius of curvature rho  
		!   (c.f. SSM/I User Manual, DEC1, 1991)

		implicit none

		integer(4),intent(IN)			:: FOV_index
		real(4),intent(IN)			:: height
		real(4),intent(IN)			:: latitude
		real(4),intent(IN)			:: roll_error
		integer(4),intent(OUT)			:: error


		real(4)					:: rho
		real(4)					:: tht_nadir
	
		real(4),parameter				:: rho0 = 6345.7
		real(4),parameter				:: a1 = 55.0 
	
		if (finite(latitude) .and. finite(height)) then
			rho = rho0 + a1*(sind(latitude))**2 	! earth radius of curvature

			tht_nadir = MSU_VIEW(abs(FOV_index-6)+1) + roll_error*sign(1,(6-FOV_index))
			if (abs(tht_nadir) .gt. 90) then
				error = 1
				calc_EIA = NAN
				return
			endif
			if (abs(sind(tht_nadir) * (height+rho)/rho) .lt. 1.0) then
				calc_EIA = abs(asind(sind(tht_nadir) * (height+rho)/rho))
				error = 0
			else
				calc_EIA = NAN
				error = 1
			endif
		else
			error = 1
			calc_EIA = NAN
		endif

		return

	end function calc_EIA
end module incidence_angle
