$freeform
module msu_orbital_and_roll_corrections

	! this module contains structures and routines that perform the orbital height
	! correction for the MSU series of satellites. 
	
	use msu_constants
	use NAN_support 
	use msu_types
	use landmask
	use maps
	use date_and_time
	use msu_date
	use interpolate

	implicit none

	! declarations for the array that contains the modeled Tb's as a function of
	! lat,lon,height,nadir angle, sea or land, and declaration of associated 
	! constants. 

	character(100),parameter	:: NCEP_MSU_TB_PATH  = 'M:\MSU_AMSU_Data\MSU_Adjustments\NCEP_Modelled_Tbs\'
	character(100),parameter	:: MERRA_MSU_TB_PATH = 'M:\MSU_AMSU_Data\MSU_Adjustments\MERRA_Modelled_Tbs\'

	real(4),parameter	::	MAX_TB  = 400.0
	real(4),parameter   ::	MIN_TB  = 100.0

	integer,parameter	::	NUM_LONS_MODEL		= 144
	integer,parameter	::	NUM_LATS_MODEL		= 73
	integer,parameter	::	NUM_THT			= 3  
	integer,parameter	::	NUM_FOV   			= 6
	integer,parameter	::	NUM_DATA			= 2
	real(4),parameter	::	NUM_PER_DEGREE		= 0.4
	real(4),parameter	::    REFERENCE_HEIGHT		= 800


	real(4),parameter,dimension(6)	::  ANGLE	= (/0.0, 9.47,18.94,28.41,37.88,47.35/)
	real(4),parameter,dimension(6)	::  NOM_EIA = (/0.0,10.71,21.51,32.51,43.91,56.19/)
	real(4),parameter				::  DELTA_EIA = 2.5


	real(4),parameter,dimension(6)  ::  V_POL_WTS = (/1.0,0.97292949,0.894649231,0.77363555,0.622992063,0.45903074/)
 	real(4),parameter,dimension(6)  ::  H_POL_WTS = (/0.0,0.02707051,0.105350769,0.22636445,0.377007937,0.54096926/)
																								 
	integer,parameter				::	TB_V		= 1
	integer,parameter				::    TB_H		= 2
	integer,parameter				::	EARTH_INC	= 3	

contains
;@***h* MLT_V4_RSS/Read_Mod_data_MERRA
; NAME
;	Read_Mod_data_MERRA
;
; PURPOSE
;	reads modelled data from MERRA for angle and roll corrections
;
; DESCRIPTION
;	reads modelled data from MERRA for angle and roll corrections.
;
;
; INPUTS
;	month		integer(4)	month for loaded data (1..12)
;	chan		integer(4)  channel for loaded data (1..4)
;
; OUTPUTS
;	Modelled_Tbs	type(map_str)	modelled Tbs from MERRA
;	error			integer(4)		error flag.  0 means OK, non-zero mean error during read.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call Read_Mod_data_MERRA(month,chan,Modelled_Tbs,error)
;
;@*****

	subroutine Read_Mod_data_MERRA(month,chan,Modelled_Tbs,error)

		integer(4),intent(IN)			:: month
		integer(4),intent(IN)			:: chan
		type(map_str),dimension(NUM_THT,NUM_FOV,NUM_SURFACES,NUM_CHANNELS,NUM_DATA)	::	Modelled_Tbs
		integer(4),intent(OUT)			:: error

		character(LEN = 120)			:: ocean_file
		character(LEN = 120)			:: land_file
		integer,parameter			    :: yy = 1990 
		integer,parameter               :: LU_TB = 41

		real(4),dimension(NUM_LATS_MODEL,NUM_LONS_MODEL,NUM_THT,NUM_FOV,NUM_DATA) :: temp_matrix

		integer(4)					:: i_tht
		integer(4)					:: i_fov
		integer(4)					:: i_dat
		
		!construct file names

		write(ocean_file,1041)trim(MERRA_MSU_TB_PATH),chan,month
1041	format(a,'merra_tbdata_CH',i1,'_S1_19901994_',i2.2,'_topo_theta.dat')
		open(unit=LU_TB,form='binary',file=ocean_file, iostat = error)
		print *,'opening file ',ocean_file

		if (error .eq. 0) then
			read(unit=LU_TB,iostat=error)temp_matrix
			if (error .eq. 0) then
				do i_tht = 1,NUM_THT
					do i_fov = 1, NUM_FOV
						do i_dat = 1, NUM_DATA
							modelled_Tbs(i_tht,i_fov,SEA,chan,i_dat)%dat = &
									transpose(temp_matrix(:,:,i_tht,i_fov,i_dat))
						enddo
					enddo
				enddo
			else
			write(6,*)     'Error reading ocean file for month ',month
			do i_tht = 1,NUM_THT
				do i_fov = 1, NUM_FOV
					do i_dat = 1, NUM_DATA
						modelled_Tbs(i_tht,i_fov,SEA,chan,i_dat)%dat = NAN
					enddo
				enddo
			enddo


			endif
			close(LU_TB)
		else
			write(6,*)'Error opening ocean file for month ',month
			do i_tht = 1,NUM_THT
				do i_fov = 1, NUM_FOV
					do i_dat = 1, NUM_DATA
						modelled_Tbs(i_tht,i_fov,SEA,chan,i_dat)%dat = NAN
					enddo
				enddo
			enddo
		endif


		write(land_file,1042)trim(MERRA_MSU_TB_PATH),chan,month
1042	format(a,'merra_tbdata_CH',i1,'_S2_19901994_',i2.2,'_topo_theta.dat')
		print *,'opening file ',land_file

		open(unit=LU_TB,form='binary',file=land_file, iostat = error)

		if (error .eq. 0) then
			read(unit=LU_TB,iostat=error)temp_matrix
			if (error .eq. 0) then

				do i_tht = 1,NUM_THT
					do i_fov = 1, NUM_FOV
						do i_dat = 1, NUM_DATA
							modelled_Tbs(i_tht,i_fov,LAND,chan,i_dat)%dat = &
									transpose(temp_matrix(:,:,i_tht,i_fov,i_dat))
						enddo
					enddo
				enddo

				!write(LOG_LU,*)'Successfully read land file for month ',month
				!write(6,*)'Successfully read land file for month ',month


			else
				write(6,*)'Error reading land file for month ',month

				do i_tht = 1,NUM_THT
					do i_fov = 1, NUM_FOV
						do i_dat = 1, NUM_DATA
							modelled_Tbs(i_tht,i_fov,LAND,chan,i_dat)%dat = NAN
						enddo
					enddo
				enddo

			endif
			close(LU_TB)

		else
			write(6,*)'Error opening land file for month ',month

			do i_tht = 1,NUM_THT
				do i_fov = 1, NUM_FOV
					do i_dat = 1, NUM_DATA
						modelled_Tbs(i_tht,i_fov,LAND,chan,i_dat)%dat = NAN
					enddo
				enddo
			enddo

		endif


	end subroutine Read_Mod_data_MERRA
	
	
;@***h* MLT_V4_RSS/Read_Mod_data
; NAME
;	Read_Mod_data
;
; PURPOSE
;	reads modelled data from NCEP for angle and roll corrections
;
; DESCRIPTION
;	reads modelled data from NCEP for angle and roll corrections.
;
;
; INPUTS
;	month		integer(4)	month for loaded data (1..12)
;	chan		integer(4)  channel for loaded data (1..4)
;
; OUTPUTS
;	Modelled_Tbs	type(map_str)	modelled Tbs from MERRA
;	error			integer(4)		error flag.  0 means OK, non-zero mean error during read.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call Read_Mod_data(month,chan,Modelled_Tbs,error)
;
;@*****

	subroutine Read_Mod_data(month,chan,Modelled_Tbs,error)

		integer(4),intent(IN)			:: month
		integer(4),intent(IN)			:: chan
		type(map_str),dimension(NUM_THT,NUM_FOV,NUM_SURFACES,NUM_CHANNELS,NUM_DATA)	::	Modelled_Tbs
		integer(4),intent(OUT)			:: error

		character(LEN = 120)			:: ocean_file
		character(LEN = 120)			:: land_file
		integer,parameter			    :: yy = 1996 
		integer,parameter               :: LU_TB = 41

		real(4),dimension(NUM_LATS_MODEL,NUM_LONS_MODEL,NUM_THT,NUM_FOV,NUM_DATA) :: temp_matrix

		integer(4)					:: i_tht
		integer(4)					:: i_fov
		integer(4)					:: i_dat
		
		!construct file names

		write(ocean_file,1041)trim(NCEP_MSU_TB_PATH),chan,yy,month
1041	format(a,'tbdata_CH',i1,'_S1_',i4.4,'_',i2.2,'_topo_theta.dat')
		open(unit=LU_TB,form='binary',file=ocean_file, iostat = error)

		if (error .eq. 0) then
			read(unit=LU_TB,iostat=error)temp_matrix
			if (error .eq. 0) then
				do i_tht = 1,NUM_THT
					do i_fov = 1, NUM_FOV
						do i_dat = 1, NUM_DATA
							modelled_Tbs(i_tht,i_fov,SEA,chan,i_dat)%dat = &
									transpose(temp_matrix(:,:,i_tht,i_fov,i_dat))
						enddo
					enddo
				enddo
			else
			write(6,*)     'Error reading ocean file for month ',month
			do i_tht = 1,NUM_THT
				do i_fov = 1, NUM_FOV
					do i_dat = 1, NUM_DATA
						modelled_Tbs(i_tht,i_fov,SEA,chan,i_dat)%dat = NAN
					enddo
				enddo
			enddo


			endif
			close(LU_TB)
		else
			write(6,*)'Error opening ocean file for month ',month
			do i_tht = 1,NUM_THT
				do i_fov = 1, NUM_FOV
					do i_dat = 1, NUM_DATA
						modelled_Tbs(i_tht,i_fov,SEA,chan,i_dat)%dat = NAN
					enddo
				enddo
			enddo
		endif


		write(land_file,1042)trim(NCEP_MSU_TB_PATH),chan,yy,month
1042	format(a,'tbdata_CH',i1,'_S2_',i4.4,'_',i2.2,'_topo_theta.dat')
		!print *,'opening file ',land_file

		open(unit=LU_TB,form='binary',file=land_file, iostat = error)

		if (error .eq. 0) then
			read(unit=LU_TB,iostat=error)temp_matrix
			if (error .eq. 0) then

				do i_tht = 1,NUM_THT
					do i_fov = 1, NUM_FOV
						do i_dat = 1, NUM_DATA
							modelled_Tbs(i_tht,i_fov,LAND,chan,i_dat)%dat = &
									transpose(temp_matrix(:,:,i_tht,i_fov,i_dat))
						enddo
					enddo
				enddo

				!write(LOG_LU,*)'Successfully read land file for month ',month
				!write(6,*)'Successfully read land file for month ',month


			else
				write(6,*)'Error reading land file for month ',month

				do i_tht = 1,NUM_THT
					do i_fov = 1, NUM_FOV
						do i_dat = 1, NUM_DATA
							modelled_Tbs(i_tht,i_fov,LAND,chan,i_dat)%dat = NAN
						enddo
					enddo
				enddo

			endif
			close(LU_TB)

		else
			write(6,*)'Error opening land file for month ',month

			do i_tht = 1,NUM_THT
				do i_fov = 1, NUM_FOV
					do i_dat = 1, NUM_DATA
						modelled_Tbs(i_tht,i_fov,LAND,chan,i_dat)%dat = NAN
					enddo
				enddo
			enddo

		endif


	end subroutine Read_Mod_data

;@***h* MLT_V4_RSS/find_modelled_Tb_theta
; NAME
;	find_modelled_Tb_theta
;
; PURPOSE
;	calculates modelled TB as a function of location,channel, theta and model name
;
; DESCRIPTION
;	calculates modelled TB as a function of location,channel, theta and model name
;	linear interpolation is used to find coefficients of a taylor expansion about the nominal theta
;	then the actual theta is plugged into the taylor expansion to retreive the desired value.
;
;
; INPUTS
;	lat			real(4)		latitude
;	lon			real(4)		longitude
;	month			integer(4)		month to use
;	land_sea		integer(4)		land or sea
;	theta			real(4)		earth incidence angle
;	FOV_index		integer(4)		FOV index of closest nominal EIA
;	chan			integer(4)		channel
;	model_to_use	string		model name to use
;
; OUTPUTS
;	return value	real(4)		modelled brightness temperature
;	error			integer(4)		error flag.  0 means OK, non-zero mean error during read.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	Tb = function find_modelled_Tb_theta(lat,lon,month,land_sea,theta,FOV_index,chan,error,model_to_use)
;
;@*****



	real(4) function find_modelled_Tb_theta(lat,lon,month,land_sea,theta,FOV_index,chan,error,model_to_use)

		! this function interpolates tables of modelled Tb's to supplied height.
		! lat and long are not interpolated.

		implicit none
		
		real(4),intent(IN)			::		lat
		real(4),intent(IN)			::		lon
		integer(4),intent(IN)		::		month
		integer(4),intent(IN)		::		land_sea					   
		real(4),intent(IN)			::		theta
		integer(4),intent(IN)		::		FOV_index
		integer(4),intent(IN)		::		chan
		integer(4),intent(OUT)		::		error
		character(10),intent(IN),OPTIONAL :: model_to_use
		
		    

		type(map_str),save,dimension(NUM_THT,NUM_FOV,NUM_SURFACES,NUM_CHANNELS,NUM_DATA)	::	Modelled_Tbs

		integer(4),dimension(4)		::		month_loaded=(/0,0,0,0/)
		logical(4)					::		Mod_Tbs_allocated = .false.

		real(4)						::		delta_theta  ! difference between theta and nominal value for FOV
		real(4)						::		V_Pol_wt
		real(4)						::		H_Pol_wt
		real(4)						::		Tb_V_Pol
		real(4)						::		Tb_H_Pol
		integer(4)					::      angle_index

		real(4)						::		Tb_V_0,Tb_V_1,Tb_V_2
		real(4)						::		Tb_H_0,Tb_H_1,Tb_H_2
		
		character(10)               ::      model
		character(10),save          ::      model_loaded
		
		
		
		model = 'ncep'
		
		if (present(model_to_use)) then
		    select case (trim(model_to_use))
		        case('MERRA','Merra','merra')
		            model = 'merra'
		        case('NCEP','Ncep','ncep')
		           model = 'ncep'
		        case default
		           print *,'Angle Model Specification Error'
		           print *,model_to_use
		           print *,'Using NCEP'
		           model = 'ncep'
		    end select
		 endif

		! check to make sure array of modelled Tb maps are allocated -- if not, allocate them

		if (.not.(Mod_Tbs_allocated)) then

			call allocate_Mod_Tbs(Modelled_Tbs,Error)

			if (error .ne. 0) then
				write(6,*)' Error Allocating space for Modelled Tb Arrays -- Suggest Aborting'
				return
			else
				write(6,*)' Allocated space for Modelled Tb Arrays'
				Mod_Tbs_allocated = .true.
			endif
		endif
		
		! check for correct month, and load Tb file if necessary

		if ((month .ne. month_loaded(chan)) .or. (model .ne. model_loaded)) then
		!	write(LOG_LU,*)'Reading Modelled Tbs for month ',month,', Channel ',chan
			
			select case (trim(model))
			    case('ncep') 
			        call Read_Mod_data(month,chan,Modelled_Tbs,error)
			    case('merra')
			        call Read_Mod_data_merra(month,chan,Modelled_Tbs,error)
			    case default
			        print *,'Invalid angle model'
			        print *,model
			        stop
			end select
			
			

			if (error .eq. 0) then
				month_loaded(chan) = month
			else
				print *,' Unable to read Modelled Tb Arr for Channel ',chan,' model: ',model
				print *,' Suggest aborting '	
				stop	
			endif
		endif
		
		model_loaded = model

		! check to make sure that the lat, lon, and height and roll error are finite

		if (.not.((finite(lat) .and. finite(lon) .and. finite(theta))))then
			error = 1
			Find_Modelled_Tb_theta = NAN
			return
		endif

		! Calculate Polarization Weighting Coefficients)

			! set roll error to 0.0 here.  -- this means I'm assuming that the
			! error is due to a misalighment of the entire instrument -- i.e.
			! the polarization is pure v-pol for FOV 6 even though the intrument is rolled
			! from nadir

		call calc_pol_wts(FOV_index,0.0,V_Pol_wt,H_Pol_wt,error)

		! get polynomial coefficients for this location

		angle_index = iabs(fov_index-6)+1

		call get_map_value(Modelled_Tbs(1,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_0,error)
		call get_map_value(Modelled_Tbs(2,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_1,error)
		call get_map_value(Modelled_Tbs(3,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_2,error)

		call get_map_value(Modelled_Tbs(1,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_0,error)
		call get_map_value(Modelled_Tbs(2,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_1,error)
		call get_map_value(Modelled_Tbs(3,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_2,error)

		delta_theta = theta - NOM_EIA(angle_index)

		if (delta_theta .gt. 4.0) then
			write(6,*)      ' Warning:  Delta Theta = ',delta_theta,' too large in modelled Tb calculation'
		endif

		Tb_V_Pol = Tb_V_0 + (Tb_V_1*delta_theta) + (Tb_V_2 * 0.5 * (delta_theta**2))
		Tb_H_Pol = Tb_H_0 + (Tb_H_1*delta_theta) + (Tb_H_2 * 0.5 * (delta_theta**2))

		find_modelled_Tb_theta = (V_Pol_wt*Tb_V_Pol) + (H_Pol_wt*Tb_H_Pol)

		return

	end function find_modelled_Tb_theta


;@***h* MLT_V4_RSS/find_modelled_Tb_theta_date_interp
; NAME
;	find_modelled_Tb_theta_date_interp
;
; PURPOSE
;	calculates modelled TB as a function of location,channel, theta and model name and date
;
; DESCRIPTION
;	calculates modelled TB as a function of location,channel, theta and model name
;	linear interpolation is used to find coefficients of a taylor expansion about the nominal theta
;	then the actual theta is plugged into the taylor expansion to retreive the desired value.
;  	values are interpolated in time to correspond to the supplied date
;
;
; INPUTS
;	lat			real(4)		latitude
;	lon			real(4)		longitude
;	date			date_Str		date to interpolate to
;	land_sea		integer(4)		land or sea
;	theta			real(4)		earth incidence angle
;	FOV_index		integer(4)		FOV index of closest nominal EIA
;	chan			integer(4)		channel
;	model_to_use	string		model name to use
;
; OUTPUTS
;	return value	real(4)		modelled brightness temperature
;	error			integer(4)		error flag.  0 means OK, non-zero mean error during read.
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	Tb = function find_modelled_Tb_theta_date_interp(lat,lon,month,land_sea,theta,FOV_index,chan,error,model_to_use)
;
;@*****

	real(4) function find_modelled_Tb_theta_date_interp(lat,lon,date,land_sea,theta,FOV_index,chan,error,model_to_use)

		! this function interpolates tables of modelled Tb's to supplied.
		! lat and long and the date are interpolated.

		implicit none
		
		real(4),intent(IN)		::		lat
		real(4),intent(IN)		::		lon
		type(date_Str),intent(IN)	::		date
		integer(4),intent(IN)		::		land_sea					   
		real(4),intent(IN)		::		theta
		integer(4),intent(IN)		::		FOV_index
		integer(4),intent(IN)		::		chan
		integer(4),intent(OUT)		::		error	
	      character(10),intent(IN),OPTIONAL :: model_to_use

		type(map_str),save,dimension(NUM_THT,NUM_FOV,NUM_SURFACES,NUM_CHANNELS,NUM_DATA)	::	Modelled_Tbs_1
		type(map_str),save,dimension(NUM_THT,NUM_FOV,NUM_SURFACES,NUM_CHANNELS,NUM_DATA)	::	Modelled_Tbs_2

		integer(4),dimension(4)		::		month_loaded_1=(/0,0,0,0/)
		integer(4),dimension(4)		::		month_loaded_2=(/0,0,0,0/)
		logical(4)					::		Mod_Tbs_allocated = .false.

		integer(4)					::		month_1,month_1_load,month_2,month_2_load

		real(4)						::		delta_theta  ! difference between theta and nominal value for FOV
		real(4)						::		V_Pol_wt
		real(4)						::		H_Pol_wt
		real(4)						::		Tb_V_Pol
		real(4)						::		Tb_H_Pol
		integer(4)					::      angle_index

		real(4)						::		Tb_V_0,Tb_V_1,Tb_V_2
		real(4)						::		Tb_H_0,Tb_H_1,Tb_H_2
		real(4)						::		Modelled_Tb_1,Modelled_Tb_2
		real(4)						::		nomeia
		integer(4)					::		leap
		
		character(10)               ::      model
		character(10),save          ::      model_loaded
		


        model = 'ncep'
		
		if (present(model_to_use)) then
		    select case (trim(model_to_use))
		        case('MERRA','Merra','merra')
		            model = 'merra'
		        case('NCEP','Ncep','ncep')
		           model = 'ncep'
		        case default
		           print *,'Angle Model Specification Error'
		           print *,model_to_use
		           print *,'Using NCEP'
		           model = 'ncep'
		    end select
		 endif

		! check to make sure array of modelled Tb maps are allocated -- if not, allocate them

		if (.not.(Mod_Tbs_allocated)) then

			call allocate_Mod_Tbs(Modelled_Tbs_1,Error)
			call allocate_Mod_Tbs(Modelled_Tbs_2,Error)

			if (error .ne. 0) then
				write(6,*)' Error Allocating space for Modelled Tb Arrays -- Suggest Aborting'
				return
			else
				!write(6,*)' Allocated space for Modelled Tb Arrays'
				Mod_Tbs_allocated = .true.
			endif
		endif

		! find the two months that need to be loaded for this data

		call find_bracket_months(date,month_1,month_2)

		month_1_load = month_1
		month_2_load = month_2

		if (month_1 .eq. 0)  month_1_load = 12
		if (month_2 .eq. 13) month_2_load = 1

		! check for correct months loaded, and load Tb file if necessary


		if ((month_1_load .ne. month_loaded_1(chan)) .or. &
		    (month_2_load .ne. month_loaded_2(chan)) .or. &
		    (model .ne. model_loaded)) then
	
		     ! check for correct month, and load Tb file if necessary
		    select case (trim(model))
			    case('ncep') 
			        call Read_Mod_data(month_1_load,chan,Modelled_Tbs_1,error)
			        if (error .eq. 0) then
				        month_loaded_1(chan) = month_1_load
			        else
				        print *,' Unable to read Modelled Tb Arr for Channel ',chan
				        print *,' aborting '
				        stop		
			        endif
			        call Read_Mod_data(month_2_load,chan,Modelled_Tbs_2,error)
			        if (error .eq. 0) then
				        month_loaded_2(chan) = month_2_load
			        else
				        print *,' Unable to read Modelled Tb Arr for Channel ',chan
				        print *,' aborting '
				        stop		
			        endif
			    case('merra')
			        call Read_Mod_data_merra(month_1_load,chan,Modelled_Tbs_1,error)
			        if (error .eq. 0) then
				        month_loaded_1(chan) = month_1_load
			        else
				        print *,' Unable to read Modelled Tb Arr for Channel ',chan
				        print *,' aborting '
				        stop		
			        endif
			        call Read_Mod_data_merra(month_2_load,chan,Modelled_Tbs_2,error)
			        if (error .eq. 0) then
				        month_loaded_2(chan) = month_2_load
			        else
				        print *,' Unable to read Modelled Tb Arr for Channel ',chan
				        print *,' aborting '
				        stop		
			        endif
			    case default
			        print *,'Invalid angle model'
			        print *,model
			        stop
			end select

			

		endif

		! check to make sure that the lat, lon, and height and roll error are finite

		if (.not.((finite(lat) .and. finite(lon) .and. finite(theta))))then
			error = 1
			Find_Modelled_Tb_theta_date_interp = NAN
			return
		endif
		
		model_loaded = model

		! Calculate Polarization Weighting Coefficients)

			! set roll error to 0.0 here.  -- this means I'm assuming that the
			! error is due to a misalighment of the entire instrument -- i.e.
			! the polarization is pure v-pol for FOV 6 even though the intrument is rolled
			! from nadir

		call calc_pol_wts(FOV_index,0.0,V_Pol_wt,H_Pol_wt,error)
		angle_index = iabs(fov_index-6)+1

		! get polynomial coefficients for this location, month_1

		call get_map_value(Modelled_Tbs_1(1,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_0,error)
		call get_map_value(Modelled_Tbs_1(2,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_1,error)
		call get_map_value(Modelled_Tbs_1(3,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_2,error)

		call get_map_value(Modelled_Tbs_1(1,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_0,error)
		call get_map_value(Modelled_Tbs_1(2,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_1,error)
		call get_map_value(Modelled_Tbs_1(3,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_2,error)

		nomeia =  NOM_EIA(angle_index)
		delta_theta = theta - NOM_EIA(angle_index)

		if (delta_theta .gt. 4.0) then
			write(6,*)      ' Warning:  Delta Theta = ',delta_theta,' too large in modelled Tb calculation'
			write(6,*) date%rd_2000
		
		endif

		Tb_V_Pol = Tb_V_0 + (Tb_V_1*delta_theta) + (Tb_V_2 * 0.5 * (delta_theta**2))
		Tb_H_Pol = Tb_H_0 + (Tb_H_1*delta_theta) + (Tb_H_2 * 0.5 * (delta_theta**2))

		modelled_Tb_1 = (V_Pol_wt*Tb_V_Pol) + (H_Pol_wt*Tb_H_Pol)
		
		! get polynomial coefficients for this location, month_2

		call get_map_value(Modelled_Tbs_2(1,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_0,error)
		call get_map_value(Modelled_Tbs_2(2,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_1,error)
		call get_map_value(Modelled_Tbs_2(3,angle_index,land_sea,chan,TB_V),lon,lat,Tb_V_2,error)

		call get_map_value(Modelled_Tbs_2(1,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_0,error)
		call get_map_value(Modelled_Tbs_2(2,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_1,error)
		call get_map_value(Modelled_Tbs_2(3,angle_index,land_sea,chan,TB_H),lon,lat,Tb_H_2,error)

		delta_theta = theta - NOM_EIA(angle_index)

		if (delta_theta .gt. 4.0) then
			write(6,*)      ' Warning:  Delta Theta = ',delta_theta,' too large in modelled Tb calculation'
			write(6,*) date%rd_2000
		
		endif

		Tb_V_Pol = Tb_V_0 + (Tb_V_1*delta_theta) + (Tb_V_2 * 0.5 * (delta_theta**2))
		Tb_H_Pol = Tb_H_0 + (Tb_H_1*delta_theta) + (Tb_H_2 * 0.5 * (delta_theta**2))

		modelled_Tb_2 = (V_Pol_wt*Tb_V_Pol) + (H_Pol_wt*Tb_H_Pol)
		

		leap = 1
		if (gregorian_leap_year(date%year)) leap = 2

		find_modelled_Tb_theta_date_interp = lin_interpolate_r8_r4(dble(date%oday-1 + date%time%m_secs/86400.0),       &
		                                                            dble(month_centers(month_1,leap)),dble(month_centers(month_2,leap)), &
																	modelled_Tb_1,modelled_Tb_2,error)

		return

	end function find_modelled_Tb_theta_date_interp



;@***h* MLT_V4_RSS/correct_to_nom_EIA_and_Nadir
; NAME
;	correct_to_nom_EIA_and_Nadir
;
; PURPOSE
;	adjusts the measured brightness temperature to correspond to nadir, and to the nominal EIA for each FOV
;
; DESCRIPTION
;	adjusts the measured brightness temperature to correspond to nadir, and to the nominal EIA for each FOV
;
; INPUTS
;	L2A_scan		type(msu_L2A_scn)		L2a scan data
;	model_to_use	string			name of the angle adjustment model to use
;
; OUTPUTS
;	error			integer(4)			error flag.  0 means OK, non-zero means error		
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call correct_to_nom_EIA_and_Nadir(L2A_scan,error,model_to_use)
;
;@*****

	subroutine correct_to_nom_EIA_and_Nadir(L2A_scan,error,model_to_use)

		use msu_constants
		use msu_types
		use NAN_support 
		

		implicit none

		type(msu_L2A_scn)		:: L2A_scan	
		integer(4),intent(OUT)	:: error
		character(10),intent(IN),OPTIONAL :: model_to_use

		integer(4)					:: fov				! Field of view index
		integer(4)					:: chan				! Channel index
		real(4)						:: long				! Longitude of cell under consideration
		real(4)						:: lat				! Latitude of cell under consideration
		integer(4)					:: land_sea			! Sea(1)/Land(2) flag
		real(4)                     :: land_fraction    ! land fraction over 2.5 degree cell at location
		real(4)						:: Tb_mod			! modeled Tb at actual incidence angle
		real(4)						:: Tb_mod_nom_EIA	! modeled Tb at nominal EIA
		real(4)						:: Tb_mod_Nadir		! modelled Tb at Nadir

		error = 0
		


		do chan = 1, NUM_CHANNELS
		


			do fov = 1,NUM_FOVS

				! determine if cell is on land or sea

				long = L2A_scan%FOV_longitude(fov)
				lat  = L2A_scan%FOV_latitude(fov)

				land_sea = find_land_sea(lat,long)
				land_fraction =  find_land_fraction(long,lat)
				if (land_sea .lt. 1) then
				  print *
				endif
				if ((land_sea .ge. 1) .and. (land_sea .le. 2)) then
				    Tb_mod =		find_modelled_Tb_theta_date_interp(lat,						&
																	    long,					&
																	    L2A_scan%date,			&
																	    land_sea,				&
																	    abs(L2A_scan%fov_incidence(chan,fov)), &
																	    fov,				&
																	    chan,					&
																	    error,                  &
																	    model_to_use)

				    Tb_mod_nom_EIA =	find_modelled_Tb_theta_date_interp(lat,					&
																	    long,					&
																	    L2A_scan%date,			&
																	    land_sea,				&
																	    Nom_EIA(abs(fov-6)+1),	&
																	    fov,				&
																	    chan,					&
																	    error,                  &
																	    model_to_use)

				    Tb_mod_nadir = find_modelled_Tb_theta_date_interp(	lat,					&
																	    long,					&
																	    L2A_scan%date,			&
																	    land_sea,				&
																	    0.0,					&
																	    6,						&
																	    chan,					&
																	    error,                  &
																	    model_to_use)

				    L2A_scan%MSU_TBs_nom_EIA_corr(chan,fov) = Tb_mod_nom_EIA - Tb_mod
				    L2A_scan%MSU_TBs_NADIR_corr(chan,fov)   = Tb_mod_nadir -   Tb_mod
				  else
				    L2A_scan%MSU_TBs_nom_EIA_corr(chan,fov) = NAN
				    L2A_scan%MSU_TBs_NADIR_corr(chan,fov)   = NAN
				    L2A_scan%fov_qual(chan,fov)  = ibset(L2A_scan%fov_qual(chan,fov),1)
				    L2A_scan%scan_qual = ibset(L2A_scan%scan_qual,4)
				  endif
			enddo
		enddo

	end subroutine correct_to_nom_EIA_and_NADIR


;@***h* MLT_V4_RSS/allocate_Mod_Tbs
; NAME
;	allocate_Mod_Tbs
;
; PURPOSE
;	allocates model Tb structure to hold Tbs from NCEP or MERRA models
;
; DESCRIPTION
;	allocates model Tb structure to hold Tbs from NCEP or MERRA models
;
; INPUTS
;	Modelled_Tbs	array of type(map_Str)		uninitialized/unallocated data structure
;
; OUTPUTS
;	Modelled_Tbs	array Of type(map_Str)		initialized/allocated data structure
;	Error			integer(4)				error flag.  0 means OK, non zero means error.		
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call allocate_Mod_Tbs(Modelled_Tbs,Error)

;
;@*****
	subroutine allocate_Mod_Tbs(Modelled_Tbs,Error)
			  		
		type(map_str),dimension(NUM_THT,NUM_FOV,NUM_SURFACES,NUM_CHANNELS,NUM_DATA),intent(INOUT)	::	Modelled_Tbs
		integer(4),intent(OUT)																	::  error

		integer(4)			:: tht
		integer(4)			:: fov
		integer(4)			:: surf
		integer(4)			:: chan
		integer(4)			:: dat

			do tht = 1,NUM_THT
				do fov = 1, NUM_FOV
					do surf = 1, NUM_SURFACES
						do chan = 1, NUM_CHANNELS
							do dat = 1,NUM_DATA

								call set_up_map( 0.0,			&
												 2.5,			&
												 144,			&
												 EDGE_CENTERED,	&
												 -90.0,			&
												 2.5,		&
												 73,			&
												 EDGE_CENTERED,	&
												 SPHERICAL,		&
												 Modelled_Tbs(tht,fov,surf,chan,dat),			&
												 error)
								if (error .ne. 0) then
									return
								endif

							enddo
						enddo
					enddo
				enddo
			enddo

		return

	end subroutine allocate_Mod_Tbs

;@***h* MLT_V4_RSS/calc_pol_wts
; NAME
;	calc_pol_wts
;
; PURPOSE
;	calculates polarization weights due to polarization rotation due to mirror angle
;
; DESCRIPTION
;	calculates polarization weights due to polarization rotation due to mirror angle
;
; INPUTS
;	FOV_index		integer(4)		FOV index
;	roll_error		real(4)		roll error
;
; OUTPUTS
;	V_Pol_wt		real(4)		V-Pol weight
;	H_Pol_wt		real(4)		H-pol weight
;	error			INTEGER(4)		error flag  - always 0
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	call calc_pol_wts(FOV_index,roll_error,V_Pol_wt,H_Pol_wt,error)
;
;@*****
	subroutine calc_pol_wts(FOV_index,roll_error,V_Pol_wt,H_Pol_wt,error)

		integer(4),intent(IN)		:: FOV_index
		real(4),intent(IN)		:: roll_error
		real(4),intent(OUT)		:: V_Pol_wt
		real(4),intent(OUT)		:: H_Pol_wt
		integer(4),intent(OUT)		:: error

		error = 0
		V_Pol_wt = (cosd(ANGLE(abs(fov_index - 6) + 1) + (roll_error*sign(1,(6-FOV_index)))))**2
		H_Pol_wt = 1 - V_Pol_wt

		return

	end subroutine	calc_pol_wts

end module msu_orbital_and_roll_corrections 


