$freeform
module msu_roll_error

	implicit none

	character(len = 100),parameter		:: Roll_Error_Coeff_Path = '\\sounder\C\Module_Data\MSU_Utilities\MSU_Roll_Fits\'
	integer(4),parameter				:: NUM_COEFFS = 4

contains

;@***h* MLT_V4_RSS/msu_roll_error_angle
; NAME
;	msu_roll_error_angle
;
; PURPOSE
;	calculates the MSU roll error as a function of satellite and time
;
;
; DESCRIPTION
;	calculates the MSU roll error as a function of satellite and time
;
;
; INPUTS
;	satnum		integer(4)		satellite number (1..9)
;	channel		integer(4)		channel number (1..4)
;	date			date_Str		date to interpolate to
;
; OUTPUTS
;	return value	real(4)		roll angle
;	error			integer(4)		error flag.  0 means OK, non-zero mean error during read.
;
; EXTERNAL DATA NEEDED
;	roll error fit coefficients
;     located in files:
;	MSU_Roll_Error_Fit_Coefficients_Ch1.txt
;	MSU_Roll_Error_Fit_Coefficients_Ch2.txt
;	MSU_Roll_Error_Fit_Coefficients_Ch3.txt
;	MSU_Roll_Error_Fit_Coefficients_Ch4.txt
;
; AUTHOR
;	Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;	THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;	THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;	AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;	AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;	SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;	THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT TO USERS.
;
; HISTORY
;	4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;	roll_Angle = msu_roll_error_angle(sat_num,channel,date,error)
;
;@*****

	real(4) function msu_roll_error_angle(sat_num,channel,date,error)
									 
		use date_and_time
		use msu_constants

		integer(4)			:: sat_num
		integer(4)			:: channel
		type(date_str)		:: date
		integer(4)			:: error


		logical,dimension(NUM_CHANNELS)						:: init = .false.
		real(4),dimension(NUM_SATS,NUM_CHANNELS,NUM_COEFFS) :: MSU_roll_coeff
		type(date_str)										:: Jan_1_1990
		integer(4)											:: date_error
		integer(4)											:: sat_index
		character(len = 120)									:: file_name
		real(8)											:: dt


		error = 0

		if (.not. init(channel)) then

			! initial jan 1, 1990
			Jan_1_1990 = set_date(1,1,1990,date_error)

			! read in the coefficients

			! find file name

			write(file_name,111) trim(Roll_Error_Coeff_Path),channel
111			format(a,'MSU_Roll_Error_Fit_Coefficients_Ch',i1,'.txt')

			open(unit = 3,file=file_name,status='old')

			do sat_index = 1,9
				read(3,222)	msu_roll_coeff(sat_index,channel,1), &
							msu_roll_coeff(sat_index,channel,2), &
							msu_roll_coeff(sat_index,channel,3), &
							msu_roll_coeff(sat_index,channel,4)
222				format(4f25.0)
			enddo
			init(channel) = .true.
		endif

		dt = (date%rd_2000 - jan_1_1990%rd_2000)/365.25

		msu_roll_error_angle = msu_roll_coeff(sat_num,channel,1)	+		&
							 dt*(msu_roll_coeff(sat_num,channel,2)	+		&
							 dt*(msu_roll_coeff(sat_num,channel,3)	+		&
							 dt*msu_roll_coeff(sat_num,channel,4)));



   end function msu_roll_error_angle
end module msu_roll_error

			



		 

