pro make_msu_monthly_map_V4_TLT,satellite_name,year_to_read,month_to_read,msu_l2a_hist_TLT,error = error, $
                                tb_hist= tb_hist,model_String=model_String,TLT_version = TLT_version
;@***h* make_msu_monthly_map_V4_TLT.pro
;
; NAME
;   make_msu_monthly_map_V4_TLT.pro
;
; PURPOSE
;   reads a monthly MSUv2 L2A file, and reorganizes it into a gridded dataset
;   and writes a MSUv2 L2C TLT map
;   
; DESCRIPTION
;   reads a monthly MSUv2 L2A file, and reorganizes it into a gridded dataset
;   and writes a MSUv2 L2C TLT map
;   
; INPUTS
;   satellite_name       ; string containing MSU satellite name
;   year_to_read         ; integer -- year to read
;   month_to_read        ; integer 1..12 -- month to read
;   model_string         ; name of angle correction model used (MERRA or NCEP)
;                        ; defaults to NCEP
;   TLT_version          ; flag to determine whether or not angle adjustment is performed
;                        ; 0 = no correction
;                        ; 1 = correction to nominal EIA is performed

;   
;   
; OUTPUTS
;   msu_l2a_hist_TLT     ; gridded dataset
;   tb_hist              ; 1D histogram of tbs on a 1K grid -- for quality control
;   error                ; error flag.  0 means everything is OK, -1 Means a read error
;     
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
;   TO USERS.
;
; HISTORY
;   10/30/2013 Initial version written
;
; USAGE
;   make_msu_monthly_map,satellite_name,year_to_read,month_to_read,channel,msu_l2a_hist_by_fov,error = error, tb_hist= tb_hist
;
;@*****    

    level_string = 'R3'
    read_msu_L2Av2_monthly_file,satellite_name,year_to_read,month_to_read,msu_L2a,level_string = level_string,error,model_String=model_String
    if error ne 0 then return
    FOV_QUAL_MASK = 125
    num_scans = n_elements(msu_l2a)
    msu_l2a_hist_tlt = fltarr(4,2,2,144,72)   ; (num,tb,tar,hgt), asc/dsc,left/right,ilon,ilat)
    
    tb_hist = fltarr(320)
    chan = 1
    iasc = 0
    for scan = 0,num_scans - 1 do begin
      if scan mod 10000 eq 0 then print,scan
      if msu_l2a[scan].scan_qual eq 0 then begin
        if msu_l2a[scan].height_corrected lt 700.0 then begin
          print,msu_l2a[scan].height,msu_l2a[scan].height_corrected
          print
        endif
        
        ; skip 2 bad days in March 1980 for NOAA-06 -- these days have bad geolocation, undetected until now
        
        if (satellite_name eq 'NOAA-06') then begin
          if (year_to_read eq 1980) then begin
            if (month_to_read eq 3) then begin
              if ((msu_l2a[scan].rd2000 gt -7244.0D0) and (msu_l2a[scan].rd2000 lt -7242.0D0)) then begin
                continue
              endif
            endif
            if (month_to_read eq 11) then begin
              if ((msu_l2a[scan].rd2000 gt -6977.0D0) and (msu_l2a[scan].rd2000 lt -6976.0D0)) then begin
                continue
              endif
            endif
          endif
        endif
        
        ; skip bad orbit in NOAA-14
        
        if (satellite_name eq 'NOAA-14') then begin
          if (year_to_read eq 1996) then begin
            if (month_to_read eq 4) then begin
              if ((msu_l2a[scan].rd2000 gt -1341.0D0) and (msu_l2a[scan].rd2000 lt -1340.85D0)) then begin
                continue
              endif
            endif
          endif
        endif
        
        iasc = 1
        if msu_l2a[scan].scan_number le 61 then iasc = 0  ; ascending
        if msu_l2a[scan].scan_number ge 181 then iasc = 0 ; ascending
        ; so 63 to 181 is descending
        ilon = floor(msu_l2a[scan].fov_longitude/2.5)
        ilat = floor((msu_l2a[scan].fov_latitude + 90.0)/2.5)
        
        if chan le 1 then begin
          target_temp = msu_l2a[scan].T.Target1_temp
        endif else begin
          target_temp = msu_l2a[scan].T.Target2_temp
        endelse
        
        for left_right = 0,1 do begin
          if left_right eq 0 then begin
            bad = where(ilon[0:3] lt 0,badct)
            if badct gt 0 then continue
            bad = where(ilon[0:3] gt 143,badct)
            if badct gt 0 then continue
            bad = where(ilat[0:3] lt 0,badct)
            if badct gt 0 then continue
            bad = where(ilat[0:3] gt 71,badct)
            if badct gt 0 then continue
            bad = where((125 and msu_l2a[scan].fov_qual[0:3,chan]) ne 0,badct)
            if badct gt 0 then continue
            bad = where(finite(msu_l2a[scan].msu_tbs_raw[0:3,chan]) eq 0,badct)
            if badct gt 0 then continue
            bad = where(msu_l2a[scan].msu_tbs_raw[0:3,chan] lt 170.0,badct)
            if badct gt 0 then continue
            bad = where(msu_l2a[scan].msu_tbs_raw[0:3,chan] gt 300.0,badct)
            if badct gt 0 then continue
            
            case TLT_version of
              0: begin
                  tbs_corr = msu_l2a[scan].msu_tbs_raw[*,chan] ;- msu_l2a[scan].MSU_TBS_NOM_EIA_CORR[*,chan]
                 end
              1: begin
                   tbs_corr = msu_l2a[scan].msu_tbs_raw[*,chan] - msu_l2a[scan].MSU_TBS_NOM_EIA_CORR[*,chan]
                 end
            endcase

            tlt_tb  = 2.0*(tbs_corr[3] + tbs_corr[2]) - $
                      1.5*(tbs_corr[1] + tbs_corr[0])
            if tlt_tb lt 180.0 then continue
            if tlt_tb gt 320.0 then continue
            if finite(tlt_tb) eq 0 then continue
            itlt = floor(tlt_tb)
            tb_hist[itlt] =   tb_hist[itlt] + 1
            for fov = 0,3 do begin
              msu_l2a_hist_tlt[0,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[0,iasc,left_right,ilon[fov],ilat[fov]] + 1
              msu_l2a_hist_tlt[1,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[1,iasc,left_right,ilon[fov],ilat[fov]] + tlt_tb
              msu_l2a_hist_tlt[2,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[2,iasc,left_right,ilon[fov],ilat[fov]] + target_temp
              msu_l2a_hist_tlt[3,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[3,iasc,left_right,ilon[fov],ilat[fov]] + msu_l2a[scan].height_corrected
            endfor
            
          endif else begin
            bad = where(ilon[7:10] lt 0,badct)
            if badct gt 0 then continue
            bad = where(ilon[7:10] gt 143,badct)
            if badct gt 0 then continue
            bad = where(ilat[7:10] lt 0,badct)
            if badct gt 0 then continue
            bad = where(ilat[7:10] gt 71,badct)
            if badct gt 0 then continue
            bad = where((125 and msu_l2a[scan].fov_qual[7:10,chan]) ne 0,badct)
            if badct gt 0 then continue
            bad = where(finite(msu_l2a[scan].msu_tbs_raw[7:10,chan]) eq 0,badct)
            if badct gt 0 then continue
            bad = where(msu_l2a[scan].msu_tbs_raw[7:10,chan] lt 170.0,badct)
            if badct gt 0 then continue
            bad = where(msu_l2a[scan].msu_tbs_raw[7:10,chan] gt 300.0,badct)
            if badct gt 0 then continue
            
            case TLT_version of
              0: begin
                  tbs_corr = msu_l2a[scan].msu_tbs_raw[*,chan] ;- msu_l2a[scan].MSU_TBS_NOM_EIA_CORR[*,chan]
                 end
              1: begin
                   tbs_corr = msu_l2a[scan].msu_tbs_raw[*,chan] - msu_l2a[scan].MSU_TBS_NOM_EIA_CORR[*,chan]
                 end
            endcase
            tlt_tb  = 2.0*(tbs_corr[7] + tbs_corr[8]) - $
              1.5*(tbs_corr[9] + tbs_corr[10])
            
            if tlt_tb lt 180.0 then continue
            if tlt_tb gt 320.0 then continue
            if finite(tlt_tb) eq 0 then continue
            itlt = floor(tlt_tb)
            tb_hist[itlt] =   tb_hist[itlt] + 1

            for fov = 7,10 do begin
              msu_l2a_hist_tlt[0,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[0,iasc,left_right,ilon[fov],ilat[fov]] + 1
              msu_l2a_hist_tlt[1,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[1,iasc,left_right,ilon[fov],ilat[fov]] + tlt_tb
              msu_l2a_hist_tlt[2,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[2,iasc,left_right,ilon[fov],ilat[fov]] + target_temp
              msu_l2a_hist_tlt[3,iasc,left_right,ilon[fov],ilat[fov]] = msu_l2a_hist_tlt[3,iasc,left_right,ilon[fov],ilat[fov]] + msu_l2a[scan].height_corrected
            endfor
          endelse
        endfor
      endif     ; if msu_l2a[scan].scan_qual eq 0
    endfor      ; scan
end
