;@***h* MLT_RSSv4/read_msu_l2av2_monthly_file.pro
; PURPOSE
;   contains 3 routines related to reading msu_l2av4 momnthly files
;
; NAME
;   init_msu_monthly_L2Av4_scan
;
; PURPOSE
;   initializes a msu L2Av4 scan
;
; DESCRIPTION
;   initializes a msu L2Av4 scan
;   scan is designed so that data from the L1B to L2A fortran progam can be directly read into an IDL structure
;
; INPUTS
;   none
;
;
; OUTPUTS
;   scan    a structure containing a msu L2Av4 scan, initialized to default values
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   5/15/2018 Initial version provided to NCEI
;
; USAGE
;   init_msu_monthly_L2Av4_scan,scan
;
; NAME
;   init_msu_monthly_L2Av4_data
;
; PURPOSE
;   init_msu_monthly_L2Av4_dataset for one month
;
; DESCRIPTION
;   init_msu_monthly_L2Av4_data
;   number of scans to create is passed to the routine
;
; INPUTS
;   number_of_swaths  integer containing the number of scans (or swaths) to create   
;
; OUTPUTS
;   msu_L2A    an one-D array of  structures containing a msu L2Av4 scans, initialized to default values
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   5/15/2018 Initial version provided to NCEI
;
; USAGE
;   init_msu_monthly_L2Av4_data,number_of_swaths,msu_L2A
;
; NAME
;   read_msu_L2Av4_monthly_file
;
; PURPOSE
;   reads msu L2Av4 data from a monthly file
;
; DESCRIPTION
;   reads msu L2Av4 data from a monthly file
;   data are contained in an array of IDL structures
;
; INPUTS
;   satellite_name    string containing the name of the satellite
;   year_to_read      integer containing the year to read
;   month_to_read     integer containing the month to read
;   level_string      string containing the level name.  Usually 'R3' for routine processing
;   model_string      string containing the name of the EIA model used to create the L2Av4 data.  Usually 'MERRA'
;
; OUTPUTS
;   msu_L2A    an one-D array of  structures containing a msu L2Av4 scans, initialized to default values
;   error      error flag.   non-zero value indicates an error condition.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   5/15/2018 Initial version provided to NCEI
;
; USAGE
;   read_msu_L2Av4_monthly_file,'NOAA-14,2000,4,msu_L2a,level_string = 'R3',error,model_string='MERRA'
;





pro init_msu_monthly_L2Av4_scan,scan
       msu_temps = {msu_temps,            $                       
                        Target1_temp:0.0D0,   $
                        Target2_temp:0.0D0,   $
                        Target1_temp_A:0.0D0,   $
                        Target1_temp_B:0.0D0,   $
                        Target2_temp_A:0.0D0,   $
                        Target2_temp_B:0.0D0}
                       
        scan = {msu_L2A_scan,         $
            sat_num:-99L,             $  
            orbit_num:-99L,           $
            scan_number:-99L,         $    
            rd2000:-999999.0D0,       $
            spare_real:-99.0 , $       
            FOV_latitude:replicate(!values.f_nan,11),      $  ; longitude of each measurement
            FOV_longitude:replicate(!values.f_nan,11),     $  ; latitude of each measurement
            FOV_incidence:replicate(!values.f_nan,11,4),     $  ; earth incidence of each measurement
            
            height:-99.0,         $         ; height, in kilometers wrt geoid
            height_corrected:-99.0, $       ; corrected for anavigation errors
            asc_desc:0L,          $         ; asc/desc flag
            scan_qual:0L,         $         ; scan quality flag
            
            fov_qual:replicate(-99B,11,4),       $
            MSU_counts:replicate(-99,14,4),      $  ; Raw MSU counts, and other data
            MSU_counts_corr:replicate(-99.0,14,4),      $
            MSU_Tbs_Raw:replicate(-99.0,11,4),      $
            MSU_Tbs_Nom_EIA_corr:replicate(-99.0,11,4),      $
            MSU_Tbs_NADIR_corr:replicate(-99.0,11,4),      $
            MSU_Tbs_Diurnal_corr:replicate(-99.0,11,4),      $
            MSU_Spare1:replicate(-99.0,11,4),      $
            MSU_Spare2:replicate(-99.0,11,4),      $
            T:msu_temps, $
            Roll_Err:replicate(-99.0,4)}

end

pro init_msu_monthly_L2Av4_data,number_of_swaths,msu_L2A
       init_msu_monthly_L2Av4_scan,scan 
       msu_L2A = replicate(scan,number_of_swaths)
end
            

pro read_msu_L2Av4_monthly_file,satellite_name,year_to_read,month_to_read,msu_L2a,level_string = level_string,error,model_string=model_string

    if keyword_Set(level_string) eq 0 then level_string = 'R1'
    if keyword_set(model_string) eq 0 then model_string = 'ncep'
    MAX_SCANS_PER_MONTH=120000L

    ; find satellite_number

    for sat_num = 1,9 do begin
      if (sat_name(sat_num,/fill) eq satellite_name) then break
    endfor

    if (sat_num gt 9) then begin
      error = -1
      return
    endif

    error = 0
    case model_string of 
      'ncep':begin
        MSU_L2A_PATH = 'M:\MSU_AMSU_Data\MSU_L2Av2\'
        filename = string(strtrim(MSU_L2A_PATH),satellite_name,satellite_name,year_to_read,month_to_read,level_string, $
        format = '(a,a,"\monthly\",a,"_monthly_L2A_",i4.4,"_",i2.2,"_",a,".dat")')
      end
      'merra':begin
        MSU_L2A_PATH = 'M:\MSU_AMSU_Data\MSU_L2Av2\'
        filename = string(strtrim(MSU_L2A_PATH),satellite_name,satellite_name,year_to_read,month_to_read,level_string, $
          format = '(a,a,"\monthly\",a,"_monthly_L2A_",i4.4,"_",i2.2,"_",a,".merra.dat")')
      end
   endcase

  ; Determine if file exists

    info = file_info(filename)
    if info.exists then begin
      print,filename
      close,3
      openr,3,filename
      init_msu_monthly_L2Av2_data,MAX_SCANS_PER_MONTH,msu_L2A
      scan_init = msu_L2A[0]
      on_ioerror,file_end
      j = 0L
      while 1 do begin
        scan = scan_init 
        readu,3,scan
        msu_L2A[j] = scan
        j = j+1L
      endwhile
      file_end:
      close,3
      on_ioerror,null
      msu_L2A = msu_L2A[0:j-1]
      error = 0   
    endif else begin
      init_msu_monthly_L2Av2_scan,scan
      msu_L2A = scan
      error = -1
    endelse
  end 