



;@***h* MLT_V4_RSS/calc_and_save_all_MSU_adjustments_from_FOV.pro
; NAME
;   MLT_V4_RSS/calc_and_save_all_MSU_adjustments_from_FOV.pro
;
; PURPOSE
;   calculates angle and diurnal adjustment for MSU for given set of angle and diurnal models
;
; DESCRIPTION
;   calculates angle and diurnal adjustment for MSU for given set of angle and diurnal models
;
; INPUTS
;   channel                 integer:  MSU channel number
;   fov_wts                 array of 11 reals.  FOV weights
;   node_wts                array of 2 reals.  Node weights
;   adj_flag                usually set to seven to do incidence, nadir, and diurnal adjustments
;   eia_model = eia_model   string containing name of EIA model.  Usually 'MERRA'
;   diur_model=diur_model   string containing name of diurnal model.  ['CCM3','MERRA','HADGEM']
;   tb_arr_adj              array of reals (144,72,336,9) of satellite radiances
;   tar_arr=tar_arr         array of reals (144,72,336,9) of satellite radiances
;   num_arr=num_arr           array of reals (144,72,336,9) of number of obs in each cell
;   sats_to_use = sats_to_use array of 9 integers.  set to 1 to use that satellite.

; OUTPUTS
;   writes diurnal and angle adjustment gridded monthly files for MSU
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   4/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   calc_and_save_all_MSU_adjustments_from_FOV,channel,fov_wts,node_wts,adj_flag,eia_model = eia_model,diur_model=diur_model,tb_arr_adj,tar_arr=tar_arr,num_arr=num_arr,sats_to_use = sats_to_use
;
;@*****

pro calc_and_save_all_MSU_adjustments_from_FOV,channel,fov_wts,node_wts,adj_flag,eia_model = eia_model,diur_model=diur_model,tb_arr_adj,tar_arr=tar_arr,num_arr=num_arr,sats_to_use = sats_to_use

if keyword_set(sats_to_use) eq 0 then sats_to_use = replicate(1.0,7)

; load angle adjustment arrays
common eia
common diurnal
;
if keyword_Set(eia_model) eq 0 then eia_model = 'MERRA'
case eia_model of
  'NCEP':model='NCEP'
  'MERRA':model='MERRA'
  else:begin
    Print,'EIA model:'+eia_model+' not recognized.  EIA model set to MERRA'
    model = 'MERRA'
  end
endcase

case diur_model of
  'CCM3':diurnal_model = 'CCM3'
  'MERRA':diurnal_model = 'MERRA'
  'HADGEM':diurnal_model = 'HADGEM'
  'CMAM':diurnal_model = 'CMAM'
  else:begin
    Print,'Diurnal model:'+diur_model+' not recognized.  Diurnal model set to CCM3'
    diurnal_model = 'CCM3'
    end 
endcase

eia__adj_arr = replicate(!values.f_nan,12,3,6,144,72)

if model eq 'MERRA' then begin
  for month = 1,12 do begin
    read_msu_merra_nadir_adj_v4,channel,month,merra_eia_adj_arr_one_month
    eia__adj_arr[month-1,*,*,*,*] = merra_eia_adj_arr_one_month
  endfor  
endif
if model eq 'NCEP' then begin
  read_land_fraction_144_72,lf
  for month = 1,12 do begin
      read_msu_ncep_nadir_adj_v4,channel,month,ncep_eia_adj_arr_one_month,/rebin ; the /rebin option automatically rebins to 144x72 cell centered
      ncep_eia_adj_arr_one_month_combined = fltarr(3,6,144,72)
      for type = 0,2 do begin
        for fov = 0,5 do begin
          ncep_eia_adj_arr_one_month_combined[type,fov,*,*] =  $
                (1.0 - lf)*ncep_eia_adj_arr_one_month[type,fov,*,*,0] + $
                lf*ncep_eia_adj_arr_one_month[type,fov,*,*,1]
        endfor
      endfor
      eia__adj_arr[month-1,*,*,*,*] = ncep_eia_adj_arr_one_month_combined
  endfor
endif

eia__lat_map = fltarr(144,72)
for ilat = 0,71 do begin
   eia__lat_map[*,ilat] = -90.0 + 1.25 + 2.5*ilat
endfor

eia__channel_loaded = channel
eia__model_loaded = model
print,'Finished Loading Angle Adjustments, Model: '+eia__model_loaded+', Channel: '+string(eia__channel_loaded)
; end load angle adjustments

; load diurnal adjustment arrays

read_msu_diurnal_adj_arr_v4,channel,diurnal_model,diur__adj_arr,err
if err eq 0 then begin
  diur__channel_loaded = channel
  diur__model_loaded = diurnal_model
  print,'Finished Loading Diurnal Adjustments, Model: '+diur__model_loaded+', Channel: '+string(diur__channel_loaded)
endif else begin
  diur__channel_loaded = -1
  diur__model_loaded  = 'none'
endelse

; end load diurnal adjustment arrays

tb_arr_adj  = fltarr(144,72,336,9)
tar_arr     = fltarr(144,72,336,9)
num_arr     = fltarr(144,72,336,9)
sats_to_use = [1,1,1,1,1,1,1,1,1]
if channel eq 3 then begin
  sats_to_use = [0,0,0,0,0,1,1,1,1]
endif
;sats_to_use = [0,0,0,0,1,0,0,0,0]  ; temporary!

for satnum = 1,9 do begin
  if sats_to_use[satnum-1] eq 0 then continue  
  for year = 1978,2005 do begin
    for month = 1,12 do begin
      adjusted_MSU_map_from_FOV_map,satnum,channel,year,month,     $
                                      adj_flag=adj_flag,            $    ; 1 = inc,2 = nadir, 4 = diur + combinations.  nadir == nadir+inc
                                                                    $    ; default will be 7
                                      fov_wts = fov_wts,            $    ; flt(30)  will forced to add to 1
                                      node_wts = node_wts,          $    ; fltarr(2) will be forced to add to 1, deault = [0.5,0.5]
                                      tb_map_adj,                   $    ; adjusted,fov-combined 144x72 tb map
                                      tar_map = tar_map,            $    ; optional variable to return tar map
                                      num_map = num_map,            $    ; optional variable to return num map
                                      diurnal_adjustment = diurnal_adjustment, $ ;optional variable containing diurnal adjustment
                                      inc_adjustment = inc_adjustment,         $ ;optional variable containing incidence adjustment
                                      nadir_adjustment = nadir_adjustment,     $
                                      error  
                                      
      if error eq 0 then begin
        print,string(sat_name(satnum,/fill),year,month,diur__model_loaded,eia__model_loaded,format = '("Calculated Adjustments, ",a," year = ",i4.4,", month = ",i2.2,1x,a,1x,a)')
        write_MSU_diurnal_adjustment_fov,satnum,channel,year,month,diur__model_loaded,diurnal_adjustment,error
        write_MSU_incidence_adjustment_fov,satnum,channel,year,month,eia__model_loaded,inc_adjustment,error
        write_MSU_nadir_adjustment_fov,satnum,channel,year,month,eia__model_loaded,nadir_adjustment,error
      endif
    endfor
  endfor   
endfor                           
end