

;@***h* MLT_V4_RSS/calc_msu_diurnal_adjustments.pro
; NAME
;   MLT_V4_RSS/calc_msu_diurnal_adjustments.pro
;
; PURPOSE
;   calculates maps of diurnal adjustments for MSU
;
; DESCRIPTION
;   calculates maps of diurnal adjustments for MSU.  The diurnal cycles are contained in the "Diurnal" common block
;   measurement times are found in this routine.
;
; INPUTS
;   satnum              integer: satellite number (1..9)
;   year                integer: year to process
;   month               integer: month of year to process
;   time_shift          real: optional time shift in hours. Default = 0.0
;
; OUTPUTS
;  diurnal_adjustment       array of reals [2,11,144,72]:  adjustments to nominal meausrement time
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   4/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   calc_msu_diurnal_adjustments,satnum,year,month,diurnal_adjustment
;
;@*****
pro calc_msu_diurnal_adjustments,satnum,year,month,diurnal_adjustment,time_shift = time_shift

  forward_function get_nth_value
  common diurnal
  
  if keyword_set(time_shift) eq 0 then time_shift = 0.0

   fov_mapping = [5,4,3,2,1,0,1,2,3,4,5]
   get_msu_measurement_times,satnum,year,month,measurement_times
   fill_msu_measurement_times,measurement_times
   
   ok = where(finite(measurement_times) eq 1,ct)
   diurnal_adjustment = replicate(!values.f_nan,2,11,144,72)
   if ct gt 10000L then begin   
     for node = 0,1 do begin
       for fov = 0,10 do begin
          diur_maps = reform(diur__adj_arr[month-1,fov_mapping[fov],*,*,*])
          meas_hours = (48.0 + time_shift-0.5 + measurement_times[*,*,fov,node]*24.0) mod 24.0
          bad = where(finite(meas_hours) eq 0,bad_ct)
          hour_lower = floor(meas_hours)
          hour_upper = hour_lower + 1
          too_high = where(hour_upper gt 23,too_high_ct)
          if too_high_ct gt 0 then hour_upper[too_high] = hour_upper[too_high]- 24
          wt_upper = meas_hours-hour_lower
          wt_lower = 1.0 - wt_upper
          if bad_ct gt 0 then hour_lower[bad] = 0
          if bad_ct gt 0 then hour_upper[bad] = 0
          tb_lower = get_nth_value(diur_maps,hour_lower)
          tb_upper = get_nth_value(diur_maps,hour_upper)
          diur_tb_meas_time = tb_lower*wt_lower + tb_upper*wt_upper
          diur_tb_midnight = (diur_maps[*,*,0]+diur_maps[*,*,23])*0.5
          diur_tb_noon = (diur_maps[*,*,11]+diur_maps[*,*,12])*0.5
          diurnal_adjustment[node,fov,*,*] = 0.0-diur_tb_meas_time
       endfor
     endfor
   endif
   
end


end
