pro adjust_offsets_by_land_fraction,offsets_in,offsets_adj

;@***h* MLT_V4_RSS/adjust_offsets_by_land_fraction
; NAME
;   adjust_offsets_by_land_fraction
;
; PURPOSE
;   adjusts offets for cases where either there is not much land, or not much ocean in a given latitude band.
;   in these cases, the offsets are moved toward the offset for the other surface type.  For example, if there 
;   is not much land (less than 20%) then the land offsets are combined with the ocean offsets to ensure some
;   degree of smoothness.
;
; DESCRIPTION
;   adjusts offets for cases where either there is not much land, or not much ocean in a given latitude band.
;   in these cases, the offsets are moved toward the offset for the other surface type.  For example, if there 
;   is not much land (less than 20%) then the land offsets are combined with the ocean offsets to ensure some
;   degree of smoothness.
;   
; INPUTS
;   offsets     fltarr array (72 x num_sats x 2) of offset values
;   
; OUTPUTS
;   offsets     fltarr array (72 x num_sats x 2) of adjusted offset values
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   1/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   adjust_offsets_by_land_fraction,offsets,offsets_adj
;
;@*****
    
    offsets = offsets_in
    sz = size(offsets,/dimensions)
    
    if n_elements(sz) ne 3 then begin
      offsets_adj = !values.f_nan
      return
    endif
    
    num_sats = sz[1]
    offsets_adj = replicate(!values.f_nan,sz[0],sz[1],sz[2])
 
    read_land_fraction_144_72,lf
    
    for ilat = 0,71 do begin
      offsets_this_lat = reform(offsets[ilat,*,*])
      offsets_exist = where(finite(offsets_this_lat) gt 0,ct,complement = offsets_bad)
      land_this_lat = total(lf[*,ilat])/144.0
      
      if ct gt 0 then begin
        sea_wt_sea = 1.0
        land_wt_sea = 0.0
        sea_wt_land = 0.0
        land_wt_land = 1.0
        if land_this_lat gt 0.8 then begin ; adjust _sea wts
          sea_wt_sea = 5.0 - land_this_lat/0.2
          land_wt_sea = 1.0 - sea_wt_sea
        endif
        if land_this_lat lt 0.2 then begin ; adjust _land wts
          land_wt_land = land_this_lat/0.2
          sea_wt_land = 1.0 - land_wt_land
        endif

        sea_offsets_bad = where(finite(offsets_this_lat[*,0]) eq 0,sea_bad_ct)
        land_offsets_bad  = where(finite(offsets_this_lat[*,1]) eq 0,land_bad_ct)

        if ((sea_bad_ct gt 0) and ((sea_wt_sea gt 0.0) or (sea_wt_land gt 0.0))) then begin
          land_wt_sea = 1.0
          land_wt_land = 1.0
          sea_wt_sea = 0.0
          sea_wt_land = 0.0
          offsets[ilat,*,0] = 0.0
        endif
        if ((land_bad_ct gt 0) and ((land_wt_sea gt 0.0) or (land_wt_land gt 0.0))) then begin
          land_wt_sea = 0.0
          land_wt_land = 0.0
          offsets[ilat,*,1] = 0.0
          sea_wt_sea = 1.0
          sea_wt_land = 1.0
        endif
        offsets_adj[ilat,*,0] = sea_wt_sea*offsets[ilat,*,0] + land_wt_sea*offsets[ilat,*,1]
        offsets_adj[ilat,*,1] = sea_wt_land*offsets[ilat,*,0] + land_wt_land*offsets[ilat,*,1]       
    endif
  endfor
end