pro apply_diurnal_fit_residuals_MSU,tb_arr,params,tb_arr_adjust

;@***h* MLT_V4_RSS/apply_diurnal_fit_residuals_MSU
; NAME
;   apply_diurnal_fit_residuals
;
; PURPOSE
;   Applies diurnal adjustments to antenna temperature gridded data.
;
; DESCRIPTION
;   Applies diurnal adjustments to antenna temperature gridded data. The parameters for each latitude band and
;   surface type are passed in the params variable
;
;   The
;   best reference for this calculation is
;
;   Mears, C. A., & Wentz, F. J. (2016). Sensitivity of Satellite-Derived Tropospheric Temperature
;   Trends to the Diurnal Cycle Adjustment. Journal of Climate, 29, 3629-3646. doi: 10.1175/JCLI-D-15-0744.1
;     ;
;   The type of diurnal fit is figures out by the size of the array in the first dimension.
;
; INPUTS
;   tb_arr          fltarr array (144 x 72 x num_months x num_sats) of antenna temperatures
;   params          fltarr array (14 (or 8) x 2 x 72) array of paramters to apply
;                       The first dimension is offsets + harmonic paramters
;                       The second dimension is surface type
;                       The third dimensions if latitude in 2.5 degree bands.
;
; OUTPUTS
;   tb_arr_adj      fltarr array (144 x 72 x num_months x num_sats) of antenna temperatures with diurnal optimized
;                   adjustments applied.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   1/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   apply_diurnal_fit_residuals_MSU,tb_arr_adj2,diurnal_residual_parameters,tb_arr_adj_temp
;
;@*****
  sz = size(tb_arr,/dimensions)
  num_lons   = sz[0]
  num_lats   = sz[1]
  num_months = sz[2]
  num_sats   = sz[3]
  num_surfs  = 2
  num_param  = 6
  
  sz = size(params,/dimensions)
  if n_elements(sz) eq 3 then begin  ; this is the TMT/TLT case
    case sz[0] of
      11:begin  ; this is the msu-derived, seasonally constant case -- 0-8 are offsets
        temp = params
        params = fltarr(num_param,num_surfs,num_lats)  
        params[0:1,*,*] = temp[9:10,*,*]
        end
      10:begin  ; this is the amsu-derived, seasonally constant case -- 0-7 are offsets
        temp = params
        params = fltarr(num_param,num_surfs,num_lats)
        params[0:1,*,*] = temp[8:9,*,*]
        end
      15:begin ; this is the msu-derived, seasonally modulated case
        params = params[9:14,*,*]
        end
      14:begin ; this is the amsu-derived, seasonally modulated case
        params = params[8:13,*,*]
        end
      else: begin
        print,'Unexpected array size for params'
        print,'sz = ',sz
        stop
        end    
    endcase
    tb_Arr_adjust = tb_arr
    
    restore,'C:\Module_Data\AMSU_utilities\measurement_times\Local_Equator_Crossing_Times_Monthly.sav',/verbose
    msu_monthly_xtime_Array = monthly_xtimes[0:335,0:8]
    msu_monthly_xtime_Array = msu_monthly_xtime_Array/(3600.0)
    
    read_land_fraction_144_72,lf
    lf[*,0:17] = 0.0

    
    for month = 0,num_months-1 do begin
      for satnum = 0,num_sats-1 do begin
        t = msu_monthly_xtime_Array[month,satnum]
        if finite(t) eq 1 then begin
          map_adjust = fltarr(num_lons,num_lats)
          
          for ilat = 0,num_lats-1 do begin
            residual_diurnal_model,t,values 
            seas_wts = [sin(2.0*!pi*month/12.0),cos(2.0*!pi*month/12.0)]
            seas_weighted_values = fltarr(6)
            seas_weighted_values[0] = values[0]
            seas_weighted_values[1] = values[1]
            seas_weighted_values[2] = seas_wts[0]*values[0]
            seas_weighted_values[3] = seas_wts[1]*values[0]
            seas_weighted_values[4] = seas_wts[0]*values[1]
            seas_weighted_values[5] = seas_wts[1]*values[1]
                        
            ocean_adjust = -1.0*(total(params[*,0,ilat]*seas_weighted_values) )
            map_adjust[*,ilat] = (1.0-lf[*,ilat])*ocean_adjust
          endfor
          
          for ilat = 0,num_lats-1 do begin
            residual_diurnal_model,t,values 
            seas_wts = [sin(2.0*!pi*month/12.0),cos(2.0*!pi*month/12.0)]
            seas_weighted_values = fltarr(6)
            seas_weighted_values[0] = values[0]
            seas_weighted_values[1] = values[1]
            seas_weighted_values[2] = seas_wts[0]*values[0]
            seas_weighted_values[3] = seas_wts[1]*values[0]
            seas_weighted_values[4] = seas_wts[0]*values[1]
            seas_weighted_values[5] = seas_wts[1]*values[1]
            
            land_adjust = -1.0*(total(params[*,1,ilat]*seas_weighted_values)) 
            map_adjust[*,ilat] = map_adjust[*,ilat]+lf[*,ilat]*land_adjust
          endfor
          
          tb_arr_adjust[*,*,month,satnum] = tb_arr_adjust[*,*,month,satnum] + map_adjust
          
        endif
      endfor
    endfor    
    
  endif
  
  if n_elements(sz) eq 2 then begin  ; this is the TTS/TLS case
    tb_Arr_adjust = tb_arr
    print,'This is likely not right -- see above for correct code....'
    stop
    restore,'C:\Module_Data\AMSU_utilities\measurement_times\Local_Equator_Crossing_Times_Monthly.sav',/verbose
    msu_monthly_xtime_Array = monthly_xtimes[0:335,0:8]
    msu_monthly_xtime_Array = msu_monthly_xtime_Array/(3600.0)
    
    read_land_fraction_144_72,lf
    
    for month = 0,num_months-1 do begin
      seas_wts = [sin(2.0*!pi*month/12.0),cos(2.0*!pi*month/12.0)]

      for satnum = 0,num_sats-1 do begin
        t = msu_monthly_xtime_Array[month,satnum]
        if finite(t) eq 1 then begin
        
          map_adjust = fltarr(num_lons,num_lats)
          for ilat = 0,num_lats-1 do begin
            residual_diurnal_model,t,values
            
            adjust = -1.0*((params[9,ilat] + params[11,ilat]*seas_wts[0] + params[12,ilat]*seas_wts[1])*values[0] + (params[10,ilat] + params[13,ilat]*seas_wts[0] + params[14,ilat]*seas_wts[1])*values[1]) ;+params[satnum,0,ilat])
            map_adjust[*,ilat] = adjust
          endfor
         
          tb_arr_adjust[*,*,month,satnum] = tb_arr_adjust[*,*,month,satnum] + map_adjust
          
        endif
      endfor
    endfor
    
    
  endif



end

  