;@***h* MLT_V4_RSS/calc_offsets_from_grpt_maps_v4_land_ocean_sep
; NAME
;   calc_offsets_from_grpt_maps_v4_land_ocean_sep
;
; PURPOSE
;   Calculates offsets for each latitude.
;
; DESCRIPTION
;   Calculates offsets for each latitude.  Offsets for Land and Ocean parts of the latitude band are calculated separately.  The
;   best reference for this calculation is
;
;   Mears, C. A., & Wentz, F. J. (2016). Sensitivity of Satellite-Derived Tropospheric Temperature
;   Trends to the Diurnal Cycle Adjustment. Journal of Climate, 29, 3629-3646. doi: 10.1175/JCLI-D-15-0744.1
;
; INPUTS
;   tb_arr_adj            fltarr array (144 x 72 x num_months x num_sats) of adjusted antenna temperatures
;   months_to_use         intarr array (num_sats x num_months) of months to use in subsequent processing.
; OUTPUTS
;   offsets               fltarr array (72 x num_sats x 2) of offsets values
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   1/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   calc_offsets_from_grpt_maps_v4_land_ocean_sep,tb_arr_adj2,months_to_use,offsets
;
;@*****

pro calc_offsets_from_grpt_maps_v4_land_ocean_sep,tb_arr_adj,months_to_use,offsets

  read_land_fraction_144_72,lf
	; compute the offsets separately for each latitude, adn separately for land and ocean
	

  offsets = replicate(!values.f_nan,72,9,2)
  stddev_resids = replicate(!values.f_nan,72,2)
  num_lons_good = intarr(72,2)
  
	for land_ocean = 0,1 do begin
  	sigma_est = 1.0
  	sz = size(tb_arr_adj,/dimensions)
  
  	num_lons   = sz[0]
  	num_lats   = sz[1]
  	num_months = sz[2]
  	num_sats   = sz[3]
  
  	bad = where(months_to_use eq 0)


  	for ilat = 3,68 do begin
  	 

  	  if land_ocean eq 0 then begin  ; ocean
  	   lon_ok = where(lf[*,ilat] lt 0.2,ct)
  	  endif else begin
  	   lon_ok = where(lf[*,ilat] gt 0.8,ct)
  	  endelse
  	  num_lons_good[ilat,land_ocean] = ct
  	  if ct gt 0 then begin
  	    tb_arr_lat = replicate(!values.f_nan,num_months,num_sats)
  	    
        for month = 0,num_months-1 do begin
  			  for satnum = 0,num_sats-1 do begin
            tb_arr_lat[month,satnum] = mean(tb_arr_adj[lon_ok,ilat,month,satnum],/nan)
          endfor
        endfor
        
        tb_arr_lat[bad] = !values.f_nan
        num_col = num_sats
        X = fltarr(num_col,10000L)
        Y = fltarr(10000L)
        row_num = 0
        if ilat eq 30 then begin
          print
        endif
  		  for month = 0,num_months-1 do begin
  			  tbs = reform(tb_arr_lat[month,*])
          ok = where(finite(tbs),ct)
  			  if ct ge 2 then begin
  				  for j = 0,ct-2 do begin
  				    row = fltarr(num_col)
  					  sat1 = ok[j]
  					  sat2 = ok[j+1]
  					  row[sat1] = 1.0
  					  row[sat2] = -1.0
  					  X[*,row_num] = row
  					  Y[row_num] = tbs[sat1]-tbs[sat2]
  					  row_num = row_num+1
  				  endfor
  			  endif
  		  endfor
  
  		  if row_num gt 10 then begin
  			  X = X[*,0:row_num-1]
  			  Y = Y[0:row_num-1]
  			  sigma_in = replicate(sigma_est,row_num)
  
  			  lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,trunc_ct = trunc_ct
  			  offsets[ilat,*,land_ocean] = float(a)
  			  stddev_resids[ilat,land_ocean] = stddev(y-fit)
  		  endif else begin
  			  offsets[ilat,*,land_ocean] = replicate(!values.f_nan,num_sats)
  		  endelse
  	  endif
    endfor
endfor

if keyword_set(plt_offsets) then begin
		set_win
		!p.multi = [0,1,1]
		ok = where(finite(offsets))
		mx = 1.0
		mn = -1.0
		lats = -88.75+ findgen(72)*2.5
		
		for land_ocean = 0,1 do begin
		  if !D.name eq 'WIN' then window,land_ocean
		  plot,lats,fltarr(72),yrange=[mn,mx],title='Offsets of a function of latitude, RAW'
		  for i = 0,8 do begin
			 oplot,lats,offsets[*,i,land_ocean],color=1+i*30
			ENDFOR
		endfor
	endif
end