;@***h* MLT_RSSv4/calc_TF_from_grpt_maps_v4.pro
;
; NAME
;   calc_TF_from_grpt_maps_v4.pro
;
; PURPOSE
;   calculates target factors from MSU radiance data and target temperature
;
; DESCRIPTION
;   calculates target factors from MSU radiance data and target temperature.  
;   Certain months/satellite are excluded as controlled by the months to use array, the num_thres value, 
;   and the sats_to_use array
;
; INPUTS
;   tb_arr            array (144,72,336,9( reals containing monthly radiance maps for each of 9 satellites
;   tar_arr           array (144,72,336,9( reals containing monthly target temperature maps for each of 9 satellites
;   num_arr           array (144,72,336,9( intengers containing monthly number of observations maps for each of 9 satellites
;   channel           string containing the channel name ('TLT','TMT','TTS','TLS')
;   sats_to_use       array of 9 integers.  Set value to one to use that satellite
;   months_to_use     array of (336,9) integers.  Set value to one to use that satellite-month, set to -1 to excluded that satellite-month
;   num_thres=num_thres   integer that is the threshold for number of observations for monthly mean to be valid
;   tf_zero_wt             weight used to "pull" target factors to zero for TLT.
;   use_AMSU              set to one to use AMSU data to help determine alpha for NOAA-14 (only used for TLT)
;   alpha_in          array of 9 real numbers.  If a valid (not NaN) at a given location, then target factor is 
;                     set to this value for that satellite.  Not generally used.
;    surf_str         string decribing what surface type to use.
;   
; OUTPUTS
;   target_factor           array of 9 target factors
;   target_means            array of 9 target means
;   offset                  array of 9 offsets
;   months_to_use           array of (336,9) integers.  if equal to 1, then that sat/month is used. 
;   covar=covar             array of 17x17 reals.  Covariaince matrix from regression
;   sigma_target_factor     array of 9 reals, uncertainty of target factors.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   5/15/2018 Initial version submitted to NCEI
;
; USAGE
;   calc_TF_from_grpt_maps_v4,tb_arr,tar_arr,num_arr,channel,sats_to_use,$
;            target_factor,target_means,offset,months_to_use,             $
;            num_thres=num_thres,covar=covar,       $
;            surf_str = surf_str,,use_AMSU = use_AMSU,sigma_target_factor = sigma_target_factor

;
;@*****



pro calc_TF_from_grpt_maps_v4,tb_arr,tar_arr,num_arr,channel,sats_to_use,$
            target_factor,target_means,offset,months_to_use,             $
            num_thres=num_thres,covar=covar,       $
            surf_str = surf_str,tf_zero_wt=tf_zero_wt,alpha_in = alpha_in,use_AMSU = use_AMSU,sigma_target_factor = sigma_target_factor

	; This routine calculates the target factors directly from the grid point maps.  The in put data is
	;
	;	  tb_arr   (should be a (144,72,336,9) array of brightness temperatures
	;   tar_arr  (should be a (144,72,336,9) array of target temperatures
	;   num_arr  (should be a (144,72,336,9) array of observation numbers
	;   channel  should be a 3 character channel string (TLT,TMT, etc)
	;   sats_to_use (9) array of integers  ) to ignore this satellite, 1, to use it
	;
	; Optional inputs
	;   surf_str
	;      set this to 'Land_and_Sea', 'Land', or 'Sea' to control which locations are used to
	;      calculate the target factors.  Default is 'Land_and_Sea' if not specified.

	; Output data
	;	  target_factor  (9) array of target factors
	;	  target_mean    (9) array of mean target temperature
	;   months_to_use  (336,9) array of months to use in the merge.

	; Note that this routine has two important functions:
	;	First, it calculates the target factors and target means
	;	Second, it decides which satellite/months to use in the rest of the merge.
	;	A few bad months are removed by hand at this point.  

	
	if keyword_set(alpha_in) eq 0 then begin
	 alpha_in = nanarr(9)
	endif
	
	if n_elements(alpha_in) ne 9 then begin
	 alpha_in = nanarr(9)
	 print,'Warning:  alpha_in must have 9 elements, set to nanarr(9)'
	endif
	
	if keyword_set(use_AMSU) then begin
	 AMSU_L3_Path = 'B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\TLT_results\'
	 amsu_version_num = 42256L
	 amsu_file = AMSU_L3_Path+'RSS_Tb_Maps_ch_TLT_V4_0_'+string(amsu_version_num,format='(i7.7)')+'.nc'

   read_monthly_amsu_tbs_netcdf_4_0,file = amsu_file,               $
     longitude = longitude,        $
     latitude  = latitude,         $
     long_bounds = long_bounds,    $
     lat_bounds  = lat_bounds,     $
     months = months,              $
     years  = years,               $
     mean_tb_maps = amsu_tbs_temp, $
     fov_wts = amsu_fov_wts,            $
     offsets=amsu_offsets,              $
     target_factors=amsu_target_factors,$
     target_means = amsu_target_means,  $
     tb_Factors=amsu_tb_factors,        $
     satellites_used = amsu_satellites_used_temp,           $
     date_L2C_processed = date_L2C_processed,     $
     global_Attributes = amsu_global_Attributes, $
     msu_amsu_diff = msu_amsu_diff
     
   sz = size(amsu_satellites_used_temp,/dimensions)
   num_months = sz[1]+240
   num_sats = sz[0]
   amsu_tbs = replicate(!values.f_nan,144,72,num_months)
   amsu_satellites_used = bytarr(num_sats,num_months)
   amsu_satellites_used[*,240:num_months-1] = amsu_satellites_used_temp
   amsu_tbs[*,*,240:num_months-1] = amsu_tbs_temp
   sz_amsu = size(amsu_tbs,/dimensions)


	endif
	
	months_to_use_in = months_to_use
	
	sigma_est = 1.0
	lat_range=[-50,50]
	sz = size(tb_arr,/dimensions)

	num_months = sz[2]
	num_sats = sz[3]

	;calculate tropical/global averages for the tb_arr,tar_arr, and the num_arr

	trop_tb_arr  = replicate(!values.f_nan,num_months,num_sats)
	trop_tar_arr = replicate(!values.f_nan,num_months,num_sats)
	trop_num_arr = replicate(0.0,num_months,num_sats)
	months_to_use = intarr(num_months,num_sats)

	lat_wts = latitude_wts(72,lat_range)

  if (keyword_set(surf_str) eq 0) then surf_str = 'Land_and_Sea'
  
	max_alt = 10000
	if channel eq 'TLT' then max_alt = 3000

	for satnum = 0,num_sats-1 do begin
		for month = 0,num_months-1 do begin
			map = tb_arr[*,*,month,satnum]
			ok = where(finite(map) eq 1,ct)
			if ct lt 5000 then continue
			
			
			trop_tb_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)

			map = tar_arr[*,*,month,satnum]
			trop_tar_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)

			map = num_arr[*,*,month,satnum]
			trop_num_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)
		endfor
	endfor
	
	
	
	if keyword_set(use_AMSU) then begin
	 amsu_tb_arr = nanarr(num_months)

	 for month = 0,num_months-1 do begin
	    map = amsu_tbs[*,*,month]
	    ok = where(finite(map) eq 1,ct)
	    if ct lt 5000 then continue
      amsu_tb_arr[month] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)
	  endfor
	endif
	
	months_to_use = intarr(num_months,num_sats)
	ok = where(trop_num_arr gt num_thres)
	months_to_use[ok] = 1
	
	include_for_sure = where(months_to_use_in eq 1,ct)
	if ct gt 0 then months_to_use[include_for_sure] = 1
	
	exclude_for_sure = where(months_to_use_in eq -1,ct)
	
	if ct gt 0 then months_to_use[exclude_for_sure] = 0
  bad = where(months_to_use eq 0,ct)
    if ct gt 0 then begin
    	trop_tb_arr[bad] = !values.f_nan
    	trop_tar_arr[bad] = !values.f_nan
    endif

	;remove means from target factors, and save means for passing back up

	target_means = fltarr(9)
	for satnum = 0,num_sats-1 do begin
		tar = trop_tar_arr[*,satnum]
		target_means[satnum] = mean(tar,/nan)
		trop_tar_arr[*,satnum] = trop_tar_arr[*,satnum]-target_means[satnum]
	endfor
	
	if keyword_set(use_AMSU) then num_sats = num_sats+1

	num_col = num_sats*2
	X = fltarr(num_col,10000L)
	Y = fltarr(10000L)
	s1 = intarr(10000L)
	s2 = intarr(10000L)
	m  = intarr(10000L)
	row_num = 0
	

	for month = 0,num_months-1 do begin
		tbs = reform(trop_tb_arr[month,*])
		ok = where(finite(tbs),ct)
		if ct ge 2 then begin
			for j = 0,ct-2 do begin
			  row = fltarr(num_col)
				sat1 = ok[0]
				sat2 = ok[1]
				row[sat1] = 1.0
				row[sat2] = -1.0
				row[sat1+num_sats] = trop_tar_arr[month,sat1]
				row[sat2+num_sats] = -trop_tar_arr[month,sat2]
				X[*,row_num] = row
				Y[row_num] = tbs[sat1]-tbs[sat2]
				s1[row_num] = sat1
				s2[row_num] = sat2
				m[row_num] = month
				row_num = row_num+1
			endfor
		endif
	endfor
	
	if keyword_set(use_AMSU) then begin   ; add in equations for noaa-14/ merged AMSU
	   for month = 0,num_months-1 do begin
	     tb_14 = trop_tb_arr[month,8]
	     tb_15 = amsu_tb_Arr[month]
	     if ((finite(tb_14) gt 0) and (finite(tb_15) gt 0)) then begin
	       row = fltarr(num_col)
	       row[8] = 1.0
	       row[9] = -1.0
	       row[8 + num_sats] = trop_tar_arr[month,8]
	       row[9 + num_sats] = 0.1
	       X[*,row_num] = row
	       Y[row_num] = tb_14-tb_15
	       s1[row_num] = 8
	       s2[row_num] = 9
	       m[row_num] = month
	       row_num = row_num+1
	       
	     endif
	   endfor
	   alpha_in = [alpha_in,0.0]
	endif
	
	; for TLT add rows that hold the target factors to
	; lower absolute values
	
	sz = size(x,/dimensions)

	if channel eq 'TLT' then begin
	  for j = 0,8 do begin
	    if finite(alpha_in[j]) eq 0 then begin ; only do for alphas that are not set.
	      row = fltarr(num_col)
	      row[j+num_sats] = tf_zero_wt
	      X[*,row_num] = row
	      Y[row_num] = 0.0
	      m[row_num] = -1
	      row_num=row_num+1
	    endif
	  endfor
	endif
	
	; remove columns where target factors are set
	
	columns_to_delete = []
	for sat = 0,num_sats-1 do begin
	 if finite(alpha_in[sat]) eq 1 then begin
	   column_to_delete = num_sats + sat
	   y = y - alpha_in[sat]*x[column_to_delete,*]
	   columns_to_delete = [columns_to_delete,column_to_delete]
	 endif
	endfor
	
	columns_to_delete = reverse(columns_to_delete)
  
	foreach column_to_delete,columns_to_delete do begin
	 sz = size(x,/dimensions) 
	 x_new_started = 0
	 if column_to_delete gt 0 then begin
	   x_new = x[0:column_to_delete-1,*]
	   x_new_started = 1
	 endif
	 if column_to_delete lt sz[0]-1 then begin
	   temp = x[column_to_delete+1:sz[0]-1,*]
	   if x_new_started then begin
	     x_new = [x_new,temp]
	   endif else begin
	     x_new = temp
	   endelse
	 endif
   x = x_new
 endforeach
	   
	

	X = X[*,0:row_num-1]
	Y = Y[0:row_num-1]
	s1 = s1[0:row_num-1]
	s2 = s2[0:row_num-1]
	m  = m[0:row_num-1]
	sigma_in = replicate(sigma_est,row_num)

	lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,trunc_ct = trunc_ct

	offset = float(a[0:num_sats-1])
	n = n_elements(a)
	if n gt 9 then begin
	   target_factor_temp = float(a[num_sats:n-1])
	endif
	j = 0
	target_factor = nanarr(num_sats)
	for sat = 0,num_sats-1 do begin
	  if finite(alpha_in[sat]) eq 1 then begin
      target_factor[sat] = alpha_in[sat]
    endif else begin
      target_factor[sat] = target_Factor_temp[j]
      j = j + 1
    endelse
	endfor
	
	if keyword_set(use_AMSU) then begin
   offset = offset[0:num_sats-2]
   target_factor = target_factor[0:num_sats-2]
  endif
  
	; scale the covariance matrix
	; the original covariance matrix was calculated using a sigma est of 1.0
	; we find a new sigma est from the after the fit residuals
	
	sigma_est_ATF = stddev(y-fit)
	covar = covar*sigma_est_ATF*sigma_est_ATF
  

end
