;@***h* MLT_V4_RSS/msu_v4_grpt_merge.tlt.pro
; NAME
;   MLT_V4_RSS/msu_v4_grpt_merge.tlt.pro
;
; PURPOSE
;   top level program that performs TLT merge for MSU
;
; DESCRIPTION
;   top level program that performs TMT merge for MSU.  The merging process is described in
;
;   Mears, C. A., & Wentz, F. J. (2016). Sensitivity of Satellite-Derived Tropospheric Temperature
;   Trends to the Diurnal Cycle Adjustment. Journal of Climate, 29, 3629-3646. doi: 10.1175/JCLI-D-15-0744.1
;
;   The program defines several common block, then read in data and performs simple QC analysis to
;   remove data that is too sparse for a given month/satellite.  The program then calculates the various merging parameters.
;   They are:
;     Target Factors
;     Adjustments to the diurnal correction
;     Intersatellite offsets
;
;   The merged data product is then written.
; INPUTS
;   input variables
; OUTPUTS
;   output variable and descriptions
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   1/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   .r MLT_V4_RSS/msu_v4_grpt_merge.tlt.pro
;
;@*****

tb_arr      = 0 
tar_arr     = 0
tb_clim_arr = 0
num_arr     = 0


print_global_pars = 1
fix_colors,colors=colors
!p.background = colors.white
!p.color = colors.black
;goto,skip

initialize_MSU_System_Variables_V4
read_land_fraction_144_72,lf
land_mask = intarr(144,72)
land = where(lf gt 0.5)
land_mask[land] = 1.0

defsysv,'!MSU_L3_Path',     'E:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_MSU_FOV_V4\results_tlt\'
defsysv,'!MSU_L3_Plot_Path','E:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_MSU_FOV_V4\reports_tlt\'

use_AMSU = 1
use_TMT_TFs = 0
apply_left_right_adjustments=1

tf_zero_wts = [0.0,0.5,1.0,1.5,2.0,2.5]
for TF_zero_factor_index = 1,5 do begin
  tf_zero_wt = tf_zero_wts[TF_zero_factor_index]
  zero_tf = 0
  channel_str = 'TLT'
  eia_model = 'MERRA' & eia_index = 1
  diurnal_models = ['NONE','CCM3','HADGEM','MERRA','CMAM','NONE','CCM3','HADGEM','MERRA','CCM3']
  
  offset_scheme = 'land_ocean_separate'
  ;offset_scheme = 'land_ocean_together'
  ;offset_scheme = 'ocean_only'
  case offset_scheme of
    'land_ocean_together': begin
        offset_scheme_index = 1
    end
      'land_ocean_separate': begin
        offset_scheme_index = 2
    end
      'ocean_only': begin
        offset_scheme_index = 3
    end
  endcase
  map_stddev_all = fltarr(10,2,3)
  
  tf_schemes = ['land_and_ocean']
  ;tf_schemes = ['ocean_only']
  ;tf_schemes = ['none']
  foreach tf_scheme,tf_schemes do begin
  
    case tf_scheme of
      'land_and_ocean':begin
        tf_surf = 'Land_and_Sea'
        tf_scheme_index = 1
        zero_tf = 0
       end
      'ocean_only':begin
        tf_surf = 'Sea'
        tf_scheme_index = 2
        zero_tf = 0
       end
       'none':begin
         tf_surf = 'Sea'
         tf_scheme_index = 3
         zero_tf = 1
       end
    endcase
     
    ;foreach diurnal_index,[0,1,2,3,5,6,7,8] do begin
    foreach diurnal_index,[6] do begin
      if diurnal_index eq 4 then continue
    
      adjust_diurnal = 0
      if diurnal_index le 4 then begin
        diurnal_model = diurnal_models[diurnal_index]
        modify_diurnal = 0
      endif
  
      if diurnal_index ge 5 then begin
        adjust_diurnal = 1
        diurnal_model = diurnal_models[diurnal_index]
      endif
    
      if diurnal_index ge 9 then begin
        adjust_diurnal = 2
        diurnal_model = diurnal_models[diurnal_index]
      endif
      
    	;channel_str = channel
    	
    	version_num = (10L*eia_index ) + (10000L*TF_zero_factor_index) + (1000L*offset_scheme_index) + (100L*tf_scheme_index)  +  diurnal_index

      if keyword_set(use_AMSU) then version_num = version_num + 100000L
      if keyword_set(use_TMT_TFs) then version_num = version_num + 200000L
      if keyword_set(apply_left_right_adjustments) then version_num = version_num + 400000L
      
    	alpha_in = nanarr(9)
    	;alpha_in[8] = 0.06
    	
      log_path    = 'C:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_MSU_FOV_V4\logs_tlt\'
      ok = check_path(log_path,/create)
        
    	print,'Version Number = ',version_num
    	defsysv,'!log_lun',2
    	log_file = string(log_path,channel_str,version_num,format='(a,"merge_log_fov_amsu_channel_",a,"_v4.",i7.7,".log.txt")')
    	close,!log_lun
    	openw,!log_lun,log_file
    	
      read_msuv2_months_to_exclude,channel_str,months_to_use_v4,error	
      get_msu_months_used_from_V3_3,channel_str,months_to_use_v3,error
         
    	tb_arr          = !values.f_nan
    	tb_arr_adj      = !values.f_nan
    	adjustment_flag = 7 ; setting this flag to this value for the netcdf write routine -- has no other effect
    	
      read_all_adjusted_MSU_maps_TLT,       diurnal_model = diurnal_model,       $
                                            eia_model = eia_model,               $
                                            fov_ok = fov_ok,                     $                                          
                                            tb_arr = tb_Arr,                 $
                                            raw_tb_arr = tb_Arr_raw,             $
                                            tar_arr = tar_arr,               $
                                            num_arr = num_arr,               $
                                            file_list=file_list,apply_left_right_adjustments=apply_left_right_adjustments
                                       	
    		sz = size(tb_arr,/dimensions)
    		num_months = sz[2]
    		num_sats = sz[3]
    		
    		; find the target factors from grid_point data.  The adjusted data is returned in tb_arr_adj.
    		; the target factors are contained in the merge_parameters variable.
    		
        num_thres = 200 ; try this value -- TEMP!
        if channel_str eq 'TLT' then num_thres = 64
    		months_to_use = months_to_use_V4
        months_to_use[24,1] = 1
        months_to_use[39,1] = 1
        months_to_use[85,2] = 1
        months_to_use[130,6] = -1
    		if keyword_set(use_star) ge 1 then begin
    		  months_to_use = intarr(336,9)
    		  months_to_use[60:62,1] = -1
    		  months_to_use[85,2] = -1
    		  months_to_use[228:335,6] = -1
    		endif
  
        if use_TMT_TFs then begin
          ;             TIROS-1   NOAA-06    NOAA-07    NOAA-08     NOAA-09     NOAA-10     NOAA-11      NOAA-12     NOAA-14
          alpha_in = [-0.00392022,0.00873068,0.00605351,0.00935464,-0.00494584,-0.00504622,-0.000787720,-0.00307802,-0.00636080]
        endif
    		calc_TF_from_grpt_maps_v4,  tb_arr,         $
    		  tar_arr,             $
    		  num_arr,             $
    		  channel_str,         $
    		  sats_to_use,         $
    		  target_factors,      $
    		  target_means,        $
    		  offsets,             $
    		  months_to_use,       $
    		  num_thres=num_thres, $
    		  trop_tb_arr = trop_tb_arr,   $
    		  trop_tar_arr = trop_tar_arr, $
    		  surf_str = tf_Surf,  $
    		  alpha_in = alpha_in, $
    		  use_AMSU = use_AMSU, $
    		  sigma_target_factor = sigma_target_factor, $
    		  tf_zero_wt=tf_zero_wt  ; only matters for TLT
   
        ;save,file = 'B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_MSU_FOV_V4\MSU_V4_grpt_merge\evaluation\target_temperatures.sav',trop_tar_arr
        ;stop
    		;save,file = 'B:\MSU_AMSU\IDL_MSU_AMSU_V4\diur_sensitivity\msu_tar_arr.sav',tar_arr,num_arr,months_to_use
   		  
  ;  		target_factors[8] = 0.02237
  ;  		version_num = version_num + 80000L
    		
    		if keyword_set(zero_Tf) then target_factors = fltarr(9)
    		
    		if keyword_Set(print_global_pars) then begin
    		  log,string(target_factors,format = '("target_Factors = [",8(f8.4,","),f8.4,"]")')
    		  log,string(target_means,format = '("target_means = [",8(f8.4,","),f8.4,"]")')
    		  log,string(offsets,format = '("offsets = [",8(f8.4,","),f8.4,"]")')
    		endif
    		
  
        apply_TF_to_grpt_maps_v4,tb_arr,tar_arr,channel_str,target_factors,target_means,months_to_use,tb_arr_adj
        apply_TF_to_grpt_maps_v4,tb_arr_raw,tar_arr,channel_str,target_factors,target_means,months_to_use,tb_arr_raw_adj
    		
    		if adjust_diurnal eq 1 then begin
    	    set_Win
    	    window,0
    	    !p.multi = [0,1,1]
    	    fix_colors
    	    t = findgen(120)/10.0
    	    plot,t,fltarr(120),yrange=[-1.2,1.2],ystyle=1,xrange=[0,12],xstyle=1
  
    	    ; for TLT and TMT, we use separate diurnal adjustments for land and ocean
    	    ; for TTS and TLS, the land and ocean differences are not used, resulting in one less 
    	    ; array dimension
    	    
    	    if ((channel_str eq 'TLT') or (channel_str eq 'TMT')) then begin
    		    diurnal_residual_parameters_MSU = fltarr(15,2,72)
    		  endif
  
    		  if  ((channel_str eq 'TTS') or (channel_str eq 'TLS')) then begin
    		    diurnal_residual_parameters_MSU = fltarr(15,72)
    		  endif
         
          for ilat = 0,71 do begin
    		    lat_range = [-90.0 + 2.5*(ilat-2),-90.0 + 2.5*(ilat+3)]
    		    if ((channel_str eq 'TMT') or (channel_str eq 'TLT')) then begin 
    		      fit_residual_diurnal_MSU_w_seasonal,tb_arr_adj,lat_range,params,max_alt = 3000
    		      print,lat_range,params
    		      if ilat le 17 then begin
    		        params[*,1] = params[*,0]
    		      endif
    		      diurnal_residual_parameters_MSU[*,*,ilat] = params[*,0:1]
    		      oplot,t,params[9,0]*sin(t*2*!pi/12.0) + params[10,0]*cos(t*2*!pi/12.0),color=1+ilat*2
    		      oplot,t,params[9,1]*sin(t*2*!pi/12.0) + params[10,1]*cos(t*2*!pi/12.0),color=120+1+ilat*2
    		    endif
    		    if ((channel_str eq 'TTS') or (channel_str eq 'TLS')) then begin
    		      print,'This code needs checking'
    		      stop
    		      if ilat eq 1 then log,'Calculating Seasonally modulated diurnal adjustments'
    		      fit_residual_diurnal_MSU_w_seasonal,tb_arr_adj,lat_range,params_w_season
   		        diurnal_residual_parameters_MSU[*,ilat] = params_w_season[*,2]
     		    endif
    	  	endfor
  
    		  save_file = string(channel_str,version_num,format='("B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_MSU_FOV_V4\MSU_V4_Diurnal\diurnal_parameters\msu_diurnal_residual_parameters_chan_",a,"_v",i7.7,".sav")')
    		  save,file=save_file,diurnal_residual_parameters_MSU
    		  log,'Saving Diurnal Adjustment Parameters'
    		  log,save_file
    		  
    		  apply_diurnal_fit_residuals_MSU,tb_arr_adj,diurnal_residual_parameters_MSU,tb_arr_temp
    		  
    		  
          tb_arr_adj2 = tb_arr_temp
          plot_MSU_diurnal_adjustments,tb_arr_raw_adj,tb_arr_adj,tb_Arr_adj2,version_num,plot_path
          tb_arr_adj = tb_arr_adj2

    		endif

    		if adjust_diurnal eq 2 then begin
    		  ; get the adjustment parameters from AMSU
    		  channel_Str_AMSU = 'TMT'
    		  version_num_AMSU = 152246L
    		  save_file = string(channel_str,version_num_AMSU,format='("B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\reports_jclim\diurnal_residual_parameters_chan_",a,"_v",i7.7,".sav")')
    		  restore,save_file,/verbose    ;diurnal_residual_parameters
    		 
    		  diurnal_residual_parameters[0:7,*,*] = 0.0
    		  diurnal_residual_parameters[*,1,0:17] = diurnal_residual_parameters[*,0,0:17]
    		  
    		  apply_diurnal_fit_residuals_MSU,tb_arr_adj,diurnal_residual_parameters,tb_arr_temp
    		  tb_arr_adj = tb_arr_temp
  
    		endif
        
        ps_file =string(!MSU_L3_Plot_Path,channel_str,channel_str,version_num,format='(a,a,"\",a,"_msu_grpt_merge_v2_report.",i7.7,".ps")')
    		log,'Postscript file = '+ps_file
    		set_up_ps,xsize=7.0,ysize=9.0,file = ps_file
  
        if offset_scheme_index eq 1 then begin
          calc_offsets_from_grpt_maps_v4,tb_arr_adj,months_to_use,offsets
          smooth_offsets_TLT,offsets,offsets_smooth
          plot_offsets,offsets,offsets2=offsets_smooth
    		; offsets should be applied in the above version of calc_offsets_from_grpt_maps
          apply_offsets_to_grpt_maps_v4,tb_arr_adj,offsets_smooth
        endif
        
        if offset_scheme_index eq 2 then begin
          calc_offsets_from_grpt_maps_v4_land_ocean_sep,tb_arr_adj,months_to_use,offsets 
          adjust_offsets_by_land_fraction,offsets,offsets_adj
          offsets_adj_sm = offsets_adj
          smooth_offsets,offsets_adj[*,*,0],temp
          offsets_adj_sm[*,*,0] = temp
          smooth_offsets,offsets_adj[*,*,1],temp
          offsets_adj_sm[*,*,1] = temp      
          plot_offsets,offsets,offsets2=offsets_adj_sm      
          apply_offsets_to_grpt_maps_v4_land_ocean_sep,tb_arr_adj,offsets_adj_sm 
        endif
        
        if offset_scheme_index eq 3 then begin
          calc_offsets_from_grpt_maps_v4_ocean_only,tb_arr_adj,months_to_use,offsets
          smooth_offsets,offsets,offsets_smooth
          plot_offsets,offsets,offsets2=offsets_adj_sm
          apply_offsets_to_grpt_maps_v4,tb_arr_adj,offsets_smooth
        endif
        
        calc_tb_clim_Arr,tb_arr_adj,months_to_use,tb_clim_arr
        
    		; find tb_anom factors from the gridded data
    		tb_arr_no_tbf = tb_arr_Adj
    		tb_arr_no_tbf[*,0:2,*,*] = !values.f_nan
    		tb_arr_no_tbf[*,69:71,*,*] = !values.f_nan
    		tb_arr_no_tbf = tb_arr_Adj
    		;calc_single_TbF_from_grpt_map_v4,tb_arr_adj,tb_clim_arr,months_to_use,tb_factors;,plt_tb_factors =plt_tb_factors
    	  sat_pairs = [[0,1],[1,2],[2,3],[1,4],[4,5],[5,6],[6,7],[6,8],[7,8]]
    	  
    	  eval_global_grpt_merge, tb_arr_Adj,       $
    	    num_arr,                                $
    	    months_to_use,                          $
    	    raw_Tbs = tb_arr,                       $
    	    mean_rms = mean_rms,                    $
    	    mean_Stddev = mean_stddev,              $
    	    stddev_diffs = stddev_diffs,            $
    	    stddev_diff_all = stddev_diff_all,      $
    	    mean_diffs = mean_diffs,                $
    	    trend_diffs = trend_diffs,              $
    	    number_of_months =  number_of_months , $
    	    yearly_trend_diffs = yearly_trend_diffs,$
    	    plot_nums = plot_nums,                  $
    	    window_base_num = window_base_num,      $
    	    version_num = version_num,              $
    	    channel_str = channel_str,              $
    	    sat_pairs = sat_pairs,                  $
    	    plot_path = plot_path,                  $
    	    tar_arr = tar_arr

    	  sat_pairs2 = [[5,6],[6,7],[6,8],[7,8]]
    	  
    	  tb_factors = fltarr(9)
    	  if keyword_set(use_star) then begin
          plot_path = 'E:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_MSU_FOV_V4\MSU_V4_grpt_merge\evaluation_plots_Star\'
        endif else begin
          plot_path = 'E:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_MSU_FOV_V4\MSU_V4_grpt_merge\evaluation_plots_TLT\'
        endelse
        ok = check_path(plot_path,/create)
    	  eval_global_grpt_merge_like_amsu_v4, tb_arr_Adj,                                 $
    		  num_arr,                                $
    		  months_to_use,                          $
    		  raw_Tbs = tb_arr,                       $
    		  mean_rms = mean_rms,                    $
    		  mean_Stddev = mean_stddev,              $
;    		  stddev_diffs = stddev_diffs,            $
;    		  mean_diffs = mean_diffs,                $
;    		  trend_diffs = trends_diffs,             $
;    		  yearly_trend_diffs = yearly_trend_diffs,$
    		  plot_nums = plot_nums,                  $
    		  window_base_num = window_base_num,      $
    		  version_num = version_num,              $
    		  channel_str = channel_str,              $
    		  sat_pairs = sat_pairs2,                  $
    		  plot_path = plot_path
    		  
    		 
    	  eval_grpt_merge_hoffmuller_v4,tb_arr_adj,num_arr,months_to_use
    	  eval_grpt_merge_regional_v4,tb_arr_adj,num_arr,months_to_use
    		eval_grpt_merge_map_v4,   tb_arr_adj,num_arr,months_to_use,diff_maps = diff_maps,adj_diff_maps=adj_diff_maps,stddev_metrics = map_metrics
    		
    		save_file =string(!MSU_L3_Path,channel_str,version_num,format='(a,"adjusted_tbs_",a,"_v",i7.7,".sav")')
    		print,'Adjusted Tb save file: '+save_file
        save,file = save_file,tb_arr_adj,tar_arr,num_arr,target_factors,sigma_target_factor,stddev_diffs,stddev_diff_all,trend_diffs,number_of_months
                                   
    		print,'stddev map_metrics = ',map_metrics
    		
    		;map_stddev_all[diurnal_index,tf_scheme_index-1,*] = map_metrics
    		tb_arr_merged = replicate(!values.f_nan,144,72,336)
    		print,'Merging'
    		pixel_thres = 10
    		for i = 0,num_months-1 do begin
    			tot_tb = fltarr(144,72)
    			tot_num = fltarr(144,72)
    			for satnum = 0,8 do begin
    				if months_to_use[i,satnum] lt 1 then continue
    				tb = tb_arr_adj[*,*,i,satnum]
    				num = num_arr[*,*,i,satnum]
    				ok = where((finite(tb) gt 0) and (num gt pixel_thres),ct)
    				if ct gt 5000 then begin
    					tot_tb[ok] = tot_tb[ok] + tb[ok]*num[ok]
    					tot_num[ok] = tot_num[ok] + num[ok]
    				endif
    			endfor ; satnum
    			tb_merged = tot_tb/tot_num
    			tb_arr_merged[*,*,i] = tb_merged
    		endfor
    
    		tb_arr_merged[*,*,324:335] = !values.f_nan
    		tb_arr_merged[*,*,0:9] = !values.f_nan
    
    		tf_array = fltarr(72,9)
    		tbf_Array = fltarr(72,9)
    		for ilat = 0,71 do begin
    			tf_array[ilat,*]  = target_factors
    			tbf_array[ilat,*] = tb_factors
    		endfor
    		
    		compare_new_merged_msu_to_baseline,tb_arr_merged,tf_array,months_to_use,version_num,channel = channel_str
        device,/close
    		log,'Writing netcdf file'
    		
      ; expand merge coefficients to maps for metadata
    		offset_arr = replicate(!values.f_nan,144,72,num_sats)
    		tarmult_arr = replicate(!values.f_nan,144,72,num_sats)
    		tbmult_arr = fltarr(144,72,num_sats)
    		
    		
    		;need to fix this eventually - -just expanding offsets for metadata.
    		for sat = 0,num_sats-1 do begin
    		  tarmult_arr[*,*,sat] = target_factors[sat]
    		  tbmult_arr[*,*,sat]  = tb_factors[sat]
    		endfor
    		
    		sz_offsets = size(offsets,/dimensions)
    		
    		if n_elements(sz_offsets) eq 2 then begin
      		for ilon = 0,143 do begin
      		  offset_arr[ilon,*,*] = offsets
      		endfor
      	endif else begin  ; sz offsets should be [72,9,2]
      	 bad = where(finite(offsets[*,*,1]) eq 0,ct)
      	 if ct gt 0 then begin
      	   temp1 = offsets[*,*,1]
      	   temp2 = offsets[*,*,0]
      	   temp1[bad] = temp2[bad]
      	   offsets[*,*,1] = temp1
      	 endif
      	 for sat = 0,num_sats-1 do begin
      	  for ilat = 0,71 do begin
      	    offset_arr[*,ilat,sat] = offsets[ilat,sat,0]*(1.0-lf[*,ilat]) + offsets[ilat,sat,1]*lf[*,ilat]
      	  endfor
      	 endfor
      	endelse
    								
    	  write_msu_only_merged_Tbs_144_72_netcdf_4_0,     $
    								  Tbs_144_72 = tb_arr_merged ,   $
    								  channel_str=channel_str,    $
    								  version_num=version_num,    $
    								  diurnal_model = diurnal_model,    $
    								  eia_model = eia_model,            $
    								  fov_wts = fov_wts,                     $
    								  node_wts = [1.0,1.0],                   $
    								  adjustment_flag = adjustment_flag,      $
    								  offsets = offset_arr,        $
    								  target_factors = tarmult_arr, $
    								  target_means = target_means,           $
    								  tb_factors = tbmult_arr,        $
    								  months_used = months_to_use,             $
    								  date_L2C_processed = date_L2C_processed,  $
    								  output_path = !MSU_L3_Path
  
  endforeach
  device,/close
  endforeach
endfor
end
