;@***h* MLT_RSSv4/read_all_adjusted_MSU_maps_from_FOV.pro
;
; NAME
;   read_all_adjusted_MSU_maps_from_FOV.pro
;
; PURPOSE
;   reads all msu L2C V4 gridded monthly files.
;
; DESCRIPTION
;   reads all msu L2C V4 gridded monthly files.  
;   Includes angle and diurnal adjustments via addition.
;   Combines data from different FOVs together.
;
; INPUTS
;   channel_str         string containing channel name ('TMT','TTS','TLS')
;   diurnal_model       string containing diurnal model to use ('CCM3','HADGEM','MERRA','NONE')
;   eia_model           string containing Earth incidence model to use ('NCEP','MERRA')
;   fov_ok              array of 11 integers.  Set to one tp use in FOV-combined results
; OUTPUTS
;   tb_arr              array of reals, [144,72,336,9] of adjusted radiances
;   raw_tb_arr          array of reals, [144,72,336,9] of unadjusted radiances
;   tar_arr             array of reals, [144,72,336,9] of target temperatures
;   num_arr             array of reals, [144,72,336,9] of number of observations in each cell
;   file_list           array of strings, names of each file included.
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.
;   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY
;   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
;   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   10/30/2013 Initial version written
;
; USAGE
;   read_all_adjusted_MSU_maps_from_FOV, channel_str = channel_str,diurnal_model = diurnal_model,eia_model = eia_model,fov_ok                    $                                          
;                                        tb_arr,raw_tb_arr,tar_arr,num_arr = num_arr_out,file_list=file_list
;
;@*****

pro read_all_adjusted_MSU_maps_from_FOV, channel_str = channel_str,           $
                                          diurnal_model = diurnal_model,       $
                                          eia_model = eia_model,               $
                                          fov_ok = fov_ok,                     $                                          
                                          tb_arr = tb_Arr_out,                 $
                                          raw_tb_arr = tb_Arr_raw_out,         $
                                          tar_arr = tar_arr_out,               $
                                          num_arr = num_arr_out,               $
                                          file_list=file_list
                                          
;  Reads in entire MSU data set for a given diurnal and eia model.
;  fovs are average over the list of fovs in fov_ok -- default is central 5
;  Note that the residual cross-track adjustments are applied in this routine, and
;  that read_msu_nadir_adjustment_fov returns adjustments WITHOUT the cross track adjustment applied.
                                          
if keyword_set(eia_model) eq 0 then eia_model = 'NCEP'
if keyword_set(diurnal_model) eq 0 then diurnal_model = 'CCM3'
if keyword_set(fov_ok) eq 0 then fov_ok = [3,4,5,6,7]

                                          
    num_months = 336
    num_sats = 9
    tb_arr_out = replicate(!values.f_nan,144,72,num_months,num_sats)
    tb_arr_raw = replicate(!values.f_nan,144,72,num_months,num_sats)
    tar_arr_out = replicate(!values.f_nan,144,72,num_months,num_sats)
    tb_arr_raw_out = replicate(!values.f_nan,144,72,num_months,num_sats)
    tb_clim_arr_out = replicate(!values.f_nan,144,72,12,num_sats)
    num_arr_out = replicate(0.0,144,72,num_months,num_sats)
    
    file_list = []
    
case channel_str of 
  'TLT'  : begin
    start_sat = 1
    end_sat = 9
    channel = 2
  end
  'TMT'  : begin
      start_sat = 1
      end_sat = 9
      channel = 2
   end
  'TTS'  : begin
    start_sat = 6
    end_sat = 9
    channel = 3
    end
  'TLS'  : begin
      start_sat = 1
    end_sat = 9
    channel = 4
    end
endcase
    
read_msuv2_months_to_exclude,channel_str,months_to_use,error
if channel_str eq 'TMT' then months_to_use[85,4] = 0
                                        
begin_year_list = [1978,1979,1981,1983,1985,1986,1988,1991,1995]
end_year_list   = [1982,1986,1985,1986,1988,1991,1998,1998,2005]

tb_arr = replicate(!values.f_nan,144,72,336,9)
file_list = []
for satnum = start_sat,end_sat do begin
  
  print,'Reading Data for '+sat_name(satnum,/fill)
  begin_year     = 1L*begin_year_list[satnum-1]
  end_year       = 1L*end_year_list[satnum-1]
  

  if channel_str ne 'TLT' then begin
    file = string(eia_model,channel,sat_name(satnum,/fill),format = '("M:\MSU_AMSU_Data\MSU_Adjustments\",a,"_Modelled_Tbs\MSU_Channel_",i2.2,"\cross_track_residuals.",a,".diurnal.txt")')
    close,3
    openr,3,file
    print,file
    xtrack_residual = fltarr(11)
    readf,3,xtrack_residual,format = '(11(f9.3,2x))
    close,3
  endif

  for year = begin_year,end_year do begin
    ;print,year,'  '+sat_name(satnum,/fill)
    for month = 1,12 do begin
      month_index = month - 1 + 12*(year-1978)
      if months_to_use[month_index,satnum-1] lt 0 then continue
      if channel_str eq 'TLT' then begin
        read_MSU_monthly_map_TLT,satnum,year,month,tb_arr,err,tb_hist=tb_hist,tb_range=tb_range,file=file,verbose=verbose
      endif else begin
        read_MSU_monthly_map_by_fov,satnum,channel,year,month,tb_arr,err,tb_hist=tb_hist,tb_range=tb_range,file=file,verbose=verbose
      endelse
      if month_index eq 308 then begin
        print,308
      endif
      if (err) eq 0 then begin
        if channel_str eq 'TLT' then begin
          
          
          ; sum over asc/dsc
          tb_arr1 = total(tb_arr,2)  ;sum over asc/dsc
          
          
          tb_left  = reform(tb_arr1[1,0,*,*]/tb_arr1[0,0,*,*])
          tb_right = reform(tb_arr1[1,1,*,*]/tb_arr1[0,1,*,*])
          
          
          ; remove side of swath with large errors
          
          tb_arr1[*,0,*,60:71] = 0.0
          tb_arr1[*,0,*,59] = tb_arr1[*,0,*,59] *0.25
          tb_arr1[*,0,*,58] = tb_arr1[*,0,*,58] *0.5
          tb_arr1[*,0,*,57] = tb_arr1[*,0,*,57] *0.75

          tb_arr1[*,1,*,0:11] = 0.0
          tb_arr1[*,1,*,12] = tb_arr1[*,1,*,12] *0.25
          tb_arr1[*,1,*,13] = tb_arr1[*,1,*,13] *0.5
          tb_arr1[*,1,*,14] = tb_arr1[*,1,*,14] *0.75
          
          ;sum over left/right
          
          tbs = reform(total(tb_arr1[1,*,*,*],2)/total(tb_arr1[0,*,*,*],2))
          tar = reform(total(tb_arr1[2,*,*,*],2)/total(tb_arr1[0,*,*,*],2))
          
        endif else begin
          ; sum over asc/dsc and fov_ok
          tbs = reform(total(total(tb_arr[1,*,fov_ok,*,*],2),2)/total(total(tb_arr[0,*,fov_ok,*,*],2),2))
          tar = reform(total(total(tb_arr[2,*,fov_ok,*,*],2),2)/total(total(tb_arr[0,*,fov_ok,*,*],2),2))
        endelse
        
        tb_arr_adj = reform(tb_arr[1,*,*,*,*])
        tb_arr_raw = tb_arr_adj
        
        if channel_str eq 'TLT' then begin
          read_msu_inc_adjustment_TLT,satnum,year,month,eia_model,nadir_adjustment,error
        endif else begin
          read_msu_nadir_adjustment_fov,satnum,channel,year,month,eia_model,nadir_adjustment,error
        endelse

        nadir_bad = where(finite(nadir_adjustment) eq 0,ct)
        if ct gt 0 then nadir_adjustment[nadir_bad] = 0.0
        
        if channel_str ne 'TLT' then begin
          for fov = 0,10 do begin
            nadir_adjustment[*,fov,*,*] = nadir_adjustment[*,fov,*,*] + xtrack_residual[fov]
            tb_Arr_adj[*,fov,*,*] = tb_arr_adj[*,fov,*,*] + tb_arr[0,*,fov,*,*]*nadir_adjustment[*,fov,*,*]
          endfor
        endif
        
        ;num_fovs_used = n_elements(fov_ok)
        ;tbs_adj = tbs + total(total(nadir_adjustment[*,fov_ok,*,*],1),1)/(2.0*num_fovs_used)  ; num FOV times 2 nodes
        
        if (diurnal_model ne 'NONE') then begin
          
          if channel_str eq 'TLT' then begin
            read_msu_diurnal_adjustment_TLT,satnum,year,month,diurnal_model,diurnal_adjustment,error
          endif else begin
            read_msu_diurnal_adjustment_fov,satnum,channel,year,month,diurnal_model,diurnal_adjustment,error
          endelse
          
          num_fovs_used = n_elements(fov_ok)
          diurnal_adjustment_baseline = diurnal_adjustment
          diurnal_bad = where(finite(diurnal_adjustment) eq 0,ct)
          if ct gt 0 then diurnal_adjustment[diurnal_bad] = 0.0
          
          if channel_str ne 'TLT' then begin
            for fov = 0,10 do begin
              tb_Arr_raw[*,fov,*,*] = tb_arr_adj[*,fov,*,*]
              tb_Arr_adj[*,fov,*,*] = tb_arr_adj[*,fov,*,*] + tb_arr[0,*,fov,*,*]*diurnal_adjustment[*,fov,*,*]
            endfor
          endif
        endif else begin
           ; set to zero if diurnal adjustment is 'NONE'
           
           if channel_str eq 'TLT' then begin
             diurnal_adjustment = fltarr(2,2,144,72)
           endif else begin
             diurnal_adjustment = fltarr(2,11,144,72)
           endelse
        endelse
        
        if channel_str eq 'TLT' then begin
          ; sum over asc/dsc and right/left
          tbs_adj = reform(total(total(tb_arr_adj[*,*,*,*],2),1)/total(total(tb_arr[0,*,*,*,*],3),2))
          ; then add in summed adjustments
          
          adjust1 = reform(total(nadir_adjustment,1)/2.0 + total(diurnal_adjustment,1)/2.0)
          ; weight the adjustment with the same left right weights....
          wts = replicate(1.0,2,144,72)
          
          wts[0,*,60:71] = 0.0
          wts[0,*,59]    = 0.25
          wts[0,*,58]    = 0.5
          wts[0,*,57]  =   0.75
          
          wts[1,*,0:11] = 0.0
          wts[1,*,12] = 0.25
          wts[1,*,13] = 0.5
          wts[1,*,14] = 0.75
          
          adjust1 = wts*adjust1
          adjustment = total(adjust1,1)/total(wts,1)
          
          tbs_adj = tbs_adj + adjustment
        endif else begin
          ; sum over asc/dsc and fov_ok
          ; other channels already had adjustments included
          tbs_adj = reform(total(total(tb_arr_adj[*,fov_ok,*,*],2),1)/total(total(tb_arr[0,*,fov_ok,*,*],3),2))
          tbs_raw = reform(total(total(tb_arr_raw[*,fov_ok,*,*],2),1)/total(total(tb_arr[0,*,fov_ok,*,*],3),2))
        endelse
        tb_arr_out[*,*,month_index,satnum-1]  = tbs_adj
        tb_arr_raw_out[*,*,month_index,satnum-1]  = tbs_raw
        tar_arr_out[*,*,month_index,satnum-1] = tar
        
        if channel_str eq 'TLT' then begin
           ; sum over asc/dsc and right/left
          num_arr_out[*,*,month_index,satnum-1] = total(total(tb_arr[0,*,*,*,*],2),2)
        endif else begin
          ; sum over asc/dsc and fov_ok
          num_arr_out[*,*,month_index,satnum-1] = total(total(tb_arr[0,*,fov_ok,*,*],2),2)
        endelse
        
        file_list = [file_list,file]
      endif ;err eq 0
    endfor  ;month
  endfor    ;year
endfor      ;satnum
        
end
