module AMSU_L2C_Update_Monthly_Maps_TLT


use AMSU_constants
use AMSU_TYPES
use AMSU_Quality
use date_and_time
use NAN_support
use topography
use portlib
use AMSU_Level_2A
use AMSU_Inventory
use AMSU_File_Locations
use AMSU_diurnal_CCM3
use AMSU_diurnal_MERRA
use AMSU_diurnal_HADGEM_SAANE


implicit none

real(4),dimension(8),parameter	:: tlt_wts   = (/-2.64,-1.14,0.44,1.41,1.61,1.17,0.40,-0.25/) ! empirical v3 (high altitude excluded from zonals, Land Franction weighted)
real(4),dimension(8),parameter	:: tlt_wts_r = (/-0.25,0.40,1.17,1.61,1.41,0.44,-1.14,-2.64/) ! empirical v3 (high altitude excluded from zonals, Land Franction weighted)

integer(4),parameter	:: GRPT_TB_REC_LENGTH     = 4 + (7*2*144*72*4)

contains

!@***m* MLT_V4_RSS/update_AMSU_monthly_maps_TLT.f90
!  NAME
!    update_AMSU_monthly_maps_TLT.f90
! 
!  PURPOSE
!    assembles AMSU L2A data into monthly maps
! 
!  DESCRIPTION
!   assembles AMSU L2A data into monthly maps, special case for TLT
!   This code calls code (in the AMSU_L2A module)  that reads AMSU data in orbit by orbit files.  
!   An inventory of these files exists, and needs to be read as well.  Information about the location of the 
!   orbit and inventory files is contained in the AMSU_File_Locations module.

!
!  INPUTS
!	satellite number		integer: satellite number to process
!	channel_str			integer: channel to process
!	first_date_to_process   date_str: first date to process			
!	last_date_to_process    date_str: last date to process			
!
!  OUTPUT
!	error				integer:  error flag.  0 means no error
!	monthly maps of AMSU radiance are written to binary files
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    4/15/2018 Initial Version prepared for NCEI
! 
!  USAGE
!    call update_AMSU_monthly_maps_TLT(sat_num,channel,first_date_to_process,last_date_to_process,error)
!
!@*****

subroutine update_AMSU_monthly_maps_TLT(sat_num,channel,first_date_to_process,last_date_to_process,diurnal_str,error)

	integer(4)				:: sat_num
	integer(4)				:: channel
	type(date_str)			:: first_date_to_process
	type(date_str)			:: last_date_to_process
	character*4             :: diurnal_str
	type(date_str)			:: strt_date
	integer(4)				:: error

!   This version makes TLT maps.  Results are saved by right/left, and ascending/descending rather than
!   by FOV as is done for the TMT, TTS, TLS maps.  Adjustments are performed for correction to nominal EIAs, and
!   for model-based diurnal adjustments.

!   Parameters
!
!	sat num					:: satellite number 10 = NOAA-15, 2 = NOAA-16, etc
!   channel					:: amsu channel number 1 - 16
!   first_date_to_process	:: Date structure containing first dat to process
!   last_date_to_process	:: Date structure containing the last date to process
!   diurnal_str             :: string containing the name of the diurnal adjustment to be applied.
!   error					:: = 0 if everything is OK, no zero otherwise
!
!	Input Files:
!		This code calls code (in the AMSU_L2A module)  that reads AMSU data in orbit by orbit files.  
!		An inventory of these files exists, and needs to be read as well.  Information about the location of the 
!		orbit and inventory files is contained in the AMSU_File_Locations module.



	real(4),dimension(NUM_AMSU_FOVS)	:: Tbs
	
	integer(4)							:: start_month
	logical								:: start_found
	integer(4)							:: inv_rec_num
	
	type(date_str)						::  rec_date
	integer(4)							::  rec_month
	integer(4)							::  strt_orbit
	integer(4)							::  last_orbit
	integer(4)							::  last_month
	integer(4)							::  last_valid_rec
	logical							    ::  last_found
	integer(4)							::  date_Error
	logical(4)                          :: adjust_to_noon
	type(amsu_inventory_str)			:: inv			


	! declaration of input data structures
	
	type(AMSU_L2A_orbit_header)							:: L2A_orbit_header
	type(AMSU_L2A_orbit_data)							:: L2A_orbit

	type(date_str)				 :: swath_date

	integer(4)					 :: orbit_num
	character(len = 7)           :: sat_name
	integer(4)					 :: n_swath
	integer(4)					 :: orbit_error

	real(4)						 :: remainder

	real(4)							  :: target_temp
	real(4),dimension(NUM_AMSU_FOVS)  :: longitude
	real(4),dimension(NUM_AMSU_FOVS)  :: latitude
	real(4),dimension(NUM_AMSU_FOVS)  :: diurnal_adjustment
	real(4),dimension(NUM_AMSU_FOVS)  :: incidence_adjustment
	integer(1),dimension(NUM_AMSU_FOVS)  :: fov_qual
	integer(4),dimension(NUM_AMSU_FOVS)  :: num_good_this_orbit
	real(4)                           :: altitude

	! declaration of gridpoint structures

	real(4),dimension(6,2,2,144,72)	    :: hist	 ! (num/tot_tb/tot_tar/tot_height/tot_diurnal/tot_incidence,asc/desc,left/right,ilon,ilat)
	real(4),dimension(500,2)	        :: hist2	 

	integer(4)			:: ilon
	integer(4)			:: ilat
	integer(4)          :: tb_int

	! Set logical unit numbers for the various files

	integer(4),parameter			:: lu_raw_Tb		= 29	! orbit_files
	integer(4),parameter			:: lu_Tbs_grpt		= 50	! Tbs
	integer(4),parameter			:: lu_log			= 70	! log 
	integer(4),parameter			:: lu_inventory     = 100	! orbit inventory file
	
	real(4)             :: tlt_left,tlt_right
	real(4)             :: diurnal_left,diurnal_right
	real(4)             :: incidence_left,incidence_right

	integer(4)			:: i
	integer(4)			:: fov
	integer(4)			:: asc_desc
	integer(4)			:: left_right
	logical(4)			:: scan_ok

	integer(4)			:: month
	integer(4)          :: month_last

	integer(4)          :: month_of_year
	integer(4)          :: year


	! log file name

	character(len=150)  :: log_file_name
	character(20)		:: ds

	! output file names

	character(len=150)  :: tb_grpt_name
						
	!----------------------------------------------

	!	call init_NAN  ! not need with new compiler

		! open log file
		ds = curr_date_string(error)
		write(log_file_name,110) channel,sat_num,trim(ds)
110     format('M:\MSU_AMSU_Data\AMSU-A_L2C\L2A_to_L2C_Logs\month_update\log_ch',i2.2,'_sat_',i2.2,'_',a,'.maps_TLT.txt')
		open(unit = lu_log,file = log_file_name)
		error = 0

		sat_name = amsu_sat_names_2(sat_num)

	    ! read in inventory file

		call read_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)

		! determine the first orbit and month to use
		! we use the first month that contains the start date, and the last month that contains the
		! end date.  Thus the region of computation is expanded somewhat.

		start_month = calc_month(first_date_to_process,remainder)
		start_found = .false.

		last_valid_rec = -1
		do inv_rec_num = 1,MX_ORBITS
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle

			rec_date   = set_date_2000(inv%end_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			
			if (rec_month .ge. start_month) then
				start_found = .true.
				if (last_valid_rec .gt. 0) then
					strt_orbit = last_valid_rec
				else
					strt_orbit = inv_rec_num
				endif
				strt_date = set_date_2000(inv%start_rd_2000(strt_orbit),error)
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo
			 
		if (.not.(start_found)) then 
			print *, 'start orbit not found for sat num ',sat_num
			return
		endif

		! determine the last orbit to use

		last_month = calc_month(last_date_to_process,remainder)
		last_found = .false.
		last_valid_rec = -1

		do inv_rec_num = 1,MX_ORBITS
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle

			rec_date = set_date_2000(inv%start_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			if (rec_month .gt. last_month) then
				last_found = .true.
				if (last_valid_rec .gt. 0) then
					last_orbit = inv_rec_num
				else
					last_orbit = inv_rec_num
				endif
				last_month = calc_month(set_date_2000(inv%start_rd_2000(last_orbit),error),remainder)
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo

		if (.not.(last_found)) then 
			last_orbit = inv%max_orbit
			last_month = calc_month(set_date_2000(inv%start_rd_2000(last_orbit),error),remainder)
		endif

		print *,strt_orbit,start_month,'   ',date_string(strt_Date)
		print *,last_orbit,last_month,'   ',date_string(set_date_2000(inv%start_rd_2000(last_orbit),error))

		write(lu_log,3351)
3351	format(' Computing Tb GrPt Maps')

		! Initialize a few variables before beginning to read

		i = 1
		error = 0

		hist = 0.0
							
		do orbit_num = strt_orbit,last_orbit
							
			orbit_error = 0
            num_good_this_orbit = 0
			call read_AMSU_L2A_orbit_file_r2(sat_name,					&
										  orbit_num,				& 
										  channel,					&
										  L2A_orbit_header,			&
										  L2A_orbit,				&
										  orbit_error)

			if (modulo(orbit_num,100) .eq. 0) print *,'Processing Orbit ',orbit_num
			write(lu_log,3354)orbit_num,orbit_error
3354        format('Processing Orbit ',i6,1x,'Error: ',i6)

			if ((orbit_error .eq. 0)) then 
				do n_swath = L2A_orbit_header%start_swath_num,L2A_orbit_header%end_swath_num
					if (L2A_orbit%scan_valid(n_swath) .le. 0) cycle  ! skip rest if scan not valid

					asc_desc = L2A_orbit%scan_data(n_swath)%asc_desc
                    
					! set date
						 
					swath_date = set_date_2000(L2A_orbit%scan_Data(n_swath)%scan_rd_2000,date_error)
					
					if (.not. finite(swath_Date%rd_2000)) cycle	 ! skip rest if bad time

					month = calc_month(swath_date,remainder)

					if (month .lt. start_month) cycle	  ! only process swaths in the correct months

					if (i .gt. 2) then
						if (month .gt. month_last) then !write out the monthly hist record for this month

						    year = 1978 + (month_last-1)/12
							month_of_year = month_last-12*(year-1978)

							write(tb_grpt_name,351)trim(AMSU_Monthly_Map_Path_V4),channel,trim(sat_name),diurnal_str,year,month_of_year
					351		format(a,'AMSU_Channel_',i2.2,'\TLT_maps\',a,'\tb_',a,'\y',i4.4,'_m',i2.2,'_TLT_Hist_144_72_V4_01.r3.dat')
                            print *,tb_grpt_name
							open(unit = lu_tbs_grpt,file = tb_grpt_name,form='binary')

							write(lu_Tbs_grpt) month_of_year,year,channel,hist,hist2

							print *,'Wrote file: ',tb_grpt_name
							close(unit=lu_tbs_grpt)
							hist  = 0.0
							hist2 = 0.0
							month_last = month
						endif
					else 
							month_last = month
					endif  ! i gt 2

					i = i+1
					scan_ok = .true.

					Tbs						= L2A_orbit%scan_data(n_swath)%Tbs
					longitude				= L2A_orbit%scan_data(n_swath)%fov_longitude
					latitude				= L2A_orbit%scan_data(n_swath)%fov_latitude 
					target_temp				= L2A_orbit%scan_data(n_swath)%warm_load_temp
					fov_qual                = L2A_orbit%scan_data(n_swath)%fov_quality
					altitude                = L2A_orbit%scan_data(n_swath)%spacecraft_altitude
					
					adjust_to_noon = .false.  ! this means the adjustment will be to local midnight
					
					select case (diurnal_str)
						case('HGS1')
							call calculate_AMSU_diurnal_correction_HADGEM_SAANE(L2A_orbit%scan_data(n_swath),channel,adjust_to_noon,lu_log,error)
						case('CCM3')
							call calculate_AMSU_diurnal_correction_CCM3(L2A_orbit%scan_data(n_swath),channel,adjust_to_noon,error)
						case('MERR')
							call calculate_AMSU_diurnal_correction_MERRA(L2A_orbit%scan_data(n_swath),channel,adjust_to_noon,lu_log,error)
						case('NONE')
						    continue
						case default
							write(lu_log,*)'Error: -- diurnal type ',diurnal_str,' not supported'
							error = -1
							return
					end select
					if (diurnal_str .eq. 'NONE') then 
					    diurnal_adjustment = 0.0
					else
					    diurnal_adjustment   = L2A_orbit%scan_data(n_swath)%diurnal_corr
					endif
					
					incidence_adjustment = L2A_orbit%scan_data(n_swath)%incidence_corr
	
	                call check_tbs_against_bounds(Tbs,channel,fov_qual)

					if ((scan_ok) .and. (month .eq. month_last)) then  ! this ignores data from brief backwards time steps
						tlt_left = 0.0
						diurnal_left = 0.0
						incidence_left = 0.0
						do fov = 1,8
				    	    if ((finite(Tbs(fov))) .and.				&
				    	        (finite(diurnal_adjustment(fov))) .and.    &
				    	        (finite(incidence_adjustment(fov))) .and.    &      
							    (iand(fov_qual(fov),3) .eq. 0) .and.			&
							    (finite(longitude(fov))) .and.			&
							    (finite(latitude(fov)))) then
							        tlt_left       = tlt_left + tlt_wts(fov)*Tbs(fov)
							        diurnal_left   = diurnal_left + tlt_wts(fov)*diurnal_adjustment(fov)
							        incidence_left = incidence_left + tlt_wts(fov)*incidence_adjustment(fov)
						     else
						       tlt_left = NAN
						       diurnal_left = NAN
						       incidence_left = NAN
						    endif
					    enddo
					    
					    
					    if (finite(tlt_left)) then
						    tb_int = floor(tlt_left)
						else
							tb_int = 500
						endif
						if ((tb_int .ge. 1) .and. (tb_int .le. 500)) hist2(tb_int,1) = hist2(tb_int,1) + 1

					    
					    tlt_right = 0.0
					    diurnal_right = 0.0
						incidence_right = 0.0

					    do fov = 23,30
					    	if ((finite(Tbs(fov))) .and.				&
				    	        (finite(diurnal_adjustment(fov))) .and.    &
				    	        (finite(incidence_adjustment(fov))) .and.    &      
						        (iand(fov_qual(fov),3) .eq. 0) .and.			&
							    (finite(longitude(fov))) .and.			&
								(finite(latitude(fov)))) then
							
								   tlt_right = tlt_right + tlt_wts_r(fov-22)*Tbs(fov)
								   diurnal_right   = diurnal_right + tlt_wts_r(fov-22)*diurnal_adjustment(fov)
							       incidence_right = incidence_right + tlt_wts_r(fov-22)*incidence_adjustment(fov)
							 else
							       tlt_right = NAN
								   diurnal_right = NAN
						           incidence_right = NAN
							endif
					    enddo
					    
					    if (finite(tlt_right)) then
						    tb_int = floor(tlt_right)
						else
							tb_int = 500
						endif
						if ((tb_int .ge. 1) .and. (tb_int .le. 500)) hist2(tb_int,2) = hist2(tb_int,2) + 1

                        ! now assign these value to locations for this scan
                        ! all scan locations that contribute to TLT are filled with the value for TLT
                        
                        if (finite(tlt_left)) then
                            do fov = 1,8 
         						ilon = 1 + floor(longitude(fov)/2.5)
								ilat = 1 + floor((latitude(fov)+90.0)/2.5)
								num_good_this_orbit(fov) = num_good_this_orbit(fov) + 1
								if ((ilon .gt. 0) .and. (ilon .lt. 145) .and. (ilat .gt. 0) .and. (ilat .lt. 73)) then
									hist(1,asc_desc,1,ilon,ilat) =  hist(1,asc_desc,1,ilon,ilat) + 1
									hist(2,asc_desc,1,ilon,ilat) =  hist(2,asc_desc,1,ilon,ilat) + tlt_left
									hist(3,asc_desc,1,ilon,ilat) =  hist(3,asc_desc,1,ilon,ilat) + target_temp
									hist(4,asc_desc,1,ilon,ilat) =  hist(4,asc_desc,1,ilon,ilat) + altitude
									hist(5,asc_desc,1,ilon,ilat) =  hist(5,asc_desc,1,ilon,ilat) + diurnal_left
									hist(6,asc_desc,1,ilon,ilat) =  hist(6,asc_desc,1,ilon,ilat) + incidence_left
								endif
							enddo
						endif
						
						if (finite(tlt_right)) then
                            do fov = 23,30
         						ilon = 1 + floor(longitude(fov)/2.5)
								ilat = 1 + floor((latitude(fov)+90.0)/2.5)
								num_good_this_orbit(fov) = num_good_this_orbit(fov) + 1
								if ((ilon .gt. 0) .and. (ilon .lt. 145) .and. (ilat .gt. 0) .and. (ilat .lt. 73)) then
									hist(1,asc_desc,2,ilon,ilat) =  hist(1,asc_desc,2,ilon,ilat) + 1
									hist(2,asc_desc,2,ilon,ilat) =  hist(2,asc_desc,2,ilon,ilat) + tlt_right
									hist(3,asc_desc,2,ilon,ilat) =  hist(3,asc_desc,2,ilon,ilat) + target_temp
									hist(4,asc_desc,2,ilon,ilat) =  hist(4,asc_desc,2,ilon,ilat) + altitude
									hist(5,asc_desc,2,ilon,ilat) =  hist(5,asc_desc,2,ilon,ilat) + diurnal_right
									hist(6,asc_desc,2,ilon,ilat) =  hist(6,asc_desc,2,ilon,ilat) + incidence_right
								endif
							enddo
						endif
					endif		! scan_ok
				enddo			! n_swath
			else
			    print *,'Read Error for Orbit ',orbit_num
			endif				! orbit error
			write(lu_log,4343) num_good_this_orbit
4343        format(30(i3,1x))
		enddo					! orbit num
	
	    close(lu_log)
end subroutine update_AMSU_monthly_maps_TLT

end module AMSU_L2C_Update_Monthly_Maps_TLT