

module AMSU_L2C_Update_Monthly_Maps_by_FOV


use AMSU_constants
use AMSU_TYPES
use date_and_time
use NAN_support
use topography
use portlib
use AMSU_Level_2A
use AMSU_Inventory
use AMSU_File_Locations
use AMSU_Quality

implicit none

integer(4),parameter	:: GRPT_TB_REC_LENGTH     = 4 + (7*2*144*72*4)

contains

!@***m* MLT_V4_RSS/update_AMSU_monthly_maps_by_fov.f90
!  NAME
!    update_AMSU_monthly_maps_by_fov.f90
! 
!  PURPOSE
!    assembles AMSU L2A data into monthly maps
! 
!  DESCRIPTION
!   assembles AMSU L2A data into monthly maps
!   This code calls code (in the AMSU_L2A module)  that reads AMSU data in orbit by orbit files.  
!   An inventory of these files exists, and needs to be read as well.  Information about the location of the 
!   orbit and inventory files is contained in the AMSU_File_Locations module.

!
!  INPUTS
!	satellite number		integer: satellite number to process
!	channel_str			integer: channel to process
!	first_date_to_process   date_str: first date to process			
!	last_date_to_process    date_str: last date to process			
!
!  OUTPUT
!	error				integer:  error flag.  0 means no error
!	monthly maps of AMSU radiance are written to binary files
!
!  AUTHOR
!    Carl Mears, Remote Sensing Systems
! 
!  COPYRIGHT
!   THIS SOFTWARE AND ITS DOCUMENTATION ARE COPYRIGHT 2012 REMOTE SENSING SYSTEMS.  
!   THEY ARE PROVIDED FOR INFORMATIONAL AND ALGORITHM VERIFICATION PURPOSES ONLY 
!   AND MAY NOT BE USED IN WHOLE OR IN PART FOR ANY OTHER PURPOSE WITHOUT WRITTEN
!   PERMISSION FROM REMOTE SENSING SYSTEMS.  THEY ARE FURNISHED "AS IS." THE 
!   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES, 
!   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE 
!   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
!   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT 
!   TO USERS.
! 
!  HISTORY
!    4/15/2018 Initial Version prepared for NCDC
! 
!  USAGE
!    call update_AMSU_monthly_maps_by_fov(sat_num,channel,first_date_to_process,last_date_to_process,error)
!
!@*****

subroutine update_AMSU_monthly_maps_by_fov(sat_num,channel,first_date_to_process,last_date_to_process,error)

	integer(4)				:: sat_num
	integer(4)				:: channel
	type(date_str)			:: first_date_to_process
	type(date_str)			:: last_date_to_process
	type(date_str)			:: strt_date
	integer(4)				:: error

	
!   Parameters
!
!	sat num					:: satellite number 10 = NOAA-15, 2 = NOAA-16, etc
!   channel					:: amsu channel number 1 - 16
!   first_date_to_process	:: Date structure containing first dat to process
!   last_date_to_process	:: Date structure containing the last date to process
!   process control			:: Logical array (20).  Controls which corrections are used and ancilliary data are calulated
!   error					:: = 0 if everything is OK, no zero otherwise
!
!	Input Files:
!		This code calls code (in the AMSU_L2A module)  that reads AMSU data in orbit by orbit files.  
!		An inventory of these files exists, and needs to be read as well.  Information about the location of the 
!		orbit and inventory files is contained in the AMSU_File_Locations module.

!	Output Files:
!		months:
!			month Files 
!
!


	real(4),dimension(NUM_AMSU_FOVS)	:: Tbs
	
	integer(4)							:: start_month
	logical								:: start_found
	integer(4)							:: inv_rec_num
	
	type(date_str)						::  rec_date
	integer(4)							::  rec_month
	integer(4)							::  strt_orbit
	integer(4)							::  last_orbit
	integer(4)							::  last_month
	integer(4)							::  last_valid_rec
	logical							::  last_found
	integer(4)							::  date_Error
	
	type(amsu_inventory_str)			:: inv			


	! declaration of input data structures
	
	type(AMSU_L2A_orbit_header)							:: L2A_orbit_header
	type(AMSU_L2A_orbit_data)							:: L2A_orbit

	type(date_str)				 :: swath_date

	integer(4)					 :: orbit_num
	character(len = 7)           :: sat_name
	integer(4)					 :: n_swath
	integer(4)					 :: orbit_error

	real(4)						 :: remainder

	real(4)							  :: target_temp
	real(4),dimension(NUM_AMSU_FOVS)  :: longitude
	real(4),dimension(NUM_AMSU_FOVS)  :: latitude
	integer(1),dimension(NUM_AMSU_FOVS)  :: fov_qual

	real(4),dimension(NUM_AMSU_FOVS)  :: diurnal_correction
	real(4),dimension(NUM_AMSU_FOVS)  :: nadir_correction
	real(4),dimension(NUM_AMSU_FOVS)  :: incidence_correction
	real(4),parameter				  :: precip_thres = 1.5

	type(Date_Str)					  :: curr_Date

	! declaration of gridpoint structures

	real(4),dimension(7,2,30,144,72)	    :: hist	 ! (num/tot_tb/tot_diur/tot_inc/tot_limb/tot_tar/tot_samp,asc/desc,FOV,ilon,ilat)
	real(4),dimension(30,500)               :: tbhist 

	integer(4)			:: ilon
	integer(4)			:: ilat
	
	integer(4)          :: tb_int

	! Set logical unit numbers for the various files

	integer(4),parameter			:: lu_raw_Tb		= 29	! orbit_files
	integer(4),parameter			:: lu_diurnal		= 30	! diurnal correction
	integer(4),parameter			:: lu_nadir			= 32	! nadir corrections (limb 
	integer(4),parameter			:: lu_inc			= 34	! incidence corrections
	integer(4),parameter			:: lu_target		= 28	! target temperature

	integer(4),parameter			:: lu_Tbs_grpt		= 50	! Tbs

	integer(4),parameter			:: lu_log			= 70	! log 
	integer(4),parameter			:: lu_inventory     = 100	! orbit inventory file

	integer(4)			:: i
	integer(4)			:: fov
	integer(4)			:: asc_desc
	logical(4)			:: scan_ok

	integer(4)			:: month
	integer(4)          :: month_last
	integer(4)			:: grpt_rec_num

	! log file name

	character(len=120)  :: log_file_name
	character(20)		:: ds

	! output file names

	character(len=150)  :: tb_grpt_name
						
	!----------------------------------------------

		!call init_NAN

		! open log file
		ds = curr_date_string(error)
		write(log_file_name,110) channel,sat_num,trim(ds)
110     format('M:\MSU_AMSU_Data\AMSU-A_L2C\L2A_to_L2C_Logs\month_update\log_ch',i1,'_sat_',i2.2,'_',a,'.maps_by_fov.txt')
		open(unit = lu_log,file = log_file_name)
		error = 0

		sat_name = amsu_sat_names_2(sat_num)

	    ! read in inventory file

		call read_amsu_L2A_inventory_file_text(sat_num,channel,inv,error)

		! determine the first orbit and month to use
		! we use the first month that contains the start date, and the last month that contains the
		! end date.  Thus the region of computation is expanded somewhat.

		start_month = calc_month(first_date_to_process,remainder)
		start_found = .false.

		last_valid_rec = -1
		do inv_rec_num = 1,MX_ORBITS
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle

			rec_date   = set_date_2000(inv%end_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			
			if (rec_month .ge. start_month) then
				start_found = .true.
				if (last_valid_rec .gt. 0) then
					strt_orbit = last_valid_rec
				else
					strt_orbit = inv_rec_num
				endif
				strt_date = set_date_2000(inv%start_rd_2000(strt_orbit),error)
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo
			 
		if (.not.(start_found)) then 
			print *, 'start orbit not found for sat num ',sat_num
			return
		endif

		! determine the last orbit to use

		last_month = calc_month(last_date_to_process,remainder)
		last_found = .false.
		last_valid_rec = -1

		do inv_rec_num = 1,MX_ORBITS
			if (.not. inv%orbit_valid(inv_rec_num)) cycle
			if (.not. finite(inv%start_rd_2000(inv_rec_num))) cycle
			if (.not. finite(inv%end_rd_2000(inv_rec_num))) cycle

			rec_date = set_date_2000(inv%start_rd_2000(inv_rec_num),error)
			rec_month = calc_month(rec_date,remainder)
			if (rec_month .gt. last_month) then
				last_found = .true.
				if (last_valid_rec .gt. 0) then
					last_orbit = inv_rec_num
				else
					last_orbit = inv_rec_num
				endif
				last_month = calc_month(set_date_2000(inv%start_rd_2000(last_orbit),error),remainder)
				exit
			endif
			last_valid_rec = inv_rec_num
		enddo

		if (.not.(last_found)) then 
			last_orbit = inv%max_orbit
			last_month = calc_month(set_date_2000(inv%start_rd_2000(last_orbit),error),remainder)
		endif

		print *,strt_orbit,start_month,'   ',date_string(strt_Date)
		print *,last_orbit,last_month,'   ',date_string(set_date_2000(inv%start_rd_2000(last_orbit),error))

		! open the output file


		write(lu_log,3351)
3351	format(' Computing Tb GrPt Maps')

		! Initialize a few variables before beginning to read

		i = 1
		error = 0

		hist = 0.0
		tbhist = 0.0
							
		do orbit_num = strt_orbit,last_orbit
							
			orbit_error = 0
            if (orbit_num .eq. 1928) then 
               print *,'sfd'
            endif
			call read_AMSU_L2A_orbit_file(sat_name,					&
										  orbit_num,				& 
										  channel,					&
										  L2A_orbit_header,			&
										  L2A_orbit,				&
										  orbit_error)
			print *,'Processing Orbit ',orbit_num

			if ((orbit_error .eq. 0)) then 
				
				do n_swath = L2A_orbit_header%start_swath_num,L2A_orbit_header%end_swath_num

					if (L2A_orbit%scan_valid(n_swath) .le. 0) cycle  ! skip rest if scan not valid

					asc_desc = L2A_orbit%scan_data(n_swath)%asc_desc

					! set date
						 
					swath_date = set_date_2000(L2A_orbit%scan_Data(n_swath)%scan_rd_2000,date_error)
					
					if (.not. finite(swath_Date%rd_2000)) cycle	 ! skip rest if bad time

					month = calc_month(swath_date,remainder)

					if (month .lt. start_month) cycle	  ! only process swaths in the correct months
					!if (month .gt. last_month) cycle
					if (i .gt. 2) then
						if (month .gt. month_last) then !write out the monthly hist record for this month

!	987						format(' Processing Channel ',i1,1x,a,2x,i2.2,'/',i2.2,'/',i4.4,'     ',i2.2,':',i2.2,':',i2.2,2x,i6,2x,i6)
						
							grpt_rec_num = 1 + month_last - AMSU_FIRST_MONTH(sat_num)

							write(tb_grpt_name,351)trim(AMSU_Monthly_Map_Path),channel,trim(sat_name),month_last
					351		format(a,'AMSU_Channel_',i2.2,'\FOV_maps\',a,'\Month_',i4.4,'_FOV_Hist_144_72_V3_02.dat')
							open(unit = lu_tbs_grpt,file = tb_grpt_name,form='binary')
							write(lu_Tbs_grpt) month_last,hist,tbhist
							print *,'Wrote file: ',tb_grpt_name
							close(unit=lu_tbs_grpt)
							hist  = 0.0
							month_last = month
						endif
					else 
							month_last = month
					endif  ! i gt 2


					i = i+1
					
					scan_ok = .true.

					Tbs						= L2A_orbit%scan_data(n_swath)%Tbs
					longitude				= L2A_orbit%scan_data(n_swath)%fov_longitude
					latitude				= L2A_orbit%scan_data(n_swath)%fov_latitude
					diurnal_correction		= L2A_orbit%scan_data(n_swath)%diurnal_corr
					nadir_correction		= L2A_orbit%scan_data(n_swath)%nadir_corr 
					incidence_correction    = L2A_orbit%scan_data(n_swath)%incidence_corr 
					target_temp				= L2A_orbit%scan_data(n_swath)%warm_load_temp
					fov_qual                = L2A_orbit%scan_data(n_swath)%fov_quality
	
					if ((scan_ok) .and. (month .eq. month_last)) then  ! this ignores data from bried backwards time steps
						call check_tbs_against_bounds(Tbs,channel,fov_qual)
						do fov = 1,30
							if ((finite(Tbs(fov))) .and.				&
								(fov_qual(fov) .eq. 0) .and.			&
								(finite(diurnal_correction(fov))) .and.	&
								(finite(nadir_correction(fov)))	.and.	&
								(finite(longitude(fov))) .and.			&
								(finite(latitude(fov)))) then

								ilon = 1 + floor(longitude(fov)/2.5)
								ilat = 1 + floor((latitude(fov)+90.0)/2.5)

								if ((ilon .gt. 0) .and. (ilon .lt. 145) .and. (ilat .gt. 0) .and. (ilat .lt. 73)) then
									hist(1,asc_desc,fov,ilon,ilat) =  hist(1,asc_desc,fov,ilon,ilat) + 1
									hist(2,asc_desc,fov,ilon,ilat) =  hist(2,asc_desc,fov,ilon,ilat) + Tbs(fov)
									hist(3,asc_desc,fov,ilon,ilat) =  hist(3,asc_desc,fov,ilon,ilat) + diurnal_correction(fov)
									hist(4,asc_desc,fov,ilon,ilat) =  hist(4,asc_desc,fov,ilon,ilat) + incidence_correction(fov)
									hist(5,asc_desc,fov,ilon,ilat) =  hist(5,asc_desc,fov,ilon,ilat) + nadir_correction(fov)
									hist(6,asc_desc,fov,ilon,ilat) =  hist(6,asc_desc,fov,ilon,ilat) + target_temp
									hist(7,asc_desc,fov,ilon,ilat) =  hist(7,asc_desc,fov,ilon,ilat) + swath_date%day
								endif
								
								tb_int = floor(tbs(fov))
								if (tb_int .lt. 1) tb_int = 1
								if (tb_int .gt. 500) tb_int = 500
								tbhist(fov,tb_int) = tbhist(fov,tb_int) + 1
							endif
						enddo	! fov
					endif		! scan_ok
				enddo			! n_swath
			endif				! orbit error & diurnal error
		enddo					! orbit num
	
	    close(lu_log)
end subroutine update_AMSU_monthly_maps_by_FOV
end module AMSU_L2C_Update_Monthly_Maps_by_FOV

