;@***h* MLT_V4_RSS/adjusted_AMSU_map_from_FOV_map.pro
; NAME
;   MLT_V4_RSS/adjusted_AMSU_map_from_FOV_map.pro
;
; PURPOSE
;   calculates maps of angle and diurnal adjustment for AMSU for a given monthly map
;
; DESCRIPTION
;   calculates maps of angle and diurnal adjustment for AMSU for a given monthly map  tb_map_adj, tar_map, and num_map are 144x72 maps, with the
;   combination radiance from different FOV's and Nodes combined in the routine
;
; INPUTS
;   satnum                  integer: satellite number (10..17)
;   channel                 integer:  AMSU channel number
;   year                    integer: year to process
;   month                   integer: month to process
;   fov_wts                 array of 11 reals.  FOV weights
;   node_wts                array of 2 reals.  Node weights
;   
; OUTPUTS
;  tb_map_adj               array of reals [144,72] map of adjusted radiances
;  tar_map = tar_map        optional array of reals [144,72] map of target temperatures
;  num_map = num_map        optional array of reals [144,72] map of number of obs per cell.
;  diurnal_adjustment       optional array of reals [2,11,144,72] map of diurnal adjustment
;  inc_adjustment           optional array of reals [2,11,144,72] map of incidence adjustment
;  nadir_adjustment         optional array of reals [2,11,144,72] map of nadir adjustment
;  error                    integer error flag. 0 means everything OK, -1 to  -7 indicates problem with correction, -8 means no tb file
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   4/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   adjusted_AMSU_map_from_FOV_map,  satnum,channel,year,month,fov_wts = fov_wts,node_wts = [0.5,0.5],      $    
;                                    tb_map_adj,tar_map = tar_map,num_map = num_map,                                  $   
;                                    diurnal_adjustment = diurnal_adjustment,inc_adjustment = inc_adjustment,         $ 
;                                    nadir_adjustment = nadir_adjustment,tb_maps_adj = tb_maps_adj,error
;@*****

pro adjusted_AMSU_map_from_FOV_map,satnum,channel,year,month,    $
                                    adj_flag=adj_flag,            $    ;1 = inc,2 = nadir, 4 = diur + combinations.  nadir == nadir+inc
                                                                  $    ; default will be 7
                                    fov_wts = fov_wts,            $    ; flt(30)  will forced to add to 1
                                    node_wts = node_wts,          $    ; fltarr(2) will be forced to add to 1, deault = [0.5,0.5]
                                    tb_map_adj,                   $    ; adjusted,fov-combined 144x72 tb map
                                    tar_map = tar_map,            $
                                    num_map = num_map,            $    ; optional variable to return num map
                                    diurnal_adjustment = diurnal_adjustment, $ ;optional variable containing diurnal adjustment
                                    inc_adjustment = inc_adjustment,         $ ;optional variable containing incidence adjustment
                                    nadir_adjustment = nadir_adjustment,     $ ;optional variable containing nadir adjustment
                                    error                              ; 0 means everything OK, -1 - -7 indicates problem with correction
                                                                       ; -8 means no tb file, + number are warnings, not defined yet

; set defaults, mormalize wts

tb_map_adj = replicate(!values.f_nan,144,72)
tar_map = replicate(!values.f_nan,144,72)
num_map = replicate(!values.f_nan,144,72)


if keyword_set(adj_flag) eq 0 then adj_flag = 7
if adj_flag gt 7 then adj_flag = 7
if adj_flag lt 0 then adj_flag = 0

if keyword_set(node_wts) eq 0 then begin
  node_wts = [0.5,0.5]
endif else begin
  if n_elements(node_wts) ne 2 then begin
    print,'Error: node_wts not array(2)
    error = -16
    return
  endif else begin
    tot = total(node_wts)
    if ((finite(tot) eq 0) or (tot eq 0.0)) then begin
      print,'Error: node wts total = NAN or 0'
      error = -16
      return
    endif
    node_wts = float(node_wts)/total(node_wts)
  endelse
endelse

if keyword_set(fov_wts) eq 0 then begin
  print,'Error: fov_wts missing
  error = -32
  return
endif else begin
  if n_elements(fov_wts) ne 30 then begin
    print,'Error: fov_wts not arr(30)
    error = -32
    return
  endif else begin
    tot = total(fov_wts)
    if ((finite(tot) eq 0) or (tot eq 0.0)) then begin
      print,'Error: fov wts total = NAN or 0'
      error = -32
      return
    endif
    fov_wts = float(fov_wts)/total(float(fov_wts))
  endelse
endelse

read_AMSU_monthly_map_by_fov,satnum,channel,year,month,tb_arr,err

if err ne 0  then begin
  error = -8
  return
endif

tb_maps     = reform(tb_arr[1,*,*,*,*]/tb_arr[0,*,*,*,*])
tar_maps    = reform(tb_arr[2,*,*,*,*]/tb_arr[0,*,*,*,*])
num_maps    = reform(tb_arr[0,*,*,*,*])

; apply angle adjustments

  altitude_maps = reform(tb_arr[3,*,*,*,*])/reform(tb_arr[0,*,*,*,*])
  nom_fov = 0
  if channel eq 9 then nom_fov = 7
  calc_amsu_angle_adjustments,altitude_maps,month,inc_adjustment,nadir_adjustment,error,nom_fov = nom_fov
   
  if (adj_flag and 2) gt 0 then begin  ; remember, adj_flag = 3 or 7 is the same as 2 or 6 - nadir includes inc
    tb_maps_adj = tb_maps + nadir_adjustment
  endif else begin
    if (adj_flag and 1) gt 0 then begin
      tb_maps_adj = tb_maps + inc_adjustment
    endif else begin
      tb_maps_adj = tb_maps
    endelse
  endelse

; apply diurnal adjustments
if (adj_flag and 4) gt 0 then begin
  calc_amsu_diurnal_adjustments,satnum,year,month,diurnal_adjustment
  tb_maps_adj = tb_maps_adj + diurnal_adjustment
endif

; combine fovs and nodes

temp = fltarr(2,144,72)
temp_wt = fltarr(2,144,72)
for fov = 0,29 do begin
  if (abs(fov_wts[fov]) gt 0.00001) then begin
    map = reform(tb_maps_adj[*,fov,*,*])
    ok = where(finite(map),ct)
    if ct gt 0 then begin
      temp[ok] = temp[ok]+fov_wts[fov]*map[ok]
      temp_wt[ok] = temp_wt[ok] + fov_wts[fov]
    endif
  endif
endfor
temp = temp/temp_wt
tb_map_adj = node_wts[0]*reform(temp[0,*,*]) + node_wts[1]*reform(temp[1,*,*])

temp = fltarr(2,144,72)
temp_wt = fltarr(2,144,72)
for fov = 0,29 do begin
  if (abs(fov_wts[fov]) gt 0.00001) then begin
    map = reform(tar_maps[*,fov,*,*])
    ok = where(finite(map),ct)
    if ct gt 0 then begin
      temp[ok] = temp[ok]+fov_wts[fov]*map[ok]
      temp_wt[ok] = temp_wt[ok] + fov_wts[fov]
    endif
  endif
endfor
temp = temp/temp_wt
tar_map = node_wts[0]*reform(temp[0,*,*]) + node_wts[1]*reform(temp[1,*,*])

temp = fltarr(2,144,72)
temp_wt = fltarr(2,144,72)
for fov = 0,29 do begin
  if (abs(fov_wts[fov]) gt 0.00001) then begin
    map = reform(num_maps[*,fov,*,*])
    ok = where(finite(map),ct)
    if ct gt 0 then begin
      temp[ok] = temp[ok]+fov_wts[fov]*map[ok]
      temp_wt[ok] = temp_wt[ok] + fov_wts[fov]
    endif
  endif
endfor
temp = temp/temp_wt
num_map = node_wts[0]*reform(temp[0,*,*]) + node_wts[1]*reform(temp[1,*,*])
error = 0

end
  