;@***h* MLT_V4_RSS/calc_amsu_angle_adjustments.pro
; NAME
;   MLT_V4_RSS/calc_amsu_angle_adjustments.pro
;
; PURPOSE
;   calculates maps of angle adjustments (incidence and nadir) for AMSU
;
; DESCRIPTION
;   calculates maps of angle adjustments (incidence and nadir) for AMSU.  This requires information about the satellite height which is passed
;   into the routine in the form of a map.  Aititudes are assumed to be constant over the month.
;
; INPUTS
;   alt_maps            array of reals [2,11,144,72]: map of satellite altitudes
;   month               integer: month of year
;   nom_fov = nom_fov   integer: optional target FOV.  If not provided, nadir adjustments refer the radiances to nadir.
;
; OUTPUTS
;  inc_adjustment       array of reals [2,11,144,72]:  adjustments to nominal incidence angle
;  nadir_adjustment     array of reals [2,11,144,72]:  adjustments to nadir
;  error                integer: error flag.  Currently always 0 [OK]
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   4/10/2018 Initial Version prepared for NCEI
;
; USAGE
;   calc_amsu_angle_adjustments,alt_maps,month,inc_adjustment,nadir_adjustment,error,nom_fov = nom_fov
;   
;@*****

pro calc_amsu_angle_adjustments,alt_maps,month,inc_adjustment,nadir_adjustment,error,nom_fov = nom_fov
      
      if keyword_set(nom_fov) eq 0 then nom_fov = 0
      common eia
      
      EIA_maps = fltarr(2,30,144,72)
      fov_mapping = [floor(14.5-findgen(15)),reverse(floor(14.5-findgen(15)))]
      for node = 0,1 do begin
        for fov = 0,29 do begin
          alt_map = reform(alt_maps[node,fov,*,*])
          geometry,amsu_view_angles[fov_mapping[fov]],eia__lat_map,alt_map,eia_map
          EIA_maps[node,fov,*,*] = eia_map
        endfor
      endfor
      
      ;calculate simulated tbs for the nominal EIA and the actual EIAs
      ;nominal value is simply the ipar = 0 map for a given view index
            
      inc_adjustment   = replicate(!values.f_nan,2,30,144,72)
      nadir_adjustment = replicate(!values.f_nan,2,30,144,72)
           
      for node = 0,1 do begin
        for fov = 0,29 do begin
          ;calculate simulated tbs for the nominal EIA and the actual EIAs
          ;nominal value is simply the ipar = 0 map for a given view index
          tb_sim_nom = reform(eia__adj_arr[month-1,0,fov_mapping[fov],*,*])
          
          delta_theta = reform(abs(eia_maps[node,fov,*,*]) - AMSU_NOM_EIAs[fov_mapping[fov]])
          
          tb_sim_act  = reform(eia__adj_arr[month-1,0,fov_mapping[fov],*,*] + $
                        eia__adj_arr[month-1,1,fov_mapping[fov],*,*]*delta_theta + $
                        0.5*eia__adj_arr[month-1,2,fov_mapping[fov],*,*]*delta_theta^2)
                        
                        
          inc_adj = tb_sim_nom - tb_sim_act
          nadir_adj = reform(eia__adj_arr[month-1,0,nom_fov,*,*])-tb_sim_act
          
          ; we will add these adjustments to the AMSU tbs to refer the measurements to nadir
          ; or the nominal EIAs
          
          inc_adjustment[node,fov,*,*]    = inc_adj
          nadir_adjustment[node,fov,*,*]  = nadir_adj
          
          ; subtract off cross track offsets.  These are defined for each satellite/channel, but are otherwise
          ; constant
          

        endfor
      endfor
      
      for fov = 0,29 do begin
        nadir_adjustment[*,fov,*,*] = nadir_adjustment[*,fov,*,*]-eia__cross_track_offsets[fov]
        inc_adjustment[*,fov,*,*]   = inc_adjustment[*,fov,*,*]-eia__cross_track_offsets[fov]
      endfor
 end