;@***h* MLT_V4_RSS/calc_and_save_all_AMSU_adjustments_from_FOV.pro
; NAME
;   MLT_V4_RSS/calc_and_save_all_AMSU_adjustments_from_FOV.pro
;
; PURPOSE
;   calculates angle and diurnal adjustment for AMSU for given set of angle and diurnal models
;
; DESCRIPTION
;   calculates angle and diurnal adjustment for AMSU for given set of angle and diurnal models
;
; INPUTS
;   channel                 integer:  MSU channel number
;   fov_wts                 array of 11 reals.  FOV weights
;   node_wts                array of 2 reals.  Node weights
;   adj_flag                usually set to seven to do incidence, nadir, and diurnal adjustments
;   eia_model = eia_model   string containing name of EIA model.  Usually 'MERRA'
;   diur_model=diur_model   string containing name of diurnal model.  ['CCM3','MERRA','HADGEM']
;   tb_arr_adj              array of reals (144,72,336,9) of satellite radiances
;   tar_arr=tar_arr         array of reals (144,72,336,9) of satellite radiances
;   num_arr=num_arr           array of reals (144,72,336,9) of number of obs in each cell
;   sats_to_use = sats_to_use array of 9 integers.  set to 1 to use that satellite.
;   angle_model = angle_model      string containing the name of hte angle model to use    
;   diurnal_model = diurnal_model  string containing the name of hte angle model to use
;   year_to_process = year_to_process 	integer year to process                  $
;   start_month = start_month			integer first month to process
;   end_month = end_month                 integer last month to process
;
; OUTPUTS
;   writes diurnal and angle adjustment gridded monthly files for AMSU
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   4/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   calc_and_save_all_AMSU_adjustments_from_FOV,  $
;        channel,                      $
;        fov_wts,                      $
;        fov_str,                      $
;        node_wts,                     $
;        adj_flag,                     $
;        tb_arr_adj,                   $
;        tar_arr=tar_arr,              $
;        num_arr=num_arr,              $
;        sats_to_use = sats_to_use,    $
;        angle_model = angle_model,    $
;        diurnal_model = diurnal_model,$
;        year_to_process = year_to_process,                  $
;        start_month = start_month,    $
;        end_month = end_month;
;@*****

pro calc_and_save_all_AMSU_adjustments_from_FOV,  $
        channel,                      $
        fov_wts,                      $
        fov_str,                      $
        node_wts,                     $
        adj_flag,                     $
        tb_arr_adj,                   $
        tar_arr=tar_arr,              $
        num_arr=num_arr,              $
        sats_to_use = sats_to_use,    $
        angle_model = angle_model,    $
        diurnal_model = diurnal_model,$
        year_to_process = year_to_process,                  $
        start_month = start_month,    $
        end_month = end_month

if keyword_set(sats_to_use) eq 0 then sats_to_use = replicate(1.0,8)

if keyword_set(angle_model) eq 0 then begin
  angle_model = 'NCEP'
  if channel ge 10 then angle_model = 'MERRA'
endif

if keyword_set(diurnal_model) eq 0 then begin
  diurnal_model = 'CCM3'
  if channel ge 10 then diurnal_model = 'MERRA'
endif

if keyword_set(year_to_process) eq 0 then stop
if keyword_set(start_month) eq 0 then start_month = 1
if keyword_set(end_month) eq 0 then end_month = 12

; load angle adjustment arrays
common eia
common diurnal
common satellites

eia__adj_arr = replicate(!values.f_nan,12,3,15,144,72)
for month = 1,12 do begin
  if angle_model eq 'MERRA' then begin 
    read_merra_nadir_adj,channel,month,merra_eia_adj_arr_one_month
    eia__adj_arr[month-1,*,*,*,*] = merra_eia_adj_arr_one_month
    
  endif else begin
    if angle_model eq 'NCEP' then begin
      read_ncep_nadir_adj,channel,month,ncep_eia_adj_arr_one_month
      eia__adj_arr[month-1,*,*,*,*] = ncep_eia_adj_arr_one_month
    endif else begin
      print,'No Match for Angle Model'
    endelse   
  endelse
endfor
eia__lat_map = fltarr(144,72)
for ilat = 0,71 do begin
   eia__lat_map[*,ilat] = -90.0 + 1.25 + 2.5*ilat
endfor

eia__channel_loaded = channel
eia__model_loaded = angle_model
print,'Finished Loading Angle Adjustments, Model: '+eia__model_loaded+', Channel: '+string(eia__channel_loaded)
; end load angle adjustments

; load diurnal adjustment arrays

read_diurnal_adj_arr,channel,diurnal_model,diur__adj_arr,err
if err eq 0 then begin
  diur__channel_loaded = channel
  diur__model_loaded = diurnal_model
  print,'Finished Loading Diurnal Adjustments, Model: '+diur__model_loaded+', Channel: '+string(diur__channel_loaded)
endif else begin
  diur__channel_loaded = -1
  diur__model_loaded  = 'none'
endelse

; end load diurnal adjustment arrays

tb_arr_adj = fltarr(144,72,satellites__num_months,satellites__num_sats)
tar_arr = fltarr(144,72,satellites__num_months,satellites__num_sats)
num_arr = fltarr(144,72,satellites__num_months,satellites__num_sats)

for satnum = 10,10 + satellites__num_sats-1  do begin
  if sats_to_use[satnum-10] eq 0 then continue
  
  
  for year = year_to_process,year_to_process do begin
    for month = start_month,end_month do begin
     
      adjusted_AMSU_map_from_FOV_map,satnum,channel,year,month,     $
                                      adj_flag=adj_flag,            $    ;1 = inc,2 = nadir, 4 = diur + combinations.  nadir == nadir+inc
                                                                    $    ; default will be 7
                                      fov_wts = fov_wts,            $    ; flt(30)  will forced to add to 1
                                      node_wts = node_wts,          $    ; fltarr(2) will be forced to add to 1, deault = [0.5,0.5]
                                      tb_map_adj,                   $    ; adjusted,fov-combined 144x72 tb map
                                      tar_map = tar_map,            $    ; optional variable to return tar map
                                      num_map = num_map,            $    ; optional variable to return num map
                                      diurnal_adjustment = diurnal_adjustment, $ ;optional variable containing diurnal adjustment
                                      inc_adjustment = inc_adjustment,         $ ;optional variable containing incidence adjustment
                                      nadir_adjustment = nadir_adjustment,     $
                                      tb_arr = tb_arr, $
                                      error  
                                       
      if error eq 0 then begin
        print,string(channel,satnum,year,month,format = '("Calculating Adjustments, channel = ",i4.4," satnum = ",i4.4," year = ",i4.4,", month = ",i2.2)')
        write_AMSU_diurnal_adjustment_fov,satnum,channel,year,month,diur__model_loaded,diurnal_adjustment,error
        write_amsu_incidence_adjustment_fov,satnum,channel,year,month,eia__model_loaded,diurnal_adjustment,error
        write_amsu_nadir_adjustment_fov,satnum,channel,year,month,eia__model_loaded,nadir_adjustment,error
      endif  
    endfor
  endfor   
endfor                           
end