;@***h* MLT_V4_RSS/amsu_v4_grpt_merge_refactor.tmt_op.pro
; NAME
;   amsu_v4_grpt_merge_refactor.tmt_op.pro
;
; PURPOSE
;   top level program that performs TMT merge for AMSU
;
; DESCRIPTION
;   top level program that performs TMT merge for AMSU.  The merging process is described in 
;
;   Mears, C. A., & Wentz, F. J. (2016). Sensitivity of Satellite-Derived Tropospheric Temperature
;   Trends to the Diurnal Cycle Adjustment. Journal of Climate, 29, 3629-3646. doi: 10.1175/JCLI-D-15-0744.1
;
;   The program defines several common block, then read in data and performs simple QC analysis to 
;   remove data that is too sparse for a given month/satellite.  The program then calculates the various merging parameters.
;   They are:
;     Target Factors
;     Adjustments to the diurnal correction
;     Intersatellite offsets
;   
;   The merged data product is then written.
; INPUTS
;   input variables
; OUTPUTS
;   output variable and descriptions
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   1/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   .r amsu_v4_grpt_merge_refactor.tmt_op.pro
;
;@*****


load_AMSU_file_locations,verbose=1
read_land_fraction_144_72,lf
land_mask = intarr(144,72)
land = where(lf gt 0.5)
land_mask[land] = 1.0

start_year = 1998
end_year = 2017
num_months = 12*(1 + end_year - start_year)
num_sats = 8


; define commom blocks
; EIA common block -- contains info need for the angle adjustments

common eia, eia__adj_arr, eia__channel_loaded,eia__model_loaded,eia__lat_map, $
       eia__cross_track_offsets,eia__cross_track_sat_loaded,eia__cross_track_channel_loaded, $
       AMSU_VIEW_ANGLES,AMSU_NOM_EIAS

eia__channel_loaded = -1
eia__cross_track_sat_loaded = -1
eia__cross_track_channel_loaded = -1

eia__model_loaded = 'none'

AMSU_VIEW_ANGLES = [1.6666666,5.0000000,8.3333333,11.666667,15.000000,  $
                    18.333333,21.666667,25.000000,28.333333,31.666667,  $
                    35.000000,38.333333,41.666667,45.000000,48.333333]

AMSU_NOM_EIAS    = [1.875947, 5.629541, 9.388301,13.155880,16.936250,   $
                    20.733890,24.554020,28.402830,32.287970,36.219100,  $
                    40.208800,44.274040,48.438570,52.737200,57.224260]
                    
; END EIA common block

; Diurnal common block
common diurnal, diur__adj_arr, diur__channel_loaded,diur__model_loaded,diur__lat_time_map
  diur__channel_loaded = -1
  diur__model_loaded  = 'none'
; END Diurnal Common Block

channels = ['TLT','TMT','TTS','TLS']

eia_model = 'MERRA'
diurnal_models = ['NONE','CCM3','HADGEM','MERRA','CMAM','NONE','CCM3','HADGEM','MERRA','CMAM']

offset_scheme = 'land_ocean_separate'
offset_scheme_index = 2
tf_schemes = ['ocean_only']
tf_surf = 'Sea'
tf_scheme_index = 2

fov_set = 'central24'
fov_ok = [3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26]
num_fovs_used = 24

channel_index = 1
channel = channels[channel_index]
      
      case channel of 
        'TLT':begin
          message,'TLT does not really work with this method because of sampling issues'
          stop
          end
        'TMT':begin
          amsu_channel = 5
          fov_wts = fltarr(30)
          fov_wts[fov_ok] = 1.0
          channel_str = channel
          eia_model = 'MERRA'
          sats_to_use = [1,0,0,1,1,1,1,1] 
          num_sats = n_elements(sats_to_use)
          node_wts = [1.0,1.0]
          adj_flag = 7
          ;print,'THIS FLAG SHOULD BE 7!'
          months_to_use_mask = intarr(num_months,num_sats)
          smooth_maps = 0
          num_thres = 14
          do_tlt_adjustment = 0
          end
        'TTS':begin
          message,'TTS not ready'
          stop
          end
          
        'TLS':begin
          message,'TLS not ready'
          stop
          end
      endcase

      use_seasonal_diurnal_fit = 1    ; use seasonal modulated diurnal optimization
      diurnal_index = 6               ; set diurnal model to optimized CCM3.
      if ((diurnal_index eq 4) and (channel_index lt 3)) then stop  ; no CMAM except for TLS
      if ((diurnal_index eq 9) and (channel_index lt 3)) then stop  ; no CMAM except for TLS
      
      diurnal_model = diurnal_models[diurnal_index]
      diurnal_factor = 1.0
      perform_diurnal_adjustment = 0
      if diurnal_index ge 5 then perform_diurnal_adjustment = 1       
      
      

      read_all_adjusted_AMSU_maps_from_FOV, amsu_channel,       $
                                              fov_wts,            $
                                              node_wts,           $
                                              adj_flag,           $
                                              num_sats,           $
                                              num_months,         $
                                              tb_arr,         $
                                              tar_arr=tar_arr,    $
                                              num_arr=num_arr,    $
                                              sats_to_use = sats_to_use,     $
                                              diurnal_model = diurnal_model, $
                                              diurnal_factor = diurnal_factor,          $
                                              eia_model = eia_model, $;, $
                                              tb_arr_raw = tb_arr_raw

       ; set up months_to_use_mask to truncate data as described in Mears and Wentz 2016.
        months_to_use_mask = intarr(num_months,num_sats)

       ; setup months_to_use_mask to truncate AQUA and NOAA-15
       months_to_use_mask[144:num_months-1,5] = -1  ; truncate AQUA after 2009
       months_to_use_mask[168:num_months-1,0] = -1  ; truncate NOAA-15 after 2011
       ; remove a couple of bad months for METOP-A
       months_to_use_mask[162,4] = 1
       months_to_use_mask[163,4] = 1
       ; remove october 2015 from NOAA-18
       months_to_use_mask[213,3] = -1
       
       sats_to_use = [1,0,0,1,1,1,1,1]  ; ignore NOAA-16 and NOAA-17
      
       ; set paths and log file
       version_num = 1
       report_path = 'M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4_op\Merge_Monthly_AMSU_FOV_V4\reports_test\'
       log_path    = 'M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4_op\Merge_Monthly_AMSU_FOV_V4\logs\'
       output_path = 'M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4_op\Merge_Monthly_AMSU_FOV_V4\results\'
           
       ok = check_path(report_path,/create)
       ok = check_path(log_path,/create)
       ok = check_path(output_path,/create)
       
       defsysv,'!log_lun',2
       log_file = string(log_path,channel_str,version_num,format='(a,"merge_log_fov_amsu_channel_",a,"_v4.",i7.7,".log.txt")')
       close,!log_lun
       openw,!log_lun,log_file
       log,'Info: AMSU Only V 4 log file"
       log,'Info:','Channel = ',channel_str
       log,'Info:','Diurnal model =',diurnal_model
       log,'Info:','EIA_model = ',eia_model
       log,'Info:','Version Num = 4.'+string(version_num,format = '(i7.7)')
       sats_used_str = ''
       for satnum = 0,num_sats - 1 do begin
         if sats_to_use[satnum] gt 0 then sats_used_str = sats_used_str + sat_name(satnum + 10)+' '
       endfor
       log,'Info:','Satellites Used: ',sats_used_Str
       
       ;--------------------------------------------------------------------
       ; calculate and apply target factors
       ;-------------------------------------------------------------------
       tb_arr_adj = tb_arr
       calc_TF_from_grpt_maps_AMSU_v4,tb_arr_adj,tar_arr,num_arr,channel_str,sats_to_use,target_factor,target_means,offset,months_to_use, $
         num_thres=num_thres,months_to_use_mask=months_to_use_mask,tf_surf=tf_Surf
       log,'Info: Target Factors =',target_factor
       log,'Info: Target Means   =',target_means

       not_used = where(sats_to_use lt 1,ct)
       if ct gt 0 then target_factor[not_used] = !values.f_nan
       if ct gt 0 then target_means[not_used] = !values.f_nan


       log,'Info: Applying Target Factors'
       apply_TF_to_grpt_maps_AMSU_v4,tb_arr_adj,tar_arr,channel,target_factor,target_means,months_to_use,tb_arr_adj2
       for month = 0,num_months-1 do begin
         for satnum = 0,7 do begin
           if months_to_use[month,satnum] lt 1 then tb_arr_adj2[*,*,month,satnum] = !values.f_nan
         endfor
       endfor
        
       if perform_diurnal_adjustment ge 1 then begin  ; calculate optimized diurnal adjustments
         if use_seasonal_diurnal_fit then begin
            diurnal_residual_parameters = fltarr(14,2,72)
         endif else begin
            diurnal_residual_parameters = fltarr(10,2,72)
         endelse
         
         if use_seasonal_diurnal_fit then begin
           log,'Info: Optimizing Diurnal Adjustment with seasonally modulated second harmonic'
         endif else begin
           log,'Info: Optimizing Diurnal Adjustment with seasonally constant second harmonic'
         endelse
         
         for ilat = 0,71 do begin
            lat_range = [-90.0 + 2.5*(ilat-2),-90.0 + 2.5*(ilat+3)]    
            if use_seasonal_diurnal_fit then begin
              fit_residual_diurnal_w_seasonal,tb_arr_adj2,lat_range,params
            endif else begin
              fit_residual_diurnal,tb_arr_adj2,lat_range,params
            endelse
            
            ; assign sea and land params the land_sea values for ilat = 0..17
            if ilat le 17 then begin
              params[*,0] = params[*,2]
              params[*,1] = params[*,2]
            endif
            diurnal_residual_parameters[*,*,ilat] = params[*,0:1]
         endfor

         ; uncomment these lines to save diurnal residual parameters in an IDL save file.
         ;save_file = string(report_path,channel_str,version_num,format='(a,"diurnal_residual_parameters_chan_",a,"_v",i7.7,".sav")')
         ;save,file=save_file,diurnal_residual_parameters
         
         diurnal_residual_parameters[0:7,*,*] = 0.0  ; zero the offsets for this step!  They'll be fixed later in calc_offsets_from_grpt_maps_AMSU_v4
         log,'Info: Applying Diurnal Adjustments'
         apply_diurnal_fit_residuals,tb_arr_adj2,diurnal_residual_parameters,tb_arr_adj_temp
         tb_arr_adj2 = tb_arr_adj_temp
        endif ; perform diurnal adjustment

        ;file = string(report_path,channel_str,version_num,format = '(a,"merge_report_fov_amsu_channel_",a,"_v4.",i7.7,".ps")')
        ;set_up_ps,file=file,xsize=7.5,ysize=10.0
        ;log,'Info: merge_report_file =',file   

        ; do the offset calculation separately for land and ocean
        
        calc_offsets_from_grpt_maps_AMSU_v4_land_ocean_sep,tb_arr_adj2,months_to_use,offsets
        adjust_offsets_by_land_fraction,offsets,offsets_adj
        offsets_adj_sm = offsets_adj
        smooth_offsets,offsets_adj[*,*,0],temp
        offsets_adj_sm[*,*,0] = temp
        smooth_offsets,offsets_adj[*,*,1],temp
        offsets_adj_sm[*,*,1] = temp
        tb_arr_adj3 = tb_arr_adj2
        apply_offsets_to_grpt_maps_AMSU_v4_land_ocean_sep,tb_arr_adj3,offsets_adj_sm
        
        ; perform the merge
        merge_tb_maps_AMSU_v4,tb_arr_adj3,num_arr,tb_arr_merged,months_used = months_used,num_thres = 4
        
        ; expand merge coefficients to maps for metadata
        offset_arr = replicate(!values.f_nan,144,72,num_sats)
        tarmult_arr = replicate(!values.f_nan,144,72,num_sats)
        tbmult_arr = fltarr(144,72,num_sats)
        
        ;expanding offsets for metadata.
        for sat = 0,num_sats-1 do begin
          tarmult_arr[*,*,sat] = target_factor[sat]
        endfor
        
        sz_offsets = size(offsets,/dimensions)
        
        if n_elements(sz_offsets) eq 2 then begin   
          for ilon = 0,143 do begin
            offset_arr[ilon,*,*] = offsets
          endfor
        endif else begin  ; sz offsets should be [72,9,2]  
          bad = where(finite(offsets[*,*,1]) eq 0,ct)
          if ct gt 0 then begin
            temp1 = offsets[*,*,1]
            temp2 = offsets[*,*,0]
            temp1[bad] = temp2[bad]
            offsets[*,*,1] = temp1
          endif
          for sat = 0,num_sats-1 do begin
            for ilat = 0,71 do begin
              offset_arr[*,ilat,sat] = offsets[ilat,sat,0]*(1.0-lf[*,ilat]) + offsets[ilat,sat,1]*lf[*,ilat]
            endfor
          endfor
        endelse

        compare_new_merged_amsu_to_baseline,tb_arr_merged,target_factor,months_to_use,version_num,channel=channel
       
        ; write the output file
       
        write_amsu_only_merged_Tbs_144_72_netcdf_4_0,       $
                      Tbs_144_72 = tb_arr_merged ,          $
                      channel_str=channel_str,              $
                      version_num=version_num,              $
                      diurnal_model = diur__model_loaded,   $
                      eia_model = eia__model_loaded,        $
                      fov_wts = fov_wts,                    $
                      node_wts = node_wts,                  $
                      adjustment_flag = adj_flag,           $
                      offsets = offset_arr,                 $  ;[144,72,num_sats]
                      target_factors = tarmult_arr,         $  ;[144,72,num_sats]
                      tb_Factors = tbmult_arr,              $  ;[144,72,num_sats]
                      target_means = target_means,          $  ;[num_sats]
                      months_used = months_used,            $  ;[num_months,num_sats]
                      output_path = output_path
                      
        save_file = 'M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4_op\Merge_Monthly_AMSU_FOV_V4\adjusted_tbs\'
        save_file = save_file + 'RSS_Tb_Maps_ch_TMT_V4_0_0000001.sav'
        save,file=save_File,tb_arr_raw,tb_arr,tb_arr_adj2,tb_arr_adj3,num_arr,tb_arr_merged,months_used
        log,'Info: Normal End'
        close,!log_lun
        print
        device,/close
        set_win
end
