;@***h* MLT_V4_RSS/amsu_v4_grpt_merge_refactor.tmt_op.pro
; NAME
;   amsu_v4_grpt_merge_refactor.tts_op.pro
;
; PURPOSE
;   top level program that performs TTS merge for AMSU
;
; DESCRIPTION
;   top level program that performs TTS merge for AMSU.  The merging process is described in 
;
;   Mears, C. A., & Wentz, F. J. (2016). Sensitivity of Satellite-Derived Tropospheric Temperature
;   Trends to the Diurnal Cycle Adjustment. Journal of Climate, 29, 3629-3646. doi: 10.1175/JCLI-D-15-0744.1
;
;   The program defines several common block, then read in data and performs simple QC analysis to 
;   remove data that is too sparse for a given month/satellite.  The program then calculates the various merging parameters.
;   They are:
;     Target Factors
;     Adjustments to the diurnal correction
;     Intersatellite offsets
;   
;   The merged data product is then written.
; INPUTS
;   input variables
; OUTPUTS
;   output variable and descriptions
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   1/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   .r amsu_v4_grpt_merge_refactor.tts_op.pro
;
;@*****

; top level program
reread_data = 1
use_seasonal_diurnal_fit = 1


load_AMSU_file_locations,verbose=1
read_land_fraction_144_72,lf
land_mask = intarr(144,72)
land = where(lf gt 0.5)
land_mask[land] = 1.0

start_year = 1998
end_year = 2018
num_months = 12*(1 + end_year - start_year)
num_sats = 8

; define commom blocks
; EIA common block -- contains info need for the angle adjustments

common eia, eia__adj_arr, eia__channel_loaded,eia__model_loaded,eia__lat_map, $
       eia__cross_track_offsets,eia__cross_track_sat_loaded,eia__cross_track_channel_loaded, $
       AMSU_VIEW_ANGLES,AMSU_NOM_EIAS

eia__channel_loaded = -1
eia__cross_track_sat_loaded = -1
eia__cross_track_channel_loaded = -1

eia__model_loaded = 'none'

AMSU_VIEW_ANGLES = [1.6666666,5.0000000,8.3333333,11.666667,15.000000,  $
                    18.333333,21.666667,25.000000,28.333333,31.666667,  $
                    35.000000,38.333333,41.666667,45.000000,48.333333]

AMSU_NOM_EIAS    = [1.875947, 5.629541, 9.388301,13.155880,16.936250,   $
                    20.733890,24.554020,28.402830,32.287970,36.219100,  $
                    40.208800,44.274040,48.438570,52.737200,57.224260]
                    
; END EIA common block

; Diurnal common block
common diurnal, diur__adj_arr, diur__channel_loaded,diur__model_loaded,diur__lat_time_map
  diur__channel_loaded = -1
  diur__model_loaded  = 'none'
; END Diurnal Common Block

fov_list = ['central8','central12','central16','central18','central24','limb8']
channels = ['TLT','TMT','TTS','TLS']
eia_model = 'MERRA'
diurnal_models = ['NONE','CCM3','HADGEM','MERRA','CMAM','NONE','CCM3','HADGEM','MERRA','CMAM']

offset_scheme = 'land_ocean_together'
;offset_scheme = 'land_ocean_separate'
;offset_scheme = 'ocean_only'

case offset_scheme of
  'land_ocean_together': begin
    offset_scheme_index = 1
  end
  'land_ocean_separate': begin
    offset_scheme_index = 2
  end
  'ocean_only': begin
    offset_scheme_index = 3
  end
endcase

tf_schemes = ['land_and_ocean']
;tf_schemes = ['ocean_only']
;tf_schemes = ['NONE']
foreach tf_scheme,tf_schemes do begin
  case tf_scheme of
    'land_and_ocean':begin
    tf_surf = 'Land_and_Sea'
    tf_scheme_index = 1
    end
    'ocean_only':begin
    tf_surf = 'Sea'
    tf_scheme_index = 2
    end
    'NONE':begin
    tf_surf = 'Land_and_Sea'
    tf_scheme_index = 3
    end
  endcase
  for fov_index = 4,4 do begin
    fov_set = fov_list[fov_index]
    ;fov_set = 'central9'
    case fov_set of
      'central8': begin
        fov_ok = [11,12,13,14,15,16,17,18]
        num_fovs_used = 8
      end
      'central12': begin
        fov_ok = [9,10,11,12,13,14,15,16,17,18,19,20]
        num_fovs_used = 12
      end
      'central16': begin
        fov_ok = [7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22]
        num_fovs_used = 16
      end
      'central18': begin
        fov_ok = [6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23]
        num_fovs_used = 18
      end
      'central24': begin
        fov_ok = [3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26]
        num_fovs_used = 24
      end
      'limb8': begin
        fov_ok = [6,7,8,9,20,21,22,23]
        num_fovs_used = 8
       end
     endcase

    for channel_index = 2,2 do begin

      channel = channels[channel_index]
      
      case channel of 
        'TLT':begin
          message,'TLT does not really work with this method because of sampling issues'
          stop
          end
        'TMT':begin
          amsu_channel = 5
          fov_wts = fltarr(30)
          fov_wts[fov_ok] = 1.0
          channel_str = channel
          eia_model = 'MERRA'
          sats_to_use = [1,0,0,1,1,1,1,1] 
          num_sats = n_elements(sats_to_use)
          node_wts = [1.0,1.0]
          adj_flag = 7
          ;print,'THIS FLAG SHOULD BE 7!'
          months_to_use_mask = intarr(num_months,num_sats)
          ;months_to_use_mask[58,1] = -1
          ;months_to_use_mask[84,1] = -1
          ;months_to_use_mask[141,5] = -1
          ;months_to_use_mask[140,5] = -1
          smooth_maps = 0
          num_thres = 14
          do_tlt_adjustment = 0
          ;stop processing of AQUA after June 2012
          
          
          end
        'TTS':begin
          amsu_channel = 7
          fov_wts = fltarr(30)
          fov_wts[fov_ok] = 1.0
          channel_str = channel
          eia_model = 'MERRA'
          sats_to_use = [1,0,0,1,0,1,1,1] 
          num_sats = n_elements(sats_to_use)
          node_wts = [1.0,1.0]
          adj_flag = 7
          months_to_use_mask = intarr(num_months,num_sats)
          ;months_to_use_mask[58,1] = -1
          ;months_to_use_mask[84,1] = -1
          ;months_to_use_mask[141,5] = -1
          ;months_to_use_mask[140,5] = -1
          smooth_maps = 0
          num_thres = 12
          do_tlt_adjustment = 0
          end 
        'TLS':begin
          amsu_channel = 9
          fov_wts = fltarr(30)
          fov_wts[6:9] = 1.0
          fov_wts[20:23] = 1.0
          channel_str = channel
          eia_model = 'MERRA'
          sats_to_use = [1,0,0,1,1,1,1,1] 
          num_sats = n_elements(sats_to_use)
          node_wts = [1.0,1.0]
          adj_flag = 7
          months_to_use_mask = intarr(num_months,num_sats)
          smooth_maps = 0
          num_thres = 12
          do_tlt_adjustment = 0
          end
      endcase

      use_seasonal_diurnal_fit = 0
      perform_fully_empirical_adjustment = 0
      land_only_adjustment = 0
      perform_N15_ad_hoc_adjustment = 0
      perform_AQUA_ad_hoc_adjustment = 0
      combine_AQUA_METOPA = 0
      use_star_data = 0

      foreach diurnal_index,[0,1,2,3,5,6,7,8] do begin    
      ;foreach diurnal_index,[5,6,7,8] do begin
      ;foreach diurnal_index,[0,1,2,3] do begin
      ;foreach diurnal_index,[0,1,2,3] do begin
      ;foreach diurnal_index,[6] do begin
        if ((diurnal_index eq 4) and (channel_index lt 3)) then continue  ; no CMAM except for TLS
        if ((diurnal_index eq 9) and (channel_index lt 3)) then continue  ; no CMAM except for TLS
        diurnal_model = diurnal_models[diurnal_index]
        diurnal_factor = 1.0
        perform_diurnal_adjustment = 0
        if diurnal_index ge 5 then perform_diurnal_adjustment = 1       
        
        data_file = 'RSS_AMSU_monthly_maps_'+ channel + '_' + fov_set + '_' + diurnal_model+'.sav'
        if keyword_Set(use_Star_data) then begin
          data_file = 'STAR_AMSU_monthly_maps_'+ channel + '_' + fov_set + '_' + diurnal_model+'.sav'      
        endif
        data_file ='M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\setup\'+data_file
        info = file_info(data_File) 
        if info.exists eq 9 then begin
          print,'Restoring '+data_file
          restore,data_file
        endif else begin
            if keyword_Set(use_Star_data) then begin
              read_all_STAR_AMSU_maps_from_FOV, $
                                                amsu_channel,                 $
                                                fov_wts,                      $
                                                node_wts,                     $
                                                adj_flag,                     $
                                                num_sats,           $
                                                num_months,         $
                                                tb_Arr,                       $
                                                tar_arr = tar_Arr,            $
                                                num_Arr = num_Arr,            $
                                                sats_to_use = sats_to_use,    $
                                                diurnal_model = diurnal_model,$
                                                eia_model = eia_model,        $
                                                diurnal_factor = diurnal_factor
                tb_arr[*,*,89,3] = !values.f_nan
                num_arr[*,*,89,3] = 0
                num_arr = num_arr*2.0
            endif else begin
                read_all_adjusted_AMSU_maps_from_FOV, amsu_channel,       $
                                              fov_wts,            $
                                              node_wts,           $
                                              adj_flag,           $
                                              num_sats,           $
                                              num_months,         $
                                              tb_arr,         $
                                              tar_arr=tar_arr,    $
                                              num_arr=num_arr,    $
                                              sats_to_use = sats_to_use,     $
                                              diurnal_model = diurnal_model, $
                                              diurnal_factor = diurnal_factor,          $
                                              eia_model = eia_model, $;, $
                                              tb_arr_raw = tb_arr_raw
           endelse
           print
           save,tb_arr,tb_arr_raw,tar_arr,num_arr,sats_to_use,months_to_use_mask,diurnal_model,diurnal_factor,eia_model,use_Star_data,file = data_file 
        endelse
        if smooth_maps eq 1 then begin
          tb_arr_adj = tb_arr
          sz = size(tb_arr_adj,/dimensions)
          num_months = sz[2]
          num_sats = sz[3]
          for satnum = 0,num_Sats-1 do begin
            if sats_to_use[satnum] gt 0 then begin
              for m = 0,num_months-1 do begin
                map = tb_arr_adj[*,*,m,satnum]
                ok = where(finite(map) gt 0,ct)
                if ct gt 2 then begin
                  tb_arr_adj[*,*,m,satnum] = smooth_144_72_map(map)
                  tar_arr[*,*,m,satnum] = smooth_144_72_map(tar_arr[*,*,m,satnum])
                endif
              endfor
            endif
          endfor
        endif else begin
          tb_arr_adj = tb_arr
        endelse
              
        ;sats_to_use_schemes = ['v33','v33_aqua_trunc','min_diur_drift','all','all_aqua_trunc']
        sats_to_use_schemes = ['all_aqua_n15_trunc']
        ;sats_to_use_schemes = ['all_aqua_trunc']
        ;sats_to_use_schemes = ['min_diur_drift']
        ;sats_to_use_schemes = ['v33']
        foreach sats_to_use_scheme,sats_to_use_schemes do begin
         months_to_use_mask = intarr(num_months,num_sats)
         case sats_to_use_scheme of
          'all':begin
             months_to_use_mask[180:num_months-1,5] = -1   
             months_to_use_mask[0:164,6] = -1   ; remove noaa-19 before 2012  
             months_to_use_mask[0:164,3] = -1   ; remove noaa-18 before 2012       
             sats_to_use_index = 1L
             sats_to_use = [1,0,0,1,0,1,1,1]
             end
          'v33': begin
             months_to_use_mask[*,6] = -1
             months_to_use_mask[*,7] = -1
             months_to_use_mask[180:num_months-1,5] = -1
             sats_to_use_index = 0L
             sats_to_use = [1,0,0,1,1,1,0,0]
             end
           'v33_aqua_trunc': begin
             months_to_use_mask[*,6] = -1
             months_to_use_mask[*,7] = -1
             months_to_use_mask[156:num_months-1,5] = -1
             sats_to_use_index = 3L
             sats_to_use = [1,0,0,1,1,1,0,0]
             end
           'min_diur_drift':begin
             months_to_use_mask[162,6] = 1
             months_to_use_mask[162,4] = 1
             months_to_use_mask[163,6] = 1
             months_to_use_mask[163,4] = 1
             ;months_to_use_mask[70,5] = 1  ; only use when truncating NOAA-15 before this time
             ;months_to_use_mask[0:164,6] = -1             ; remove NOAA-19 before 2012.  This is always necessary -- data are screwed up with lots of gaps            
             months_to_use_mask[72:num_months-1,0] = -1  ;remove NOAA-15 after 2003  -  to avoid NOAA-15 diurnal drift
             ; 72 is the baseline decision
             
             months_to_use_mask[156:num_months-1,3] = -1  ;remove NOAA-18 after 2011  - to avoid NOAA-18 diurnal drift
             months_to_use_mask[144:num_months-1,5] = -1  ; AQUA after 2009  - to avoid AQUA calibration drifts
             ;months_to_use_mask[144+36:num_months-1,5] = -1  ; AQUA after 2013  - to avoid AQUA calibration drifts
             sats_to_use_index = 2L
             sats_to_use = [1,0,0,1,1,1,1,1]
             end
             'all_aqua_trunc':begin
             sats_to_use_index = 4L
             months_to_use_mask[144:num_months-1,5] = -1  ; truncate AQUA after 2009
             ;months_to_use_mask[161,4] = 1
             months_to_use_mask[162,4] = 1
             months_to_use_mask[163,4] = 1
             sats_to_use = [1,0,0,1,1,1,1,1]
             end
             'all_aqua_n15_trunc':begin
             sats_to_use_index = 5L
             months_to_use_mask[144:num_months-1,5] = -1  ; truncate AQUA after 2009
             months_to_use_mask[168:num_months-1,0] = -1  ; truncate NOAA-15 after 2011
             ;months_to_use_mask[161,4] = 1
             months_to_use_mask[162,4] = 1
             months_to_use_mask[163,4] = 1
             sats_to_use = [1,0,0,1,0,1,1,1]
             end
       endcase
       adhoc_index = perform_N15_ad_hoc_adjustment + 2*perform_aqua_ad_hoc_adjustment
       version_num = (1000000L*land_only_adjustment + 100000L*(use_seasonal_diurnal_fit)) + (10000L*(sats_to_use_index)) + (1000L*offset_scheme_index) + (100L*tf_scheme_index)  + (10L*fov_index) + diurnal_index
       version_str = string(version_num,format = '(i7.7)')
       report_path = 'M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\reports\'+channel_str+'\'+version_str+'\'
       log_path    = 'M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\logs_\'+channel_str+'\'+version_str+'\'
       output_path = 'M:\MSU_AMSU_Data\AMSU-A_L3\MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\results\'+channel_str+'\'+version_str+'\'
       
       if keyword_Set(use_star_data) then begin
         report_path = 'C:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\reports_star\'+channel_str+'\'+version_str+'\'
         log_path    = 'C:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\logs_star\'+channel_str+'\'+version_str+'\'
         output_path = 'C:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\results_star\'+channel_str+'\'+version_str+'\'
       endif
       
       ok = check_path(report_path,/create)
       ok = check_path(log_path,/create)
       ok = check_path(output_path,/create)
       
       defsysv,'!log_lun',2
       log_file = string(log_path,channel_str,version_num,format='(a,"merge_log_fov_amsu_channel_",a,"_v4.",i7.7,".log.txt")')
       close,!log_lun
       openw,!log_lun,log_file
       log,'Info: AMSU Only V 4 log file"
       log,'Info:','Channel = ',channel_str
       log,'Info:','Diurnal model =',diurnal_model
       log,'Info:','EIA_model = ',eia_model
       log,'Info:','Version Num = 4.'+string(version_num,format = '(i7.7)')
       sats_used_str = ''
       for satnum = 0,num_sats - 1 do begin
         if sats_to_use[satnum] gt 0 then sats_used_str = sats_used_str + sat_name(satnum + 10)+' '
       endfor
       log,'Info:','Satellites Used: ',sats_used_Str
       
       ;--------------------------------------------------------------------
       ; calculate and apply target factors
       ;--------------------------------------------------------------------
       ;save,file = 'B:\MSU_AMSU\IDL_MSU_AMSU_V4\diur_sensitivity\amsu_tar_arr.sav',tar_arr
       calc_TF_from_grpt_maps_AMSU_v4_TTS,tb_arr_adj,tar_arr,num_arr,channel_str,sats_to_use,target_factor,target_means,offset,months_to_use,num_thres=num_thres,months_to_use_mask=months_to_use_mask,tf_surf=tf_Surf,report_path=report_path
       log,'Info: Target Factors =',target_factor
       log,'Info: Target Means   =',target_means
       save,file = 'B:\MSU_AMSU\IDL_MSU_AMSU_V4\diur_sensitivity\amsu_tar_arr_TTS.sav',tar_arr,months_to_use,num_arr,months_to_use_mask,sats_to_use

       not_used = where(sats_to_use lt 1,ct)
       if ct gt 0 then target_factor[not_used] = !values.f_nan
       if ct gt 0 then target_means[not_used] = !values.f_nan

       if tf_scheme_index ne 3 then begin
         log,'Info: Applying Target Factors'
         apply_TF_to_grpt_maps_AMSU_v4,tb_arr_adj,tar_arr,channel,target_factor,target_means,months_to_use,tb_arr_adj2
         apply_TF_to_grpt_maps_AMSU_v4,tb_arr_raw,tar_arr,channel,target_factor,target_means,months_to_use,tb_arr_raw2
         for month = 0,num_months-1 do begin
           for satnum = 0,7 do begin
             if months_to_use[month,satnum] lt 1 then tb_arr_adj2[*,*,month,satnum] = !values.f_nan
             if months_to_use[month,satnum] lt 1 then tb_arr_raw2[*,*,month,satnum] = !values.f_nan
           endfor
         endfor
      
       endif else begin
         log,'Info: Target Factors NOT Applied'
         tb_Arr_adj2 = tb_arr_adj
         for satnum = 0,num_sats-1 do begin
           for month = 0,num_months-1 do begin
             if months_to_use[month,satnum] le 0 then begin
               tb_arr_adj2[*,*,month,satnum] = !values.f_nan
             endif
           endfor
         endfor
       endelse

       if combine_AQUA_METOPA gt 0 then begin
         ;-----------------------------
         ; Adjust METOP-A to match AQUA
         ;-----------------------------
         set_up_ps,xsize=8.0,ysize=6.0,file = string(version_num,format = '("B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\reports_jclim\AQUA_METOPA_plots\AQUA_METOPA_Diffs_",i8.8,".ps")')
         eval_aqua_metop_a_diff_v4_amsu,tb_arr_adj2,zonal_diff_fit,title2=title2
         set_win
         ; assign sea and land params the land_sea values for ilat = 0..17
         
         zonal_diff_fit[0:17,*,0] = zonal_diff_fit[0:17,*,2]
         zonal_diff_fit[0:17,*,1] = zonal_diff_fit[0:17,*,2]
  
         temp = tb_arr_adj2
         apply_adjustment_to_metop_a,tb_arr_adj2,zonal_diff_fit
         
         
         ;plot_aqua_metopa_time_series,temp,tb_arr_adj2
         ; now metop-a is referred to the aqua measurement time of about 1:30
         
         combine_aqua_metop_a,tb_arr_adj2,tar_arr,num_arr
         sats_to_use = [1,0,0,1,0,1,1,1]
         months_to_use[*,4] = 0
         months_to_use[120:203,5] = 1
         version_num = version_num + 2000000L
       endif
       
       if perform_fully_empirical_adjustment ge 1 then begin
            global_adj_Arr_baseline = replicate(!values.f_nan,204,8,3)
            global_adj_Arr = replicate(!values.f_nan,204,8,3)
            version_num = version_num + 2000000L
            ; in this scheme, we adjust each zonal band of NOAA-15, NOAA-18, and NOAA-19 to
            ; match the combined aqua/metop-a Tbs found in the section above.
            for satnum = 0,7 do begin
              if satnum eq 1 then continue
              if satnum eq 2 then continue
              if satnum eq 4 then continue
              if satnum eq 5 then continue
              calc_zonal_diff_from_aqua_metopa,satnum,months_to_use,tb_arr_adj2,zonal_diff_n15
              surf_list = ['Sea','Land','Land_and_Sea']
              ;plot_zonal_diff_arr,zonal_diff_n15,0,surf_list,1998,2014,'NOAA-15 minus AQUA_METOPA'
              ;plot_zonal_diff_arr,zonal_diff_n15,1,surf_list,1998,2014,'NOAA-15 minus AQUA_METOPA'
            
              if satnum eq 0 then begin
                smooth_and_extend_zonal_diff_arr,zonal_diff_n15,zonal_diff_n15_smoothed,/extend_earlier
              endif else begin
                smooth_and_extend_zonal_diff_arr,zonal_diff_n15,zonal_diff_n15_smoothed
              endelse
              
              plot_zonal_diff_arr,zonal_diff_n15_smoothed,0,surf_list,1998,2014,'NOAA-15 minus AQUA_METOPA'
              plot_zonal_diff_arr,zonal_diff_n15_smoothed,1,surf_list,1998,2014,'NOAA-15 minus AQUA_METOPA'
   
              tb_arr_temp = tb_arr_adj2
              read_land_fraction_144_72,lf
              for m = 0,num_months-1 do begin
                if months_to_use[m,satnum] eq 1 then begin
                  adj_map = fltarr(144,72)
                  baseline_diur_adj_map = tb_arr_adj2[*,*,m,satnum] - tb_arr_raw[*,*,m,satnum]
                  for ilat = 0,71 do begin
                    adj_map[*,ilat] = lf[*,ilat]*zonal_diff_n15_smoothed[ilat,m,1] + (1.0 - lf[*,ilat])*zonal_diff_n15_smoothed[ilat,m,0]
                  endfor
                  tb_arr_adj2[*,*,m,satnum] = tb_arr_adj2[*,*,m,satnum] - adj_map
                  tot_diur_adj_map = tb_arr_adj2[*,*,m,satnum] - tb_arr_raw[*,*,m,satnum]
                  
                  lat_range=[-60.0,60.0]
                  num_lats = 72
                  lat_wts = latitude_wts(num_lats,lat_range)
                  global_adj_Arr[m,satnum,0] = compute_global_average_from_144_72_map(tot_diur_adj_map,lat_wts,surf_str = 'Sea',/no_cosine)
                  global_adj_Arr[m,satnum,1] = compute_global_average_from_144_72_map(tot_diur_adj_map,lat_wts,surf_str = 'Land',/no_cosine)
                  global_adj_Arr[m,satnum,2] = compute_global_average_from_144_72_map(tot_diur_adj_map,lat_wts,surf_str = 'Land_and_Sea',/no_cosine)
  
                  global_adj_Arr_baseline[m,satnum,0] = compute_global_average_from_144_72_map(baseline_diur_adj_map,lat_wts,surf_str = 'Sea',/no_cosine)
                  global_adj_Arr_baseline[m,satnum,1] = compute_global_average_from_144_72_map(baseline_diur_adj_map,lat_wts,surf_str = 'Land',/no_cosine)
                  global_adj_Arr_baseline[m,satnum,2] = compute_global_average_from_144_72_map(baseline_diur_adj_map,lat_wts,surf_str = 'Land_and_Sea',/no_cosine)
                endif
              endfor
            endfor
            plot_num = 1
            for satnum = 0,7 do begin

              if satnum eq 1 then continue
              if satnum eq 2 then continue
              if satnum eq 4 then continue
              if satnum eq 5 then continue

              ok = where(finite(global_adj_arr[*,satnum,1]) eq 1,ct)
              print,ct,ok[0:11]
              yrs = 1998.0 + 1.0/24.0 + findgen(204)/12.
              if plot_num eq 1 then begin
                p = plot(yrs,0.4 + global_adj_Arr[*,satnum,1]- mean(global_adj_Arr[ok[0:11],satnum,1],/nan),dimension = [1100,700],layout = [2,2,plot_num],color='red')
              endif else begin
                p = plot(yrs,0.4 + global_adj_Arr[*,satnum,1]- mean(global_adj_Arr[ok[0:11],satnum,1],/nan),/current,layout = [2,2,plot_num],color='red')
              endelse
              p1 = plot(yrs,fltarr(2004),color='gray',/overplot)
              p1 = plot(yrs,0.4 + fltarr(2004),color='gray',/overplot)


              p1 = plot(yrs,0.4 + global_adj_Arr_baseline[*,satnum,1]- mean(global_adj_Arr_baseline[ok[0:11],satnum,1],/nan),/overplot,color='tomato')
              p1 = plot(yrs,global_adj_Arr_baseline[*,satnum,0]- mean(global_adj_Arr_baseline[ok[0:11],satnum,0],/nan),/overplot,color='sky_blue')
              p1 = plot(yrs,global_adj_Arr[*,satnum,0]- mean(global_adj_Arr[ok[0:11],satnum,0],/nan),/overplot,color='blue')
              p.yrange=[-0.1,0.8]
              p.xrange= [1998,2016]
              p.title = sat_name(satnum + 10)
              plot_num  = plot_num + 1
            endfor

            png_file = string(version_num,format = '("B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\refer_To_aqua_METOPA_results\diurnal_adjustments_",i8.8,".png")')
            p.save,png_file
            ps_file = string(version_num,format = '("B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\refer_To_aqua_METOPA_results\diurnal_adjustments_",i8.8,".ps")')
            p.save,ps_file      
            
            sav_file =    string(version_num,format = '("B:\MSU_AMSU\IDL_MSU_AMSU_V4\Merge_Monthly_AMSU_FOV_V4\refer_To_aqua_METOPA_results\diurnal_adjustments_",i8.8,".sav")') 
            save,file=sav_file,global_adj_Arr,global_adj_Arr_Baseline
       endif


 
       if perform_diurnal_adjustment ge 1 then begin
         ; In in this section, the diurnal adjustment is optimized using an additional
         ; second harmonic adjustment.
         ; This adjustment can either be seasonally modulated or not.  This doesn't
         ; make much difference for the overall results, but the intersatellite differences
         ; are better near the poles with the seasonal modulation

         if use_seasonal_diurnal_fit then begin
            diurnal_residual_parameters = fltarr(14,2,72)
         endif else begin
            diurnal_residual_parameters = fltarr(10,2,72)
         endelse
         t = findgen(120)/10.0
        
         if use_seasonal_diurnal_fit then begin
           log,'Info: Optimizing Diurnal Adjustment with seasonally modulated second harmonic'
         endif else begin
           log,'Info: Optimizing Diurnal Adjustment with seasonally constant second harmonic'
         endelse
         
         for ilat = 0,71 do begin
            lat_range = [-90.0 + 2.5*(ilat-2),-90.0 + 2.5*(ilat+3)]  
  
            if use_seasonal_diurnal_fit then begin
              fit_residual_diurnal_w_seasonal,tb_arr_adj2,lat_range,params
            endif else begin
              fit_residual_diurnal,tb_arr_adj2,lat_range,params
            endelse
            
            ; assign sea and land params the land_sea values for ilat = 0..17
            if ilat le 17 then begin
              params[*,0] = params[*,2]
              params[*,1] = params[*,2]
            endif
            
            ;for the case of TTS, we don't use separate land and ocean diurnal cycles.
            if channel_str eq 'TTS' then begin
              params[*,0] = params[*,2]
              params[*,1] = params[*,2]
            endif  
            diurnal_residual_parameters[*,*,ilat] = params[*,0:1]
         endfor
         
         if keyword_set(land_only_adjustment) then begin
            diurnal_residual_parameters[*,0,*]  = 0.0
         endif
         save_file = string(report_path,channel_str,version_num,format='(a,"diurnal_residual_parameters_chan_",a,"_v",i7.7,".sav")')
         print,save_file
         save,file=save_file,diurnal_residual_parameters
         
         diurnal_residual_parameters[0:7,*,*] = 0.0  ; zero the offsets for this step!  They'll be fixed later in calc_offsets_from_grpt_maps_AMSU_v4
         log,'Info: Applying Diurnal Adjustments'
         apply_diurnal_fit_residuals,tb_arr_adj2,diurnal_residual_parameters,tb_arr_adj_temp
         
         ; make plot of diurnal adjustments relative to tb_arr_raw
         ; tb_arr_raw has no diurnal applied
         ; tb_arr_adj2 
         plot_AMSU_diurnal_adjustments_TTS,tb_arr_raw2,tb_arr_adj2,tb_Arr_adj_temp,version_num,report_path
         tb_arr_adj2 = tb_arr_adj_temp
        endif ; perform diurnal adjustment

      
        ; for TLS and TTS, the only choice is for the offsets to be independent of surface type
        
        calc_offsets_from_grpt_maps_AMSU_v4,tb_arr_adj2,months_to_use,offsets,plt_offsets=0
        for satnum = 0,num_sats-1 do begin
          if sats_to_use[satnum] lt 1 then offsets[*,satnum] = !values.f_nan
        endfor
        smooth_offsets,offsets,offsets_smooth
        plot_offsets_TTS,offsets,offsets2=offsets_smooth,report_path=report_path
        ; offsets should be applied in the above version of calc_offsets_from_grpt_maps
        tb_arr_adj3 = tb_arr_adj2
        apply_offsets_to_grpt_maps_AMSU_v4,tb_arr_adj3,offsets_smooth   
        eval_grpt_merge_global_amsu_tts,  tb_arr_adj3,num_arr,months_to_use,raw_Tbs = tb_arr,mean_rms = mean_rms,mean_Stddev = mean_stddev,stddev_diff = stddev_diff,mean_diffs = mean_diffs,channel = channel_str,report_path = report_path
     
        ; perform the merge
        merge_tb_maps_AMSU_v4,tb_arr_adj3,num_arr,tb_arr_merged,months_used = months_used,num_thres = 4
        
        ; expand merge coefficients to maps for metadata
        offset_arr = replicate(!values.f_nan,144,72,num_sats)
        tarmult_arr = replicate(!values.f_nan,144,72,num_sats)
        tbmult_arr = fltarr(144,72,num_sats)
        
        ;need to fix this eventually - -just expanding offsets for metadata.
        for sat = 0,num_sats-1 do begin
          tarmult_arr[*,*,sat] = target_factor[sat]
        endfor
        
        sz_offsets = size(offsets,/dimensions)
        
        if n_elements(sz_offsets) eq 2 then begin   
          for ilon = 0,143 do begin
            offset_arr[ilon,*,*] = offsets
          endfor
        endif else begin  ; sz offsets should be [72,9,2]  
          bad = where(finite(offsets[*,*,1]) eq 0,ct)
          if ct gt 0 then begin
            temp1 = offsets[*,*,1]
            temp2 = offsets[*,*,0]
            temp1[bad] = temp2[bad]
            offsets[*,*,1] = temp1
          endif
          for sat = 0,num_sats-1 do begin
            for ilat = 0,71 do begin
              offset_arr[*,ilat,sat] = offsets[ilat,sat,0]*(1.0-lf[*,ilat]) + offsets[ilat,sat,1]*lf[*,ilat]
            endfor
          endfor
        endelse

        compare_new_merged_amsu_to_baseline_TTS,tb_arr_merged,target_factor,months_to_use,version_num,channel=channel,report_path=report_path
       
        ; write the output file
       
        write_amsu_only_merged_Tbs_144_72_netcdf_4_0,       $
                      Tbs_144_72 = tb_arr_merged ,          $
                      channel_str=channel_str,              $
                      version_num=version_num,              $
                      diurnal_model = diur__model_loaded,   $
                      eia_model = eia__model_loaded,        $
                      fov_wts = fov_wts,                    $
                      node_wts = node_wts,                  $
                      adjustment_flag = adj_flag,           $
                      offsets = offset_arr,                 $  ;[144,72,num_sats]
                      target_factors = tarmult_arr,         $  ;[144,72,num_sats]
                      tb_Factors = tbmult_arr,              $  ;[144,72,num_sats]
                      target_means = target_means,          $  ;[num_sats]
                      months_used = months_used,            $  ;[num_months,num_sats]
                      output_path = output_path
                      
        log,'Info: Normal End'
        close,!log_lun
        set_win
      endforeach
      endforeach
    endfor
  endfor
endforeach
end
