pro apply_diurnal_fit_residuals,tb_arr,params,tb_arr_adj
  ;@***h* MLT_V4_RSS/apply_diurnal_fit_residuals
  ; NAME
  ;   apply_diurnal_fit_residuals
  ;
  ; PURPOSE
  ;   Applies diurnal adjustments to antenna temperature gridded data.
  ;
  ; DESCRIPTION
  ;   Applies diurnal adjustments to antenna temperature gridded data. The parameters for each latitude band and
  ;   surface type are passed in the params variable
  ;
  ;   The 
  ;   best reference for this calculation is
  ;   
  ;   Mears, C. A., & Wentz, F. J. (2016). Sensitivity of Satellite-Derived Tropospheric Temperature 
  ;   Trends to the Diurnal Cycle Adjustment. Journal of Climate, 29, 3629-3646. doi: 10.1175/JCLI-D-15-0744.1
  ;     ;
  ;   The type of diurnal fit is figures out by the size of the array in the first dimension.
  ;   
  ; INPUTS
  ;   tb_arr          fltarr array (144 x 72 x num_months x num_sats) of antenna temperatures 
  ;   params          fltarr array (14 (or 8) x 2 x 72) array of paramters to apply
  ;                       The first dimension is offsets + harmonic paramters
  ;                       The second dimension is surface type
  ;                       The third dimensions if latitude in 2.5 degree bands.
  ;   
  ; OUTPUTS
  ;   tb_arr_adj      fltarr array (144 x 72 x num_months x num_sats) of antenna temperatures with diurnal optimized
  ;                   adjustments applied.
  ;
  ; AUTHOR
  ;   Carl Mears, Remote Sensing Systems
  ;
  ; COPYRIGHT
  ;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
  ;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
  ;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
  ;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
  ;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
  ;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
  ;   TO USERS.
  ;
  ; HISTORY
  ;   1/10/2018 Initial Version prepared for NCDC
  ;
  ; USAGE
  ;   apply_diurnal_fit_residuals,tb_arr_adj2,diurnal_residual_parameters,tb_arr_adj_temp
  ;
  ;@*****
  sz = size(tb_arr,/dimensions)
  num_lons = sz[0]
  num_lats = sz[1]
  num_months = sz[2]
  num_sats = sz[3]
  
  tb_Arr_adj = tb_arr
  
  restore,'M:\MSU_AMSU_data\AMSU_Navigation\AMSU_crossing_times.sav'
  amsu_monthly_xtime_Array = amsu_monthly_xtime_Array[*,240:num_months + 239]
  amsu_monthly_xtime_Array = amsu_monthly_xtime_Array/(3600.0)
  amsu_monthly_xtime_Array[5,184:*] = 13.6
  read_land_fraction_144_72,lf
  
  ; set lf to zero for ilat = 0..17.
  ; ocean diurnal cycle values are set to land+ocean values for this region
  ; so this ensures that the land+ocean values are used.
  
  lf[*,0:17] = 0.0
  
  sz = size(params,/dimensions)
  num_params = sz[0]
  case num_params of
    num_sats+2:begin
      for month = 0,num_months-1 do begin
        for satnum = 0,num_sats-1 do begin
          t = amsu_monthly_xtime_Array[satnum,month]
          if finite(t) eq 1 then begin
    
            map_adjust = fltarr(num_lons,num_lats)
            
            ;ocean_adjust = reform(params[8,0,*]) + reform(params[9,0,*])
            for ilat = 0,num_lats-1 do begin
              residual_diurnal_model,t,values
              ocean_adjust = -1.0*(params[8,0,ilat]*values[0] + params[9,0,ilat]*values[1]+params[satnum,0,ilat]) 
              map_adjust[*,ilat] = (1.0-lf[*,ilat])*ocean_adjust
            endfor
            
            for ilat = 0,num_lats-1 do begin
                residual_diurnal_model,t,values
                land_adjust = -1.0*(params[8,1,ilat]*values[0] + params[9,1,ilat]*values[1]+params[satnum,1,ilat])
                map_adjust[*,ilat] = map_adjust[*,ilat]+lf[*,ilat]*land_adjust
            endfor
            
            tb_arr_adj[*,*,month,satnum] = tb_arr_adj[*,*,month,satnum] + map_adjust
          
          endif
        endfor
      endfor
    end
    
    ; this is the case with the seasonal variation in the second harmonic adjustment without the
    ; seasonal constant adjustment
    
    num_sats+6:begin
      for month = 0,num_months-1 do begin
        seas_wts = [sin(2.0*!pi*month/12.0),cos(2.0*!pi*month/12.0)]
        for satnum = 0,num_sats-1 do begin
          t = amsu_monthly_xtime_Array[satnum,month]
          if finite(t) eq 1 then begin
            map_adjust = fltarr(num_lons,num_lats)
            for ilat = 0,num_lats-1 do begin
              residual_diurnal_model,t,values
              sin_amp = params[num_sats,0,ilat]   + params[num_sats+2,0,ilat]*seas_wts[0] + params[num_sats+3,0,ilat]*seas_wts[1]
              cos_amp = params[num_sats+1,0,ilat] + params[num_sats+4,0,ilat]*seas_wts[0] + params[num_sats+5,0,ilat]*seas_wts[1]
              ocean_adjust = -1.0*(sin_amp*values[0] + cos_amp*values[1] + params[satnum,0,ilat])
              map_adjust[*,ilat] = (1.0-lf[*,ilat])*ocean_adjust
            endfor
            
            for ilat = 0,num_lats-1 do begin
              residual_diurnal_model,t,values
              sin_amp = params[num_sats,1,ilat]   + params[num_sats+2,1,ilat]*seas_wts[0] + params[num_sats+3,1,ilat]*seas_wts[1]
              cos_amp = params[num_sats+1,1,ilat] + params[num_sats+4,1,ilat]*seas_wts[0] + params[num_sats+5,1,ilat]*seas_wts[1]
              land_adjust = -1.0*(sin_amp*values[0] + cos_amp*values[1] + params[satnum,1,ilat])
              map_adjust[*,ilat] = map_adjust[*,ilat] + lf[*,ilat]*land_adjust
            endfor
            tb_arr_adj[*,*,month,satnum] = tb_arr_adj[*,*,month,satnum] + map_adjust
            mean_adj = mean(map_adjust[*,8:63],/nan)
          endif
        endfor
      endfor
    end   ;num_sats+6
  endcase

end

  