pro calc_TF_from_grpt_maps_AMSU_v4,tb_arr,tar_arr,num_arr,channel,sats_to_use,target_factor,target_means,offset,months_to_use,num_thres = num_thres,months_to_use_mask = months_to_use_mask,tf_surf=tf_surf
  ;@***h* MLT_V4_RSS/calc_TF_from_grpt_maps_AMSU_v4
  ; NAME
  ;   calc_TF_from_grpt_maps_AMSU_v4
  ;
  ; PURPOSE
  ;   calculates target factors from gridded monthly maps of antenna temperature
  ;
  ; DESCRIPTION
  ;   calculates target factors from gridded monthly maps of antenna temperature.  The best reference for this calculation is
  ;
  ;   Mears, CA, FJ Wentz, 2009,
  ;   Construction of the Remote Sensing Systems V3.2 atmospheric temperature records from the MSU and AMSU microwave sounders
  ;   Journal of Atmospheric and Oceanic Technology, 26, 1040-1056.
  ;
  ;   The operates directly on the adjusted tb maps with the preliminary diurnal and nadir adjustments already applied.
  ;   
  ;   Note that this routine has two important functions:
  ;     First, it calculates the target factors and target means
  ;     Second, it decides which satellite/months to use in the rest of the merge.
  ;   
  ; INPUTS
  ;   tb_arr                fltarr array (144 x 72 x num_months x num_sats) of adjusted antenna temperatures
  ;   tar_arr               fltarr array (144 x 72 x num_months x num_sats) of calibration target temperatures
  ;   num_arr               fltarr array (144 x 72 x num_months x num_sats) of number of observations (used for QC)
  ;   channel               AMSU channel code - should be a 3 character channel string (TLT,TMT, etc)
  ;   sats_to_use           intarr(num_sats)  = 1 is the satnum is to be used in subsequent processing
  ;   num_thres             (optional) threshold for number of observations for a grid cell to pass QC
  ;   months_to_use_mask    (optional) integer mask to force month_num/sat_num use or not.
  ;   tf_surf               (optional) sets surface type to use in calculation.  Set to Land_and_Sea by default.
  ;   
  ; OUTPUTS
  ;   target_factor         values of computed target factors
  ;   target_means          mean values of target temperatures
  ;   offset                offsets applied to each satellite (this is not used, because the offsets are calculated again as a function of
  ;                         latitude later in the merging process)
  ;   months_to_use         intarr(num_sats x num_months) of months to use in subsequent processing.  This is found by combining information
  ;                         from the number of obs, and the months_to_use_mask
  ;
  ; AUTHOR
  ;   Carl Mears, Remote Sensing Systems
  ;
  ; COPYRIGHT
  ;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
  ;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
  ;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
  ;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
  ;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
  ;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
  ;   TO USERS.
  ;
  ; HISTORY
  ;   1/10/2018 Initial Version prepared for NCDC
  ;
  ; USAGE
  ;     calc_TF_from_grpt_maps_AMSU_v4,tb_arr_adj,tar_arr,num_arr,channel_str,sats_to_use,target_factor,target_means,offset,months_to_use, $
  ;                                    num_thres=num_thres,months_to_use_mask=months_to_use_mask,tf_surf=tf_Surf
  ;
  ;@*****



	sigma_est = 0.3
	lat_range=[-50,50]
	sz = size(tb_arr,/dimensions)

	num_months = sz[2]
	num_sats   = sz[3]
	
	;calculate  tropical/temperate global averages for the tb_arr,tar_arr, and the num_arr
	
	mean_tb_arr = fltarr(num_months,num_sats)
	mean_tar_arr = fltarr(num_months,num_sats)
	mean_num_arr = fltarr(num_months,num_sats)
	months_to_use = intarr(num_months,num_sats)

	lat_wts = latitude_wts(72,lat_range)
	if keyword_set(tf_Surf) eq 0 then begin
	   surf_str = 'Land_and_Sea'
	endif else begin
	   surf_str = tf_surf
	endelse
	
	; set maximum material surface altitude for a grid cell to be considered
	; this is set lower for TLT because TLT extends much further into the lower atmosphere
	
	max_alt = 10000
	if channel eq 'TLT' then max_alt = 3000

	for satnum = 0,num_sats-1 do begin
		if sats_to_use[satnum] lt 1 then continue
		for month = 0,num_months-1 do begin
		
			map = tb_arr[*,*,month,satnum]
			ok = where(finite(map) eq 1,ct)		
			mean_tb_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)

			map = tar_arr[*,*,month,satnum]
			mean_tar_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)

      map = num_arr[*,*,month,satnum]
      mean_num_arr[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,surf_str = surf_str,/no_cosine,max_alt=max_alt)

		endfor
	endfor

; construct months_to_use using num_thres
  good = where(mean_num_arr gt num_thres,ct)
  if ct gt 0 then months_to_use[good] = 1
  
; apply additional constraints from mask
  bad = where(months_to_use_mask  eq -1,bad_ct)
  if bad_ct gt 0 then begin
    months_to_use[bad] = 0
  endif
  good_always = where(months_to_use_mask  eq 1,good_ct)
  if good_ct gt 0 then begin
    months_to_use[good_always] = 1
  endif

  ; now use months to use to remove bad data
    
    bad = where(months_to_use eq 0)
    if ct gt 0 then begin
      mean_tb_arr[bad] = !values.f_nan
      mean_tar_arr[bad] = !values.f_nan
    endif

	;remove means from target factors, and save means for passing back up

	target_means = fltarr(num_sats)
	for satnum = 0,num_sats-1 do begin
		tar = mean_tar_arr[*,satnum]
		target_means[satnum] = mean(tar,/nan)
		mean_tar_arr[*,satnum] = mean_tar_arr[*,satnum]-target_means[satnum]
	endfor


  if total(sats_to_use) gt 1.0 then begin  
    ; if more than one satellite is used, then we can calculate the target factors.
    ; if not, we set the target factor to 0
 
  	;construct the regression array
  
  	num_col = num_sats*2
  	X = fltarr(num_col,10000L)
  	Y = fltarr(10000L)
  	s1 = intarr(10000L)
  	s2 = intarr(10000L)
  	m  = intarr(10000L)
  	row_num = 0
    zero_wt = 1.0
    
    for month = 0,num_months-1 do begin
  		tbs = reform(mean_tb_arr[month,*])
  		ok = where(finite(tbs),ct)
  		if ct ge 2 then begin
  			for j = 0,ct-2 do begin
  			  row = fltarr(num_col)
  				sat1 = ok[j]
  				sat2 = ok[j+1]
  				row[sat1] = 1.0
  				row[sat2] = -1.0
  				row[sat1+num_sats] = mean_tar_arr[month,sat1]
  				row[sat2+num_sats] = -mean_tar_arr[month,sat2]
  				X[*,row_num] = row
  				Y[row_num] = tbs[sat1]-tbs[sat2]
  				s1[row_num] = sat1
  				s2[row_num] = sat2
  				m[row_num] = month
  				row_num = row_num+1
  			endfor
  		endif
  	endfor
  
  	X = X[*,0:row_num-1]
  	Y = Y[0:row_num-1]
  	s1 = s1[0:row_num-1]
  	s2 = s2[0:row_num-1]
  	m  = m[0:row_num-1]
  	sigma_in = replicate(sigma_est,row_num)
  
  	lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,trunc_ct = trunc_ct
  
  	; the offsets calculated here are discarded.
  	offset = float(a[0:num_sats-1])
  	target_factor = float(a[num_sats:num_col-1])
  	
    bad = where(sats_to_use eq 0,ct)
    if ct gt 0 then begin
      target_factor[bad] = !values.f_nan
      target_means[bad]  = !values.f_nan
    endif
  endif else begin
    target_factor = replicate(!values.f_nan,num_sats)
    ok = where(sats_to_use gt 0,ct)
    if ct gt 0 then target_factor[ok] = 0.0
  endelse
end
