pro fit_residual_diurnal,tb_arr,lat_range,params

  ;@***h* MLT_V4_RSS/fit_residual_diurnal
  ; NAME
  ;   fit_residual_diurnal
  ;
  ; PURPOSE
  ;   calculates second harmonic adjustment to diurnal cycle to minimize intersatellite differences.
  ;
  ; DESCRIPTION
  ;   calculates second harmonic adjustment to diurnal cycle to minimize intersatellite differences.  For this
  ;   version, the adjustment is not seasonally modulated.  This version is not used in the baseline merging.
  ;   The best reference for this calculation is
  ;
  ;   Mears, C. A., & Wentz, F. J. (2016). Sensitivity of Satellite-Derived Tropospheric Temperature
  ;   Trends to the Diurnal Cycle Adjustment. Journal of Climate, 29, 3629-3646. doi: 10.1175/JCLI-D-15-0744.1
  ;   
  ;   The operates directly on the adjusted tb maps, using the range of latitudes specified in the call. In the
  ;   current merging procedure, this is called repeatedly for each range of latitudes
  ;
  ; INPUTS
  ;   tb_arr          fltarr array (144 x 72 x num_months x num_sats) of antenna temperatures with target temperature
  ;                   adjustments applied.
  ;   lat_range       range of latitudes to include in the calculation
  ;
  ;   amsu_monthly_xtime_Array (this is read in from a file with each call)
  ; OUTPUTS
  ;   params          fltarr array (num_sats+2 x 3) of fit parameters.
  ;                   the first num_sats parameters are offsets that are not used later
  ;                   the last 2 parameters are coefficients of sin(t), cos(t)
  ;
  ; AUTHOR
  ;   Carl Mears, Remote Sensing Systems
  ;
  ; COPYRIGHT
  ;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
  ;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
  ;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
  ;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
  ;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
  ;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
  ;   TO USERS.
  ;
  ; HISTORY
  ;   1/10/2018 Initial Version prepared for NCDC
  ;
  ; USAGE
  ;   fit_residual_diurnal,tb_arr_adj2,lat_range,params
  ;
  ;@*****
   
  sz = size(tb_arr,/dimensions)
  num_lons   = sz[0]
  num_lats   = sz[1]
  num_months = sz[2]
  num_sats   = sz[3]


  mean_ts = fltarr(num_months,num_sats)
  lat_wts = latitude_weights(lat_range=lat_range,num_lats = num_lats)
  params = fltarr(num_sats+2,3)
  surf_list = ['Sea','Land','Land_and_Sea']
  for isurf = 0,2 do begin
    surf_str = surf_list[isurf]
    for satnum = 0,num_sats-1 do begin
      for month = 0,num_months-1 do begin
        map = tb_arr[*,*,month,satnum]
        mean_ts[month,satnum] = compute_global_average_from_144_72_map(map,lat_wts,/no_cosine,surf_str = surf_str)
      endfor
    endfor

    restore,'M:\MSU_AMSU_data\AMSU_Navigation\AMSU_crossing_times.sav'
    amsu_monthly_xtime_Array = amsu_monthly_xtime_Array[*,240:455]
    amsu_monthly_xtime_Array = amsu_monthly_xtime_Array/(3600.0)
    amsu_monthly_xtime_Array[5,184:*] = 13.6

    x = fltarr(num_sats+2,10000)
    y = fltarr(10000)
    delta_T = fltarr(10000)

    k = 0

    for month = 0,num_months-1 do begin
      z = reform(mean_ts[month,*])
      ok = where(finite(z) eq 1,ct)
      if ct gt 1 then begin
        for j = 0,ct-2 do begin
          sat1 = ok[j]
          sat2 = ok[j+1]
          z_diff = z[sat1] - z[sat2]

          row = fltarr(num_sats + 2)
          row[sat1] = 1
          row[sat2] = -1

          t1 = amsu_monthly_xtime_Array[sat1,month]
          t2 = amsu_monthly_xtime_Array[sat2,month]

          residual_diurnal_model,t1,values1
          residual_diurnal_model,t2,values2

          row[num_sats] = values1[0] - values2[0]     ; sin coefficients
          row[num_sats+1] = values1[1] - values2[1]   ; cos coefficients

          x[*,k] = row
          y[k] = z_diff
          delta_t[k] = t1-t2

          k = k+1
        endfor
      endif
    endfor
    if k gt 10 then begin
      x = x[*,0:k-1]
      y = y[0:k-1]
      delta_t=delta_t[0:k]
      sigma_in = replicate(0.3,k)


      lin_regress_SVD,X,Y,sigma_in,A,sigma_out,fit,error,covar=covar,$
        w_thres = w_thres,trunc_ct = trunc_ct

      params[*,isurf] = real_part(a)
    endif else begin
      params[*,isurf] = !values.f_nan
    endelse
  endfor
end