pro read_all_adjusted_AMSU_maps_from_FOV,                     $
            channel,            $
            fov_wts,            $
            node_wts,           $
            adj_flag,           $
            num_sats,           $
            num_months,         $
            tb_arr_adj,         $
            tar_arr=tar_arr,    $
            num_arr=num_arr,    $
            sats_to_use = sats_to_use,             $
            diurnal_model = diurnal_model,         $
            eia_model = eia_model,                 $
            diurnal_factor = diurnal_factor,       $
            do_TLT_adjustment = do_TLT_adjustment, $;, $
            tb_arr_raw = tb_arr_raw

;@***h* MLT_V4_RSS/read_all_adjusted_AMSU_maps_from_FOV
; NAME
;   read_all_adjusted_AMSU_maps_from_FOV
;
; PURPOSE
;   reads in AMSU data for use in the merging system
;
; DESCRIPTION
;   Reads in gridded monthly AMSU data.  The FOV weights, node weight, and level of adjustment is specified during the call
;   The diurnal model and EIA model are also specified.
;   The number of satellites, number of months, antenna temperatures, target temperatures(optional), number of observations per grid cell(optional),
;   satellite months the passed QC(optional), and the unadjusted antenna radiances(optional) all are returned
;   
; INPUTS
;   channel,            AMSU channel number
;   fov_wts,            real(30) array of field of view weights
;   node_wts,           real(2)  array of node weights
;   adj_flag,           integer flag controlling the level of correction applied.
;   diurnal_model       name of diurnal model to apply
;   eia_model           name of eia model applied
;   diurnal factor      factor (usually 1) applied to diurnal climatology
;   do_TLT_adjustment   flag to perform high latitute TLT adjsutment.  Only used for TLT

; OUTPUTS
;   num_sats            number of satellites loaded
;   num_months          number of months loaded
;   tb_arr_adj          fltarr array (144 x 72 x num_months x num_sats) of adjusted antenna temperatures
;   tar_arr             fltarr (144 x 72 x num_months x num_sats) of warm target temperatures
;   num_arr             fltarr (144 x 72 x num_months x num_sats) of number of observations in each grid cell
;   sats_to_use         intarr (num_months x num_sats).  if 1 then the satellite/month should be used in merge.
;   tb_arr_raw          fltarr array (144 x 72 x num_months x num_sats) of unadjusted antenna temperatures
;
; AUTHOR
;   Carl Mears, Remote Sensing Systems
;
; COPYRIGHT
;   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC DOMAIN AND
;   THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE.  THEY ARE FURNISHED "AS IS." THE
;   AUTHORS, THE UNITED STATES GOVERNMENT, ITS INSTRUMENTALITIES, OFFICERS, EMPLOYEES,
;   AND AGENTS MAKE NO WARRANTY, EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE
;   SOFTWARE AND DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
;   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL SUPPORT
;   TO USERS.
;
; HISTORY
;   1/10/2018 Initial Version prepared for NCDC
;
; USAGE
;   read_all_adjusted_AMSU_maps_from_FOV, amsu_channel,       $
;                                              fov_wts,            $
;                                              node_wts,           $
;                                              adj_flag,           $
;                                              num_sats,           $
;                                              num_months,         $
;                                              tb_arr,         $
;                                              tar_arr=tar_arr,    $
;                                              num_arr=num_arr,    $
;                                              sats_to_use = sats_to_use,     $
;                                              diurnal_model = diurnal_model, $
;                                              diurnal_factor = diurnal_factor,          $
;                                              eia_model = eia_model, $;, $
;                                              tb_arr_raw = tb_arr_raw

;
;@*****
                                         
start_year = 1998
end_year = 1998 + (num_months -1)/12

if keyword_set(diurnal_factor) eq 0 then diurnal_factor = 1.0
if keyword_set(do_TLT_adjustment) eq 0 then do_TLT_adjustment = 0
if keyword_set(sats_to_use) eq 0 then sats_to_use = replicate(1.0,7)

if keyword_set(adj_flag) eq 0 then adj_flag = 7
if adj_flag gt 7 then adj_flag = 7
if adj_flag lt 0 then adj_flag = 0

if keyword_set(node_wts) eq 0 then begin
  node_wts = [0.5,0.5]
endif else begin
  if n_elements(node_wts) ne 2 then begin
    log,'Error: node_wts not array(2)
    error = -16
    return
  endif else begin
    tot = total(node_wts)
    if ((finite(tot) eq 0) or (tot eq 0.0)) then begin
      log,'Error: node wts total = NAN or 0'
      error = -16
      return
    endif
    node_wts = float(node_wts)/total(node_wts)
  endelse
endelse

if keyword_set(fov_wts) eq 0 then begin
  log,'Error: fov_wts missing
  error = -32
  return
endif else begin
  if n_elements(fov_wts) ne 30 then begin
    log,'Error: fov_wts not arr(30)
    error = -32
    return
  endif else begin
    tot = total(fov_wts)
    if ((finite(tot) eq 0) or (tot eq 0.0)) then begin
      log,'Error: fov wts total = NAN or 0'
      error = -32
      return
    endif
    fov_wts = float(fov_wts)/total(float(fov_wts))
  endelse
endelse

log,'Info: FOV wts =',fov_wts
log,'Info: Node wts =',node_wts
log,'Info:','Adjustment Flag:',adj_flag

tb_arr_adj = replicate(!values.f_nan,144,72,num_months,num_sats)
tb_arr_raw = replicate(!values.f_nan,144,72,num_months,num_sats)
tar_arr    = replicate(!values.f_nan,144,72,num_months,num_sats)
num_arr    = replicate(0.0,144,72,num_months,num_sats)

for satnum = 10,9+num_sats do begin
  if sats_to_use[satnum-10] eq 0 then continue
  AMSU_cross_track_file = string(channel,satnum,channel,eia_model,format = '("M:\MSU_AMSU_Data\AMSU_Adjustments\cross_track_corrections\AMSU_Channel_",i2.2,"\Sat_",i2.2,"_Channel_",i2.2,"_cross_track_corrections.",a,".txt")')
  close,3
  openr,3,AMSU_cross_track_file
  log,'Info: Cross Track adjustment file = ',AMSU_cross_track_file
  xtrack_residual = fltarr(30)
  readf,3,xtrack_residual,format = '(f9.3)'
  
  close,3
  log,string(sat_na
  me(satnum),format = '("Info: Reading Data for ",a)')

  for year = start_year,end_year do begin
    log,string(sat_name(satnum),year,format = '("Info: Reading Data ",a," for ",i4.4)') ,quiet = 1
    for month = 1,12 do begin
      imonth = 12*(year-1998)+month-1

      log,string(sat_name(satnum),month,year,format = '("Info: Reading Data ",a," for ",i2.2,"/",i4.4)') ,quiet = 1
      read_AMSU_monthly_map_by_fov,satnum,channel,year,month,tb_arr,err
      
      if err eq 0 then begin
        if do_TLT_adjustment gt 0 then begin
            tb_arr[*,0,*,*,60:71] = 0.0
            tb_arr[*,0,*,*,59] = tb_arr[*,0,*,*,59]*0.25
            tb_arr[*,0,*,*,58] = tb_arr[*,0,*,*,58]*0.5
            tb_arr[*,0,*,*,57] = tb_arr[*,0,*,*,57]*0.75
            tb_arr[*,1,*,*,0:11] = 0.0
            tb_arr[*,1,*,*,12] = tb_arr[*,1,*,*,12]*0.25
            tb_arr[*,1,*,*,13] = tb_arr[*,1,*,*,13]*0.5
            tb_arr[*,1,*,*,14] = tb_arr[*,1,*,*,14]*0.75
        endif
      endif
      
      
      
      
      tb_maps     = reform(tb_arr[1,*,*,*,*]/tb_arr[0,*,*,*,*])
      tar_maps    = reform(tb_arr[2,*,*,*,*]/tb_arr[0,*,*,*,*])
      num_maps    = reform(tb_arr[0,*,*,*,*])
      tb_maps_adj = tb_maps
      tb_maps_raw = tb_maps
     
      if err eq 0 then begin
        if (adj_flag and 4) gt 0 then begin
          if diurnal_model ne 'NONE' then begin
            read_amsu_diurnal_adjustment_fov,satnum,channel,year,month,diurnal_model,diurnal_adjustment,diur_error
            tb_maps_adj = tb_maps_adj + diurnal_factor*diurnal_adjustment
            tb_maps_raw = tb_maps_raw ;+ diurnal_factor*diurnal_adjustment
          endif
        endif
        if (adj_flag and 3) eq 1 then begin
          read_amsu_incidence_adjustment_fov,satnum,channel,year,month,eia_model,incidence_adjustment,incidence_error
          tb_maps_adj = tb_maps_adj + incidence_adjustment
          tb_maps_raw = tb_maps_raw + incidence_adjustment
        endif
        if (adj_flag and 2) eq 2 then begin
          read_amsu_nadir_adjustment_fov,satnum,channel,year,month,eia_model,nadir_adjustment,nadir_error
          for fov = 0,29 do begin
            nadir_adjustment[*,fov,*,*] = nadir_adjustment[*,fov,*,*] + xtrack_residual[fov]
          endfor
          tb_maps_adj = tb_maps_adj + nadir_adjustment     
          tb_maps_raw = tb_maps_raw + nadir_adjustment  
        endif
          
        ; combine fovs and nodes
  
        temp = fltarr(2,144,72)
        temp_wt = fltarr(2,144,72)
        for fov = 0,29 do begin
          if (abs(fov_wts[fov]) gt 0.00001) then begin
            map = reform(tb_maps_adj[*,fov,*,*])
            ok = where(finite(map),ct)
            if ct gt 0 then begin
              temp[ok] = temp[ok]+fov_wts[fov]*map[ok]
              temp_wt[ok] = temp_wt[ok] + fov_wts[fov]
            endif
          endif
        endfor
        temp = temp/temp_wt
        asc_map = node_wts[0]*reform(temp[0,*,*])
        dsc_map = node_wts[1]*reform(temp[1,*,*])
        
        tot_map = fltarr(144,72)
        num_map = fltarr(144,72)
        
        ok = where(finite(asc_map) eq 1,ct)
        if ct gt 0 then begin
          tot_map[ok] = tot_map[ok] + asc_map[ok]
          num_map[ok] = num_map[ok] + replicate(1.0,ct)*node_wts[0]
        endif
        
        ok = where(finite(dsc_map) eq 1,ct)
        if ct gt 0 then begin
          tot_map[ok] = tot_map[ok] + dsc_map[ok]
          num_map[ok] = num_map[ok] + replicate(1.0,ct)*node_wts[1]
        endif
             
        tb_map_adj = tot_map/num_map
        
        temp = fltarr(2,144,72)
        temp_wt = fltarr(2,144,72)
        for fov = 0,29 do begin
          if (abs(fov_wts[fov]) gt 0.00001) then begin
            map = reform(tb_maps_raw[*,fov,*,*])
            ok = where(finite(map),ct)
            if ct gt 0 then begin
              temp[ok] = temp[ok]+fov_wts[fov]*map[ok]
              temp_wt[ok] = temp_wt[ok] + fov_wts[fov]
            endif
          endif
        endfor
        temp = temp/temp_wt
        asc_map = node_wts[0]*reform(temp[0,*,*])
        dsc_map = node_wts[1]*reform(temp[1,*,*])

        tot_map = fltarr(144,72)
        num_map = fltarr(144,72)

        ok = where(finite(asc_map) eq 1,ct)
        if ct gt 0 then begin
          tot_map[ok] = tot_map[ok] + asc_map[ok]
          num_map[ok] = num_map[ok] + replicate(1.0,ct)*node_wts[0]
        endif

        ok = where(finite(dsc_map) eq 1,ct)
        if ct gt 0 then begin
          tot_map[ok] = tot_map[ok] + dsc_map[ok]
          num_map[ok] = num_map[ok] + replicate(1.0,ct)*node_wts[1]
        endif

        tb_map_raw = tot_map/num_map
          
        temp = fltarr(2,144,72)
        temp_wt = fltarr(2,144,72)
        for fov = 0,29 do begin
          if (abs(fov_wts[fov]) gt 0.00001) then begin
            map = reform(tar_maps[*,fov,*,*])
            ok = where(finite(map),ct)
            if ct gt 0 then begin
              temp[ok] = temp[ok]+fov_wts[fov]*map[ok]
              temp_wt[ok] = temp_wt[ok] + fov_wts[fov]
            endif
          endif
        endfor
        temp = temp/temp_wt
        asc_map = node_wts[0]*reform(temp[0,*,*])
        dsc_map = node_wts[1]*reform(temp[1,*,*])
        tot_map = fltarr(144,72)
        num_map = fltarr(144,72)
        
        ok = where(finite(asc_map) eq 1,ct)
        if ct gt 0 then begin
          tot_map[ok] = tot_map[ok] + asc_map[ok]
          num_map[ok] = num_map[ok] + replicate(1.0,ct)*node_wts[0]
        endif
        ok = where(finite(dsc_map) eq 1,ct)
        if ct gt 0 then begin
          tot_map[ok] = tot_map[ok] + dsc_map[ok]
          num_map[ok] = num_map[ok] + replicate(1.0,ct)*node_wts[1]
        endif
         
        tar_map = tot_map/num_map       
         
        temp = fltarr(2,144,72)
        temp_wt = fltarr(2,144,72)
        for fov = 0,29 do begin
          if (abs(fov_wts[fov]) gt 0.00001) then begin
            map = reform(num_maps[*,fov,*,*])
            ok = where(finite(map),ct)
            if ct gt 0 then begin
              temp[ok] = temp[ok]+abs(fov_wts[fov])*map[ok]
              temp_wt[ok] = temp_wt[ok] + abs(fov_wts[fov])
            endif
          endif
        endfor
        temp = temp/temp_wt
        num_map = node_wts[0]*reform(temp[0,*,*]) + node_wts[1]*reform(temp[1,*,*])
        error = 0
        if year eq 2015 then begin
          print
        endif
        tb_arr_adj[*,*,imonth,satnum-10]  = tb_map_adj
        tb_arr_raw[*,*,imonth,satnum-10]  = tb_map_raw
        tar_arr[*,*,imonth,satnum-10]     = tar_map
        num_arr[*,*,imonth,satnum-10]     = num_map
      endif
    endfor
  endfor   
endfor                           
end